@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,7 +8,7 @@
 
 #include "base/observer_list.h"
 #include "base/observer_list_threadsafe.h"
-#include "base/threading/sequenced_task_runner_handle.h"
+#include "base/task/sequenced_task_runner.h"
 #include "base/trace_event/base_tracing.h"
 #include "base/tracing_buildflags.h"
 
@@ -27,10 +27,10 @@ class MemoryPressureObserver {
   ~MemoryPressureObserver() = delete;
 
   void AddObserver(MemoryPressureListener* listener, bool sync) {
-    // TODO(crbug.com/1063868): DCHECK instead of silently failing when a
+    // TODO(crbug.com/40123466): DCHECK instead of silently failing when a
     // MemoryPressureListener is created in a non-sequenced context. Tests will
     // need to be adjusted for that to work.
-    if (SequencedTaskRunnerHandle::IsSet()) {
+    if (SequencedTaskRunner::HasCurrentDefault()) {
       async_observers_->AddObserver(listener);
     }
 
@@ -51,8 +51,9 @@ class MemoryPressureObserver {
     async_observers_->Notify(FROM_HERE, &MemoryPressureListener::Notify,
                              memory_pressure_level);
     AutoLock lock(sync_observers_lock_);
-    for (auto& observer : sync_observers_)
+    for (auto& observer : sync_observers_) {
       observer.SyncNotify(memory_pressure_level);
+    }
   }
 
  private:
@@ -131,8 +132,9 @@ void MemoryPressureListener::NotifyMemoryPressure(
         data->set_level(
             trace_event::MemoryPressureLevelToTraceEnum(memory_pressure_level));
       });
-  if (AreNotificationsSuppressed())
+  if (AreNotificationsSuppressed()) {
     return;
+  }
   DoNotifyMemoryPressure(memory_pressure_level);
 }
 
