@@ -84,7 +84,13 @@
 #include "base/memory/safe_ref_traits.h"
 #include "base/sequence_checker.h"
 #include "base/synchronization/atomic_flag.h"
-#include "base/types/pass_key.h"
+
+namespace performance_manager {
+class FrameNodeImpl;
+class PageNodeImpl;
+class ProcessNodeImpl;
+class WorkerNodeImpl;
+}  // namespace performance_manager
 
 namespace base {
 
@@ -92,8 +98,8 @@ namespace sequence_manager::internal {
 class TaskQueueImpl;
 }
 
-template <typename T> class SupportsWeakPtr;
-template <typename T> class WeakPtr;
+template <typename T>
+class WeakPtr;
 
 namespace internal {
 // These classes are part of the WeakPtr implementation.
@@ -170,43 +176,6 @@ class BASE_EXPORT WeakReferenceOwner {
   scoped_refptr<WeakReference::Flag> flag_;
 };
 
-// This class can only be instantiated if the constructor argument inherits
-// from SupportsWeakPtr<T> in exactly one way.
-template <typename T>
-struct ExtractSinglyInheritedBase;
-template <typename T>
-struct ExtractSinglyInheritedBase<SupportsWeakPtr<T>> {
-  using Base = T;
-  explicit ExtractSinglyInheritedBase(SupportsWeakPtr<T>*);
-};
-template <typename T>
-ExtractSinglyInheritedBase(SupportsWeakPtr<T>*)
-  -> ExtractSinglyInheritedBase<SupportsWeakPtr<T>>;
-
-// This class provides a common implementation of common functions that would
-// otherwise get instantiated separately for each distinct instantiation of
-// SupportsWeakPtr<>.
-class SupportsWeakPtrBase {
- public:
-  // A safe static downcast of a WeakPtr<Base> to WeakPtr<Derived>. This
-  // conversion will only compile if Derived singly inherits from
-  // SupportsWeakPtr<Base>. See base::AsWeakPtr() below for a helper function
-  // that makes calling this easier.
-  //
-  // Precondition: t != nullptr
-  template<typename Derived>
-  static WeakPtr<Derived> StaticAsWeakPtr(Derived* t) {
-    static_assert(std::is_base_of_v<internal::SupportsWeakPtrBase, Derived>,
-                  "AsWeakPtr argument must inherit from SupportsWeakPtr");
-    using Base = typename decltype(ExtractSinglyInheritedBase(t))::Base;
-    // Ensure SupportsWeakPtr<Base>::AsWeakPtr() is called even if the subclass
-    // hides or overloads it.
-    WeakPtr<Base> weak = static_cast<SupportsWeakPtr<Base>*>(t)->AsWeakPtr();
-    return WeakPtr<Derived>(weak.CloneWeakReference(),
-                            static_cast<Derived*>(weak.ptr_));
-  }
-};
-
 // Forward declaration from safe_ptr.h.
 template <typename T>
 SafeRef<T> MakeSafeRefFromWeakPtrInternals(internal::WeakReference&& ref,
@@ -214,7 +183,8 @@ SafeRef<T> MakeSafeRefFromWeakPtrInternals(internal::WeakReference&& ref,
 
 }  // namespace internal
 
-template <typename T> class WeakPtrFactory;
+template <typename T>
+class WeakPtrFactory;
 
 // The WeakPtr class holds a weak reference to |T*|.
 //
@@ -293,6 +263,16 @@ class TRIVIAL_ABI WeakPtr {
     ptr_ = nullptr;
   }
 
+  // Do not use this method. Almost all callers should instead use operator
+  // bool().
+  //
+  // There are a few rare cases where the caller intentionally needs to check
+  // validity of a base::WeakPtr on a sequence different from the bound sequence
+  // as an unavoidable performance optimization. This is the only valid use-case
+  // for this method. See
+  // https://docs.google.com/document/d/1IGzq9Nx69GS_2iynGmPWo5sFAD2DcCyBY1zIvZwF7k8
+  // for details.
+  //
   // Returns false if the WeakPtr is confirmed to be invalid. This call is safe
   // to make from any thread, e.g. to optimize away unnecessary work, but
   // RefIsValid() must always be called, on the correct sequence, before
@@ -308,9 +288,8 @@ class TRIVIAL_ABI WeakPtr {
   bool WasInvalidated() const { return ptr_ && !ref_.IsValid(); }
 
  private:
-  friend class internal::SupportsWeakPtrBase;
-  template <typename U> friend class WeakPtr;
-  friend class SupportsWeakPtr<T>;
+  template <typename U>
+  friend class WeakPtr;
   friend class WeakPtrFactory<T>;
   friend class WeakPtrFactory<std::remove_const_t<T>>;
 
@@ -360,6 +339,24 @@ class BASE_EXPORT WeakPtrFactoryBase {
 };
 }  // namespace internal
 
+namespace subtle {
+
+// Restricts access to WeakPtrFactory::BindToCurrentSequence() to authorized
+// callers.
+class BASE_EXPORT BindWeakPtrFactoryPassKey {
+ private:
+  BindWeakPtrFactoryPassKey() = default;
+
+  friend class BindWeakPtrFactoryForTesting;
+  friend class performance_manager::FrameNodeImpl;
+  friend class performance_manager::PageNodeImpl;
+  friend class performance_manager::ProcessNodeImpl;
+  friend class performance_manager::WorkerNodeImpl;
+  friend class sequence_manager::internal::TaskQueueImpl;
+};
+
+}  // namespace subtle
+
 // A class may be composed of a WeakPtrFactory and thereby
 // control how it exposes weak pointers to itself.  This is helpful if you only
 // need weak pointers within the implementation of a class.  This class is also
@@ -422,62 +419,14 @@ class WeakPtrFactory : public internal::WeakPtrFactoryBase {
     return weak_reference_owner_.HasRefs();
   }
 
-  // Rebind the factory to the current sequence. This allows creating a task
-  // queue and associated weak pointers on a different thread from the one they
-  // are used on.
-  void BindToCurrentSequence(
-      PassKey<sequence_manager::internal::TaskQueueImpl>) {
+  // Rebind the factory to the current sequence. This allows creating an object
+  // and associated weak pointers on a different thread from the one they are
+  // used on.
+  void BindToCurrentSequence(subtle::BindWeakPtrFactoryPassKey) {
     weak_reference_owner_.BindToCurrentSequence();
   }
 };
 
-// A class may extend from SupportsWeakPtr to let others take weak pointers to
-// it. This avoids the class itself implementing boilerplate to dispense weak
-// pointers.  However, since SupportsWeakPtr's destructor won't invalidate
-// weak pointers to the class until after the derived class' members have been
-// destroyed, its use can lead to subtle use-after-destroy issues.
-template <class T>
-class SupportsWeakPtr : public internal::SupportsWeakPtrBase {
- public:
-  SupportsWeakPtr() = default;
-
-  SupportsWeakPtr(const SupportsWeakPtr&) = delete;
-  SupportsWeakPtr& operator=(const SupportsWeakPtr&) = delete;
-
-  WeakPtr<T> AsWeakPtr() {
-    return WeakPtr<T>(weak_reference_owner_.GetRef(), static_cast<T*>(this));
-  }
-
- protected:
-  ~SupportsWeakPtr() = default;
-
- private:
-  internal::WeakReferenceOwner weak_reference_owner_;
-};
-
-// Helper function that uses type deduction to safely return a WeakPtr<Derived>
-// when Derived doesn't directly extend SupportsWeakPtr<Derived>, instead it
-// extends a Base that extends SupportsWeakPtr<Base>.
-//
-// EXAMPLE:
-//   class Base : public base::SupportsWeakPtr<Producer> {};
-//   class Derived : public Base {};
-//
-//   Derived derived;
-//   base::WeakPtr<Derived> ptr = base::AsWeakPtr(&derived);
-//
-// Note that the following doesn't work (invalid type conversion) since
-// Derived::AsWeakPtr() is WeakPtr<Base> SupportsWeakPtr<Base>::AsWeakPtr(),
-// and there's no way to safely cast WeakPtr<Base> to WeakPtr<Derived> at
-// the caller.
-//
-//   base::WeakPtr<Derived> ptr = derived.AsWeakPtr();  // Fails.
-
-template <typename Derived>
-WeakPtr<Derived> AsWeakPtr(Derived* t) {
-  return internal::SupportsWeakPtrBase::StaticAsWeakPtr<Derived>(t);
-}
-
 }  // namespace base
 
 #endif  // BASE_MEMORY_WEAK_PTR_H_
