@@ -14,8 +14,10 @@
 #include <math.h>
 
 #include <algorithm>
+#include <atomic>
 #include <memory>
 #include <string>
+#include <string_view>
 #include <utility>
 
 #include "base/compiler_specific.h"
@@ -33,7 +35,6 @@
 #include "base/metrics/statistics_recorder.h"
 #include "base/notreached.h"
 #include "base/pickle.h"
-#include "base/ranges/algorithm.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/synchronization/lock.h"
@@ -63,10 +64,9 @@ bool ReadHistogramArguments(PickleIterator* iter,
 
   // Since these fields may have come from an untrusted renderer, do additional
   // checks above and beyond those in Histogram::Initialize()
-  if (*declared_max <= 0 ||
-      *declared_min <= 0 ||
+  if (*declared_max <= 0 || *declared_min <= 0 ||
       *declared_max < *declared_min ||
-      INT_MAX / sizeof(HistogramBase::Count) <= *bucket_count ||
+      INT_MAX / sizeof(HistogramBase::Count32) <= *bucket_count ||
       *bucket_count < 2) {
     DLOG(ERROR) << "Values error decoding Histogram: " << histogram_name;
     return false;
@@ -84,24 +84,24 @@ bool ValidateRangeChecksum(const HistogramBase& histogram,
   // Normally, |histogram| should have type HISTOGRAM or be inherited from it.
   // However, if it's expired, it will actually be a DUMMY_HISTOGRAM.
   // Skip the checks in that case.
-  if (histogram.GetHistogramType() == DUMMY_HISTOGRAM)
+  if (histogram.GetHistogramType() == DUMMY_HISTOGRAM) {
     return true;
-  const Histogram& casted_histogram =
-      static_cast<const Histogram&>(histogram);
+  }
+  const Histogram& casted_histogram = static_cast<const Histogram&>(histogram);
 
   return casted_histogram.bucket_ranges()->checksum() == range_checksum;
 }
 
 }  // namespace
 
-typedef HistogramBase::Count Count;
-typedef HistogramBase::Sample Sample;
+typedef HistogramBase::Count32 Count32;
+typedef HistogramBase::Sample32 Sample32;
 
 class Histogram::Factory {
  public:
   Factory(std::string_view name,
-          HistogramBase::Sample minimum,
-          HistogramBase::Sample maximum,
+          Sample32 minimum,
+          Sample32 maximum,
           size_t bucket_count,
           int32_t flags)
       : Factory(name, HISTOGRAM, minimum, maximum, bucket_count, flags) {}
@@ -116,8 +116,8 @@ class Histogram::Factory {
  protected:
   Factory(std::string_view name,
           HistogramType histogram_type,
-          HistogramBase::Sample minimum,
-          HistogramBase::Sample maximum,
+          Sample32 minimum,
+          Sample32 maximum,
           size_t bucket_count,
           int32_t flags)
       : name_(name),
@@ -150,8 +150,8 @@ class Histogram::Factory {
   // unnecessary parameters everywhere.
   const std::string_view name_;
   const HistogramType histogram_type_;
-  HistogramBase::Sample minimum_;
-  HistogramBase::Sample maximum_;
+  Sample32 minimum_;
+  Sample32 maximum_;
   size_t bucket_count_;
   int32_t flags_;
 };
@@ -162,8 +162,9 @@ HistogramBase* Histogram::Factory::Build() {
     // constructor. Refactor code to avoid the additional call.
     bool should_record = StatisticsRecorder::ShouldRecordHistogram(
         HashMetricNameAs32Bits(name_));
-    if (!should_record)
+    if (!should_record) {
       return DummyHistogram::GetInstance();
+    }
     // To avoid racy destruction at shutdown, the following will be leaked.
     const BucketRanges* created_ranges = CreateRanges();
 
@@ -232,7 +233,7 @@ HistogramBase* Histogram::Factory::Build() {
     // return would cause Chrome to crash; better to just record it for later
     // analysis.
     UmaHistogramSparse("Histogram.MismatchedConstructionArguments",
-                       static_cast<Sample>(HashMetricName(name_)));
+                       static_cast<Sample32>(HashMetricName(name_)));
     DLOG(ERROR) << "Histogram " << name_
                 << " has mismatched construction arguments";
     return DummyHistogram::GetInstance();
@@ -240,9 +241,34 @@ HistogramBase* Histogram::Factory::Build() {
   return histogram;
 }
 
+HistogramBase* Histogram::FactoryGet(std::string_view name,
+                                     Sample32 minimum,
+                                     Sample32 maximum,
+                                     size_t bucket_count,
+                                     int32_t flags) {
+  return FactoryGetInternal(name, minimum, maximum, bucket_count, flags);
+}
+
+HistogramBase* Histogram::FactoryTimeGet(std::string_view name,
+                                         TimeDelta minimum,
+                                         TimeDelta maximum,
+                                         size_t bucket_count,
+                                         int32_t flags) {
+  return FactoryTimeGetInternal(name, minimum, maximum, bucket_count, flags);
+}
+
+HistogramBase* Histogram::FactoryMicrosecondsTimeGet(std::string_view name,
+                                                     TimeDelta minimum,
+                                                     TimeDelta maximum,
+                                                     size_t bucket_count,
+                                                     int32_t flags) {
+  return FactoryMicrosecondsTimeGetInternal(name, minimum, maximum,
+                                            bucket_count, flags);
+}
+
 HistogramBase* Histogram::FactoryGet(const std::string& name,
-                                     Sample minimum,
-                                     Sample maximum,
+                                     Sample32 minimum,
+                                     Sample32 maximum,
                                      size_t bucket_count,
                                      int32_t flags) {
   return FactoryGetInternal(name, minimum, maximum, bucket_count, flags);
@@ -266,8 +292,8 @@ HistogramBase* Histogram::FactoryMicrosecondsTimeGet(const std::string& name,
 }
 
 HistogramBase* Histogram::FactoryGet(const char* name,
-                                     Sample minimum,
-                                     Sample maximum,
+                                     Sample32 minimum,
+                                     Sample32 maximum,
                                      size_t bucket_count,
                                      int32_t flags) {
   return FactoryGetInternal(name, minimum, maximum, bucket_count, flags);
@@ -311,14 +337,14 @@ std::unique_ptr<HistogramBase> Histogram::PersistentCreate(
 // that bucket onward we do use the exponential growth of buckets.
 //
 // static
-void Histogram::InitializeBucketRanges(Sample minimum,
-                                       Sample maximum,
+void Histogram::InitializeBucketRanges(Sample32 minimum,
+                                       Sample32 maximum,
                                        BucketRanges* ranges) {
   double log_max = log(static_cast<double>(maximum));
   double log_ratio;
   double log_next;
   size_t bucket_index = 1;
-  Sample current = minimum;
+  Sample32 current = minimum;
   ranges->set_range(bucket_index, current);
   size_t bucket_count = ranges->bucket_count();
 
@@ -330,12 +356,13 @@ void Histogram::InitializeBucketRanges(Sample minimum,
     log_ratio = (log_max - log_current) / (bucket_count - bucket_index);
     // See where the next bucket would start.
     log_next = log_current + log_ratio;
-    Sample next;
+    Sample32 next;
     next = static_cast<int>(std::round(exp(log_next)));
-    if (next > current)
+    if (next > current) {
       current = next;
-    else
+    } else {
       ++current;  // Just do a narrow bucket, and keep trying.
+    }
     ranges->set_range(bucket_index, current);
   }
   ranges->set_range(ranges->bucket_count(), HistogramBase::kSampleType_MAX);
@@ -347,29 +374,34 @@ const int Histogram::kCommonRaceBasedCountMismatch = 5;
 
 uint32_t Histogram::FindCorruption(const HistogramSamples& samples) const {
   uint32_t inconsistencies = NO_INCONSISTENCIES;
-  Sample previous_range = -1;  // Bottom range is always 0.
+  Sample32 previous_range = -1;  // Bottom range is always 0.
   for (size_t index = 0; index < bucket_count(); ++index) {
     int new_range = ranges(index);
-    if (previous_range >= new_range)
+    if (previous_range >= new_range) {
       inconsistencies |= BUCKET_ORDER_ERROR;
+    }
     previous_range = new_range;
   }
 
-  if (!bucket_ranges()->HasValidChecksum())
+  if (!bucket_ranges()->HasValidChecksum()) {
     inconsistencies |= RANGE_CHECKSUM_ERROR;
+  }
 
   int64_t delta64 = samples.redundant_count() - samples.TotalCount();
   if (delta64 != 0) {
     int delta = static_cast<int>(delta64);
-    if (delta != delta64)
+    if (delta != delta64) {
       delta = INT_MAX;  // Flag all giant errors as INT_MAX.
+    }
     if (delta > 0) {
-      if (delta > kCommonRaceBasedCountMismatch)
+      if (delta > kCommonRaceBasedCountMismatch) {
         inconsistencies |= COUNT_HIGH_ERROR;
+      }
     } else {
       DCHECK_GT(0, delta);
-      if (-delta > kCommonRaceBasedCountMismatch)
+      if (-delta > kCommonRaceBasedCountMismatch) {
         inconsistencies |= COUNT_LOW_ERROR;
+      }
     }
   }
   return inconsistencies;
@@ -379,21 +411,23 @@ const BucketRanges* Histogram::bucket_ranges() const {
   return unlogged_samples_->bucket_ranges();
 }
 
-Sample Histogram::declared_min() const {
+Sample32 Histogram::declared_min() const {
   const BucketRanges* ranges = bucket_ranges();
-  if (ranges->bucket_count() < 2)
+  if (ranges->bucket_count() < 2) {
     return -1;
+  }
   return ranges->range(1);
 }
 
-Sample Histogram::declared_max() const {
+Sample32 Histogram::declared_max() const {
   const BucketRanges* ranges = bucket_ranges();
-  if (ranges->bucket_count() < 2)
+  if (ranges->bucket_count() < 2) {
     return -1;
+  }
   return ranges->range(ranges->bucket_count() - 1);
 }
 
-Sample Histogram::ranges(size_t i) const {
+Sample32 Histogram::ranges(size_t i) const {
   return bucket_ranges()->range(i);
 }
 
@@ -402,9 +436,9 @@ size_t Histogram::bucket_count() const {
 }
 
 // static
-bool Histogram::InspectConstructionArguments(StringPiece name,
-                                             Sample* minimum,
-                                             Sample* maximum,
+bool Histogram::InspectConstructionArguments(std::string_view name,
+                                             Sample32* minimum,
+                                             Sample32* maximum,
                                              size_t* bucket_count) {
   bool check_okay = true;
 
@@ -417,11 +451,12 @@ bool Histogram::InspectConstructionArguments(StringPiece name,
 
   // Defensive code for backward compatibility.
   if (*minimum < 1) {
-    // TODO(crbug.com/1288842): Temporarily disabled during cleanup.
+    // TODO(crbug.com/40211696): Temporarily disabled during cleanup.
     // DLOG(ERROR) << "Histogram: " << name << " has bad minimum: " << *minimum;
     *minimum = 1;
-    if (*maximum < 1)
+    if (*maximum < 1) {
       *maximum = 1;
+    }
   }
   if (*maximum >= kSampleType_MAX) {
     DLOG(ERROR) << "Histogram: " << name << " has bad maximum: " << *maximum;
@@ -429,7 +464,7 @@ bool Histogram::InspectConstructionArguments(StringPiece name,
   }
   if (*bucket_count > kBucketCount_MAX) {
     UmaHistogramSparse("Histogram.TooManyBuckets.1000",
-                       static_cast<Sample>(HashMetricName(name)));
+                       static_cast<Sample32>(HashMetricName(name)));
 
     // Blink.UseCounter legitimately has more than 1000 entries in its enum.
     if (!StartsWith(name, "Blink.UseCounter")) {
@@ -464,7 +499,7 @@ bool Histogram::InspectConstructionArguments(StringPiece name,
 
   if (!check_okay) {
     UmaHistogramSparse("Histogram.BadConstructionArguments",
-                       static_cast<Sample>(HashMetricName(name)));
+                       static_cast<Sample32>(HashMetricName(name)));
   }
 
   return check_okay;
@@ -478,8 +513,8 @@ HistogramType Histogram::GetHistogramType() const {
   return HISTOGRAM;
 }
 
-bool Histogram::HasConstructionArguments(Sample expected_minimum,
-                                         Sample expected_maximum,
+bool Histogram::HasConstructionArguments(Sample32 expected_minimum,
+                                         Sample32 expected_maximum,
                                          size_t expected_bucket_count) const {
   return (expected_bucket_count == bucket_count() &&
           expected_minimum == declared_min() &&
@@ -494,18 +529,20 @@ void Histogram::AddCount(int value, int count) {
   DCHECK_EQ(0, ranges(0));
   DCHECK_EQ(kSampleType_MAX, ranges(bucket_count()));
 
-  if (value > kSampleType_MAX - 1)
+  if (value > kSampleType_MAX - 1) {
     value = kSampleType_MAX - 1;
-  if (value < 0)
+  }
+  if (value < 0) {
     value = 0;
+  }
   if (count <= 0) {
     NOTREACHED();
-    return;
   }
   unlogged_samples_->Accumulate(value, count);
 
-  if (UNLIKELY(StatisticsRecorder::have_active_callbacks()))
+  if (StatisticsRecorder::have_active_callbacks()) [[unlikely]] {
     FindAndRunCallbacks(value);
+  }
 }
 
 std::unique_ptr<HistogramSamples> Histogram::SnapshotSamples() const {
@@ -562,8 +599,8 @@ std::unique_ptr<HistogramSamples> Histogram::SnapshotFinalDelta() const {
   return SnapshotUnloggedSamples();
 }
 
-void Histogram::AddSamples(const HistogramSamples& samples) {
-  unlogged_samples_->Add(samples);
+bool Histogram::AddSamples(const HistogramSamples& samples) {
+  return unlogged_samples_->Add(samples);
 }
 
 bool Histogram::AddSamplesFromPickle(PickleIterator* iter) {
@@ -611,7 +648,7 @@ Histogram::Histogram(const char* name,
 
 Histogram::~Histogram() = default;
 
-const std::string Histogram::GetAsciiBucketRange(size_t i) const {
+std::string Histogram::GetAsciiBucketRange(size_t i) const {
   return GetSimpleAsciiBucketRange(ranges(i));
 }
 
@@ -635,20 +672,22 @@ HistogramBase* Histogram::DeserializeInfoImpl(PickleIterator* iter) {
   // Find or create the local version of the histogram in this process.
   HistogramBase* histogram = Histogram::FactoryGet(
       histogram_name, declared_min, declared_max, bucket_count, flags);
-  if (!histogram)
+  if (!histogram) {
     return nullptr;
+  }
 
   // The serialized histogram might be corrupted.
-  if (!ValidateRangeChecksum(*histogram, range_checksum))
+  if (!ValidateRangeChecksum(*histogram, range_checksum)) {
     return nullptr;
+  }
 
   return histogram;
 }
 
 // static
 HistogramBase* Histogram::FactoryGetInternal(std::string_view name,
-                                             Sample minimum,
-                                             Sample maximum,
+                                             Sample32 minimum,
+                                             Sample32 maximum,
                                              size_t bucket_count,
                                              int32_t flags) {
   bool valid_arguments =
@@ -668,10 +707,10 @@ HistogramBase* Histogram::FactoryTimeGetInternal(std::string_view name,
                                                  TimeDelta maximum,
                                                  size_t bucket_count,
                                                  int32_t flags) {
-  DCHECK_LT(minimum.InMilliseconds(), std::numeric_limits<Sample>::max());
-  DCHECK_LT(maximum.InMilliseconds(), std::numeric_limits<Sample>::max());
-  return FactoryGetInternal(name, static_cast<Sample>(minimum.InMilliseconds()),
-                            static_cast<Sample>(maximum.InMilliseconds()),
+  DCHECK_LT(minimum.InMilliseconds(), std::numeric_limits<Sample32>::max());
+  DCHECK_LT(maximum.InMilliseconds(), std::numeric_limits<Sample32>::max());
+  return FactoryGetInternal(name, static_cast<Sample32>(minimum.InMilliseconds()),
+                            static_cast<Sample32>(maximum.InMilliseconds()),
                             bucket_count, flags);
 }
 
@@ -682,10 +721,10 @@ HistogramBase* Histogram::FactoryMicrosecondsTimeGetInternal(
     TimeDelta maximum,
     size_t bucket_count,
     int32_t flags) {
-  DCHECK_LT(minimum.InMicroseconds(), std::numeric_limits<Sample>::max());
-  DCHECK_LT(maximum.InMicroseconds(), std::numeric_limits<Sample>::max());
-  return FactoryGetInternal(name, static_cast<Sample>(minimum.InMicroseconds()),
-                            static_cast<Sample>(maximum.InMicroseconds()),
+  DCHECK_LT(minimum.InMicroseconds(), std::numeric_limits<Sample32>::max());
+  DCHECK_LT(maximum.InMicroseconds(), std::numeric_limits<Sample32>::max());
+  return FactoryGetInternal(name, static_cast<Sample32>(minimum.InMicroseconds()),
+                            static_cast<Sample32>(maximum.InMicroseconds()),
                             bucket_count, flags);
 }
 
@@ -719,8 +758,8 @@ Value::Dict Histogram::GetParameters() const {
 class LinearHistogram::Factory : public Histogram::Factory {
  public:
   Factory(std::string_view name,
-          HistogramBase::Sample minimum,
-          HistogramBase::Sample maximum,
+          Sample32 minimum,
+          Sample32 maximum,
           size_t bucket_count,
           int32_t flags,
           const DescriptionPair* descriptions)
@@ -753,8 +792,9 @@ class LinearHistogram::Factory : public Histogram::Factory {
     // Normally, |base_histogram| should have type LINEAR_HISTOGRAM or be
     // inherited from it. However, if it's expired, it will actually be a
     // DUMMY_HISTOGRAM. Skip filling in that case.
-    if (base_histogram->GetHistogramType() == DUMMY_HISTOGRAM)
+    if (base_histogram->GetHistogramType() == DUMMY_HISTOGRAM) {
       return;
+    }
     LinearHistogram* histogram = static_cast<LinearHistogram*>(base_histogram);
     // Set range descriptions.
     if (descriptions_) {
@@ -771,9 +811,25 @@ class LinearHistogram::Factory : public Histogram::Factory {
 
 LinearHistogram::~LinearHistogram() = default;
 
+HistogramBase* LinearHistogram::FactoryGet(std::string_view name,
+                                           Sample32 minimum,
+                                           Sample32 maximum,
+                                           size_t bucket_count,
+                                           int32_t flags) {
+  return FactoryGetInternal(name, minimum, maximum, bucket_count, flags);
+}
+
+HistogramBase* LinearHistogram::FactoryTimeGet(std::string_view name,
+                                               TimeDelta minimum,
+                                               TimeDelta maximum,
+                                               size_t bucket_count,
+                                               int32_t flags) {
+  return FactoryTimeGetInternal(name, minimum, maximum, bucket_count, flags);
+}
+
 HistogramBase* LinearHistogram::FactoryGet(const std::string& name,
-                                           Sample minimum,
-                                           Sample maximum,
+                                           Sample32 minimum,
+                                           Sample32 maximum,
                                            size_t bucket_count,
                                            int32_t flags) {
   return FactoryGetInternal(name, minimum, maximum, bucket_count, flags);
@@ -788,8 +844,8 @@ HistogramBase* LinearHistogram::FactoryTimeGet(const std::string& name,
 }
 
 HistogramBase* LinearHistogram::FactoryGet(const char* name,
-                                           Sample minimum,
-                                           Sample maximum,
+                                           Sample32 minimum,
+                                           Sample32 maximum,
                                            size_t bucket_count,
                                            int32_t flags) {
   return FactoryGetInternal(name, minimum, maximum, bucket_count, flags);
@@ -816,8 +872,8 @@ std::unique_ptr<HistogramBase> LinearHistogram::PersistentCreate(
 
 HistogramBase* LinearHistogram::FactoryGetWithRangeDescription(
     std::string_view name,
-    Sample minimum,
-    Sample maximum,
+    Sample32 minimum,
+    Sample32 maximum,
     size_t bucket_count,
     int32_t flags,
     const DescriptionPair descriptions[]) {
@@ -861,17 +917,18 @@ LinearHistogram::LinearHistogram(
     HistogramSamples::Metadata* logged_meta)
     : Histogram(name, ranges, counts, logged_counts, meta, logged_meta) {}
 
-const std::string LinearHistogram::GetAsciiBucketRange(size_t i) const {
+std::string LinearHistogram::GetAsciiBucketRange(size_t i) const {
   int range = ranges(i);
   BucketDescriptionMap::const_iterator it = bucket_description_.find(range);
-  if (it == bucket_description_.end())
+  if (it == bucket_description_.end()) {
     return Histogram::GetAsciiBucketRange(i);
+  }
   return it->second;
 }
 
 // static
-void LinearHistogram::InitializeBucketRanges(Sample minimum,
-                                             Sample maximum,
+void LinearHistogram::InitializeBucketRanges(Sample32 minimum,
+                                             Sample32 maximum,
                                              BucketRanges* ranges) {
   double min = minimum;
   double max = maximum;
@@ -880,7 +937,7 @@ void LinearHistogram::InitializeBucketRanges(Sample minimum,
   for (size_t i = 1; i < bucket_count; ++i) {
     double linear_range =
         (min * (bucket_count - 1 - i) + max * (i - 1)) / (bucket_count - 2);
-    auto range = static_cast<Sample>(linear_range + 0.5);
+    auto range = static_cast<Sample32>(linear_range + 0.5);
     ranges->set_range(i, range);
   }
   ranges->set_range(ranges->bucket_count(), HistogramBase::kSampleType_MAX);
@@ -889,8 +946,8 @@ void LinearHistogram::InitializeBucketRanges(Sample minimum,
 
 // static
 HistogramBase* LinearHistogram::FactoryGetInternal(std::string_view name,
-                                                   Sample minimum,
-                                                   Sample maximum,
+                                                   Sample32 minimum,
+                                                   Sample32 maximum,
                                                    size_t bucket_count,
                                                    int32_t flags) {
   return FactoryGetWithRangeDescription(name, minimum, maximum, bucket_count,
@@ -903,10 +960,10 @@ HistogramBase* LinearHistogram::FactoryTimeGetInternal(std::string_view name,
                                                        TimeDelta maximum,
                                                        size_t bucket_count,
                                                        int32_t flags) {
-  DCHECK_LT(minimum.InMilliseconds(), std::numeric_limits<Sample>::max());
-  DCHECK_LT(maximum.InMilliseconds(), std::numeric_limits<Sample>::max());
-  return FactoryGetInternal(name, static_cast<Sample>(minimum.InMilliseconds()),
-                            static_cast<Sample>(maximum.InMilliseconds()),
+  DCHECK_LT(minimum.InMilliseconds(), std::numeric_limits<Sample32>::max());
+  DCHECK_LT(maximum.InMilliseconds(), std::numeric_limits<Sample32>::max());
+  return FactoryGetInternal(name, static_cast<Sample32>(minimum.InMilliseconds()),
+                            static_cast<Sample32>(maximum.InMilliseconds()),
                             bucket_count, flags);
 }
 
@@ -926,8 +983,9 @@ HistogramBase* LinearHistogram::DeserializeInfoImpl(PickleIterator* iter) {
 
   HistogramBase* histogram = LinearHistogram::FactoryGet(
       histogram_name, declared_min, declared_max, bucket_count, flags);
-  if (!histogram)
+  if (!histogram) {
     return nullptr;
+  }
 
   if (!ValidateRangeChecksum(*histogram, range_checksum)) {
     // The serialized histogram might be corrupted.
@@ -941,22 +999,9 @@ HistogramBase* LinearHistogram::DeserializeInfoImpl(PickleIterator* iter) {
 // scales input counts.
 //------------------------------------------------------------------------------
 
-ScaledLinearHistogram::ScaledLinearHistogram(const std::string& name,
-                                             Sample minimum,
-                                             Sample maximum,
-                                             size_t bucket_count,
-                                             int32_t scale,
-                                             int32_t flags)
-    : ScaledLinearHistogram(name.data(),
-                            minimum,
-                            maximum,
-                            bucket_count,
-                            scale,
-                            flags) {}
-
-ScaledLinearHistogram::ScaledLinearHistogram(const char* name,
-                                             Sample minimum,
-                                             Sample maximum,
+ScaledLinearHistogram::ScaledLinearHistogram(std::string_view name,
+                                             Sample32 minimum,
+                                             Sample32 maximum,
                                              size_t bucket_count,
                                              int32_t scale,
                                              int32_t flags)
@@ -969,34 +1014,63 @@ ScaledLinearHistogram::ScaledLinearHistogram(const char* name,
   DCHECK(histogram_);
   DCHECK_LT(1, scale);
   DCHECK_EQ(1, minimum);
-  CHECK_EQ(static_cast<Sample>(bucket_count), maximum - minimum + 2)
+  CHECK_EQ(static_cast<Sample32>(bucket_count), maximum - minimum + 2)
       << " ScaledLinearHistogram requires buckets of size 1";
 
   // Normally, |histogram_| should have type LINEAR_HISTOGRAM or be
   // inherited from it. However, if it's expired, it will be DUMMY_HISTOGRAM.
-  if (histogram_->GetHistogramType() == DUMMY_HISTOGRAM)
+  if (histogram_->GetHistogramType() == DUMMY_HISTOGRAM) {
     return;
+  }
 
   DCHECK_EQ(histogram_->GetHistogramType(), LINEAR_HISTOGRAM);
   LinearHistogram* histogram = static_cast<LinearHistogram*>(histogram_);
   remainders_.resize(histogram->bucket_count(), 0);
 }
 
+ScaledLinearHistogram::ScaledLinearHistogram(const std::string& name,
+                                             Sample32 minimum,
+                                             Sample32 maximum,
+                                             size_t bucket_count,
+                                             int32_t scale,
+                                             int32_t flags)
+    : ScaledLinearHistogram(std::string_view(name),
+                            minimum,
+                            maximum,
+                            bucket_count,
+                            scale,
+                            flags) {}
+
+ScaledLinearHistogram::ScaledLinearHistogram(const char* name,
+                                             Sample32 minimum,
+                                             Sample32 maximum,
+                                             size_t bucket_count,
+                                             int32_t scale,
+                                             int32_t flags)
+    : ScaledLinearHistogram(std::string_view(name),
+                            minimum,
+                            maximum,
+                            bucket_count,
+                            scale,
+                            flags) {}
+
 ScaledLinearHistogram::~ScaledLinearHistogram() = default;
 
-void ScaledLinearHistogram::AddScaledCount(Sample value, int64_t count) {
-  if (histogram_->GetHistogramType() == DUMMY_HISTOGRAM)
+void ScaledLinearHistogram::AddScaledCount(Sample32 value, int64_t count) {
+  if (histogram_->GetHistogramType() == DUMMY_HISTOGRAM) {
     return;
-  if (count == 0)
+  }
+  if (count == 0) {
     return;
+  }
   if (count < 0) {
-    NOTREACHED();
+    DUMP_WILL_BE_NOTREACHED();
     return;
   }
 
   DCHECK_EQ(histogram_->GetHistogramType(), LINEAR_HISTOGRAM);
   LinearHistogram* histogram = static_cast<LinearHistogram*>(histogram_);
-  const auto max_value = static_cast<Sample>(histogram->bucket_count() - 1);
+  const auto max_value = static_cast<Sample32>(histogram->bucket_count() - 1);
   value = std::clamp(value, 0, max_value);
 
   int64_t scaled_count = count / scale_;
@@ -1010,7 +1084,7 @@ void ScaledLinearHistogram::AddScaledCount(Sample value, int64_t count) {
         &remainders_[static_cast<size_t>(value)], remainder);
     // If remainder passes 1/2 scale, increment main count (thus rounding up).
     // The remainder is decremented by the full scale, though, which will
-    // cause it to go negative and thus requrire another increase by the full
+    // cause it to go negative and thus require another increase by the full
     // scale amount before another bump of the scaled count.
     if (remainder >= scale_ / 2) {
       scaled_count += 1;
@@ -1020,8 +1094,10 @@ void ScaledLinearHistogram::AddScaledCount(Sample value, int64_t count) {
   }
 
   if (scaled_count > 0) {
-    DCHECK(scaled_count <= std::numeric_limits<int>::max());
-    histogram->AddCount(value, static_cast<int>(scaled_count));
+    int capped_scaled_count = scaled_count > std::numeric_limits<int>::max()
+                                  ? std::numeric_limits<int>::max()
+                                  : static_cast<int>(scaled_count);
+    histogram->AddCount(value, capped_scaled_count);
   }
 }
 
@@ -1050,6 +1126,11 @@ class BooleanHistogram::Factory : public Histogram::Factory {
   }
 };
 
+HistogramBase* BooleanHistogram::FactoryGet(std::string_view name,
+                                            int32_t flags) {
+  return FactoryGetInternal(name, flags);
+}
+
 HistogramBase* BooleanHistogram::FactoryGet(const std::string& name,
                                             int32_t flags) {
   return FactoryGetInternal(name, flags);
@@ -1105,10 +1186,11 @@ HistogramBase* BooleanHistogram::DeserializeInfoImpl(PickleIterator* iter) {
     return nullptr;
   }
 
-  HistogramBase* histogram = BooleanHistogram::FactoryGet(
-      histogram_name, flags);
-  if (!histogram)
+  HistogramBase* histogram =
+      BooleanHistogram::FactoryGet(histogram_name, flags);
+  if (!histogram) {
     return nullptr;
+  }
 
   if (!ValidateRangeChecksum(*histogram, range_checksum)) {
     // The serialized histogram might be corrupted.
@@ -1124,7 +1206,7 @@ HistogramBase* BooleanHistogram::DeserializeInfoImpl(PickleIterator* iter) {
 class CustomHistogram::Factory : public Histogram::Factory {
  public:
   Factory(std::string_view name,
-          const std::vector<Sample>* custom_ranges,
+          const std::vector<Sample32>* custom_ranges,
           int32_t flags)
       : Histogram::Factory(name, CUSTOM_HISTOGRAM, 0, 0, 0, flags) {
     custom_ranges_ = custom_ranges;
@@ -1139,8 +1221,9 @@ class CustomHistogram::Factory : public Histogram::Factory {
     std::vector<int> ranges = *custom_ranges_;
     ranges.push_back(0);  // Ensure we have a zero value.
     ranges.push_back(HistogramBase::kSampleType_MAX);
-    ranges::sort(ranges);
-    ranges.erase(ranges::unique(ranges), ranges.end());
+    std::ranges::sort(ranges);
+    auto removed = std::ranges::unique(ranges);
+    ranges.erase(removed.begin(), removed.end());
 
     BucketRanges* bucket_ranges = new BucketRanges(ranges.size());
     for (size_t i = 0; i < ranges.size(); i++) {
@@ -1156,19 +1239,26 @@ class CustomHistogram::Factory : public Histogram::Factory {
   }
 
  private:
-  raw_ptr<const std::vector<Sample>> custom_ranges_;
+  raw_ptr<const std::vector<Sample32>> custom_ranges_;
 };
 
+HistogramBase* CustomHistogram::FactoryGet(
+    std::string_view name,
+    const std::vector<Sample32>& custom_ranges,
+    int32_t flags) {
+  return FactoryGetInternal(name, custom_ranges, flags);
+}
+
 HistogramBase* CustomHistogram::FactoryGet(
     const std::string& name,
-    const std::vector<Sample>& custom_ranges,
+    const std::vector<Sample32>& custom_ranges,
     int32_t flags) {
   return FactoryGetInternal(name, custom_ranges, flags);
 }
 
 HistogramBase* CustomHistogram::FactoryGet(
     const char* name,
-    const std::vector<Sample>& custom_ranges,
+    const std::vector<Sample32>& custom_ranges,
     int32_t flags) {
   return FactoryGetInternal(name, custom_ranges, flags);
 }
@@ -1189,10 +1279,10 @@ HistogramType CustomHistogram::GetHistogramType() const {
 }
 
 // static
-std::vector<Sample> CustomHistogram::ArrayToCustomEnumRanges(
-    base::span<const Sample> values) {
-  std::vector<Sample> all_values;
-  for (Sample value : values) {
+std::vector<Sample32> CustomHistogram::ArrayToCustomEnumRanges(
+    base::span<const Sample32> values) {
+  std::vector<Sample32> all_values;
+  for (Sample32 value : values) {
     all_values.push_back(value);
 
     // Ensure that a guard bucket is added. If we end up with duplicate
@@ -1217,10 +1307,11 @@ CustomHistogram::CustomHistogram(
 void CustomHistogram::SerializeInfoImpl(Pickle* pickle) const {
   Histogram::SerializeInfoImpl(pickle);
 
-  // Serialize ranges. First and last ranges are alwasy 0 and INT_MAX, so don't
+  // Serialize ranges. First and last ranges are always 0 and INT_MAX, so don't
   // write them.
-  for (size_t i = 1; i < bucket_ranges()->bucket_count(); ++i)
+  for (size_t i = 1; i < bucket_ranges()->bucket_count(); ++i) {
     pickle->WriteInt(bucket_ranges()->range(i));
+  }
 }
 
 // static
@@ -1238,17 +1329,19 @@ HistogramBase* CustomHistogram::DeserializeInfoImpl(PickleIterator* iter) {
   }
 
   // First and last ranges are not serialized.
-  std::vector<Sample> sample_ranges(bucket_count - 1);
+  std::vector<Sample32> sample_ranges(bucket_count - 1);
 
-  for (Sample& sample : sample_ranges) {
-    if (!iter->ReadInt(&sample))
+  for (Sample32& sample : sample_ranges) {
+    if (!iter->ReadInt(&sample)) {
       return nullptr;
+    }
   }
 
-  HistogramBase* histogram = CustomHistogram::FactoryGet(
-      histogram_name, sample_ranges, flags);
-  if (!histogram)
+  HistogramBase* histogram =
+      CustomHistogram::FactoryGet(histogram_name, sample_ranges, flags);
+  if (!histogram) {
     return nullptr;
+  }
 
   if (!ValidateRangeChecksum(*histogram, range_checksum)) {
     // The serialized histogram might be corrupted.
@@ -1260,7 +1353,7 @@ HistogramBase* CustomHistogram::DeserializeInfoImpl(PickleIterator* iter) {
 // static
 HistogramBase* CustomHistogram::FactoryGetInternal(
     std::string_view name,
-    const std::vector<Sample>& custom_ranges,
+    const std::vector<Sample32>& custom_ranges,
     int32_t flags) {
   CHECK(ValidateCustomRanges(custom_ranges));
 
@@ -1269,15 +1362,62 @@ HistogramBase* CustomHistogram::FactoryGetInternal(
 
 // static
 bool CustomHistogram::ValidateCustomRanges(
-    const std::vector<Sample>& custom_ranges) {
+    const std::vector<Sample32>& custom_ranges) {
   bool has_valid_range = false;
-  for (Sample sample : custom_ranges) {
-    if (sample < 0 || sample > HistogramBase::kSampleType_MAX - 1)
+  for (Sample32 sample : custom_ranges) {
+    if (sample < 0 || sample > HistogramBase::kSampleType_MAX - 1) {
       return false;
-    if (sample != 0)
+    }
+    if (sample != 0) {
       has_valid_range = true;
+    }
   }
   return has_valid_range;
 }
 
+namespace internal {
+
+namespace {
+// The pointer to the atomic const-pointer also needs to be atomic as some
+// threads might already be alive when it's set. It requires acquire-release
+// semantics to ensure the memory it points to is seen in its initialized state.
+constinit std::atomic<const std::atomic<TimeTicks>*> g_last_foreground_time_ref;
+}  // namespace
+
+void SetSharedLastForegroundTimeForMetrics(
+    const std::atomic<TimeTicks>* last_foreground_time_ref) {
+  g_last_foreground_time_ref.store(last_foreground_time_ref,
+                                   std::memory_order_release);
+}
+
+const std::atomic<TimeTicks>*
+GetSharedLastForegroundTimeForMetricsForTesting() {
+  return g_last_foreground_time_ref.load(std::memory_order_acquire);
+}
+
+bool OverlapsBestEffortRange(TimeTicks sample_time, TimeDelta sample_interval) {
+  // std::memory_order_acquire semantics required as documented above to make
+  // sure the memory pointed to by the stored `const std::atomic<TimeTicks>*`
+  // is initialized from this thread's POV.
+  auto last_foreground_time_ref =
+      g_last_foreground_time_ref.load(std::memory_order_acquire);
+  if (!last_foreground_time_ref) {
+    return false;
+  }
+
+  // std::memory_order_relaxed is sufficient here as we care about the stored
+  // TimeTicks value but don't assume the state of any other shared memory based
+  // on the result.
+  auto last_foreground_time =
+      last_foreground_time_ref->load(std::memory_order_relaxed);
+  // `last_foreground_time.is_null()` indicates we're currently under
+  // best-effort priority and thus assume overlap. Otherwise we compare whether
+  // the range of interest is fully contained within the last time this process
+  // was running at a foreground priority.
+  return last_foreground_time.is_null() ||
+         (sample_time - sample_interval) < last_foreground_time;
+}
+
+}  // namespace internal
+
 }  // namespace base
