@@ -4,103 +4,33 @@
 
 #include "base/metrics/persistent_sample_map.h"
 
-#include "base/atomicops.h"
+#include <stddef.h>
+#include <stdint.h>
+
+#include <atomic>
+#include <type_traits>
+
 #include "base/check_op.h"
 #include "base/containers/contains.h"
-#include "base/debug/crash_logging.h"
+#include "base/metrics/histogram_base.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/metrics/histogram_samples.h"
 #include "base/metrics/persistent_histogram_allocator.h"
+#include "base/metrics/sample_map_iterator.h"
 #include "base/notreached.h"
-#include "base/numerics/safe_conversions.h"
+#include "build/buildflag.h"
+
+#if !BUILDFLAG(IS_NACL)
+#include "base/debug/crash_logging.h"
+#endif
 
 namespace base {
 
-typedef HistogramBase::Count Count;
-typedef HistogramBase::Sample Sample;
+using Count32 = HistogramBase::Count32;
+using Sample32 = HistogramBase::Sample32;
 
 namespace {
 
-// An iterator for going through a PersistentSampleMap. The logic here is
-// identical to that of the iterator for SampleMap but with different data
-// structures. Changes here likely need to be duplicated there.
-template <typename T, typename I>
-class IteratorTemplate : public SampleCountIterator {
- public:
-  explicit IteratorTemplate(T& sample_counts)
-      : iter_(sample_counts.begin()), end_(sample_counts.end()) {
-    SkipEmptyBuckets();
-  }
-
-  ~IteratorTemplate() override;
-
-  // SampleCountIterator:
-  bool Done() const override { return iter_ == end_; }
-  void Next() override {
-    DCHECK(!Done());
-    ++iter_;
-    SkipEmptyBuckets();
-  }
-  void Get(HistogramBase::Sample* min,
-           int64_t* max,
-           HistogramBase::Count* count) override;
-
- private:
-  void SkipEmptyBuckets() {
-    while (!Done() && subtle::NoBarrier_Load(iter_->second) == 0) {
-      ++iter_;
-    }
-  }
-
-  I iter_;
-  const I end_;
-};
-
-typedef std::map<HistogramBase::Sample, HistogramBase::Count*> SampleToCountMap;
-typedef IteratorTemplate<const SampleToCountMap,
-                         SampleToCountMap::const_iterator>
-    PersistentSampleMapIterator;
-
-template <>
-PersistentSampleMapIterator::~IteratorTemplate() = default;
-
-// Get() for an iterator of a PersistentSampleMap.
-template <>
-void PersistentSampleMapIterator::Get(Sample* min, int64_t* max, Count* count) {
-  DCHECK(!Done());
-  *min = iter_->first;
-  *max = strict_cast<int64_t>(iter_->first) + 1;
-  // We have to do the following atomically, because even if the caller is using
-  // a lock, a separate process (that is not aware of this lock) may
-  // concurrently modify the value (note that iter_->second is a pointer to a
-  // sample count, which may live in shared memory).
-  *count = subtle::NoBarrier_Load(iter_->second);
-}
-
-typedef IteratorTemplate<SampleToCountMap, SampleToCountMap::iterator>
-    ExtractingPersistentSampleMapIterator;
-
-template <>
-ExtractingPersistentSampleMapIterator::~IteratorTemplate() {
-  // Ensure that the user has consumed all the samples in order to ensure no
-  // samples are lost.
-  DCHECK(Done());
-}
-
-// Get() for an extracting iterator of a PersistentSampleMap.
-template <>
-void ExtractingPersistentSampleMapIterator::Get(Sample* min,
-                                                int64_t* max,
-                                                Count* count) {
-  DCHECK(!Done());
-  *min = iter_->first;
-  *max = strict_cast<int64_t>(iter_->first) + 1;
-  // We have to do the following atomically, because even if the caller is using
-  // a lock, a separate process (that is not aware of this lock) may
-  // concurrently modify the value (note that iter_->second is a pointer to a
-  // sample count, which may live in shared memory).
-  *count = subtle::NoBarrier_AtomicExchange(iter_->second, 0);
-}
-
 // This structure holds an entry for a PersistentSampleMap within a persistent
 // memory allocator. The "id" must be unique across all maps held by an
 // allocator or they will get attached to the wrong sample map.
@@ -111,9 +41,19 @@ struct SampleRecord {
   // Expected size for 32/64-bit check.
   static constexpr size_t kExpectedInstanceSize = 16;
 
-  uint64_t id;   // Unique identifier of owner.
-  Sample value;  // The value for which this record holds a count.
-  Count count;   // The count associated with the above value.
+  uint64_t id;                 // Unique identifier of owner.
+  Sample32 value;              // The value for which this record holds a count.
+  std::atomic<Count32> count;  // The count associated with the above value.
+
+  // `count` may operate inter-process and so must be lock-free.
+  static_assert(std::atomic<Count32>::is_always_lock_free);
+
+  // For backwards compatibility, `std::atomic<Count>` and `Count` must have
+  // the same memory layouts. If this ever changes, make sure to increment
+  // `kPersistentTypeId` above.
+  static_assert(std::is_standard_layout_v<std::atomic<Count32>>);
+  static_assert(sizeof(std::atomic<Count32>) == sizeof(Count32));
+  static_assert(alignof(std::atomic<Count32>) == alignof(Count32));
 };
 
 }  // namespace
@@ -126,49 +66,44 @@ PersistentSampleMap::PersistentSampleMap(
 
 PersistentSampleMap::~PersistentSampleMap() = default;
 
-void PersistentSampleMap::Accumulate(Sample value, Count count) {
+void PersistentSampleMap::Accumulate(Sample32 value, Count32 count) {
   // We have to do the following atomically, because even if the caller is using
   // a lock, a separate process (that is not aware of this lock) may
   // concurrently modify the value.
-  subtle::NoBarrier_AtomicIncrement(GetOrCreateSampleCountStorage(value),
-                                    count);
-  IncreaseSumAndCount(strict_cast<int64_t>(count) * value, count);
+  GetOrCreateSampleCountStorage(value)->fetch_add(count,
+                                                  std::memory_order_relaxed);
+  IncreaseSumAndCount(int64_t{count} * value, count);
 }
 
-Count PersistentSampleMap::GetCount(Sample value) const {
-  // Have to override "const" to make sure all samples have been loaded before
-  // being able to know what value to return.
-  Count* count_pointer =
-      const_cast<PersistentSampleMap*>(this)->GetSampleCountStorage(value);
-  return count_pointer ? subtle::NoBarrier_Load(count_pointer) : 0;
+Count32 PersistentSampleMap::GetCount(Sample32 value) const {
+  const std::atomic<Count32>* const count_pointer = GetSampleCountStorage(value);
+  return count_pointer ? count_pointer->load(std::memory_order_relaxed) : 0;
 }
 
-Count PersistentSampleMap::TotalCount() const {
-  // Have to override "const" in order to make sure all samples have been
-  // loaded before trying to iterate over the map.
-  const_cast<PersistentSampleMap*>(this)->ImportSamples(
-      /*until_value=*/std::nullopt);
-
-  Count count = 0;
+Count32 PersistentSampleMap::TotalCount() const {
+  // Make sure all samples have been loaded before trying to iterate over the
+  // map.
+  ImportSamples();
+  Count32 count = 0;
   for (const auto& entry : sample_counts_) {
-    count += subtle::NoBarrier_Load(entry.second);
+    count += entry.second->load(std::memory_order_relaxed);
   }
   return count;
 }
 
 std::unique_ptr<SampleCountIterator> PersistentSampleMap::Iterator() const {
-  // Have to override "const" in order to make sure all samples have been
-  // loaded before trying to iterate over the map.
-  const_cast<PersistentSampleMap*>(this)->ImportSamples(
-      /*until_value=*/std::nullopt);
-  return std::make_unique<PersistentSampleMapIterator>(sample_counts_);
+  // Make sure all samples have been loaded before trying to iterate over the
+  // map.
+  ImportSamples();
+  return std::make_unique<SampleMapIterator<SampleToCountMap, false>>(
+      sample_counts_);
 }
 
 std::unique_ptr<SampleCountIterator> PersistentSampleMap::ExtractingIterator() {
   // Make sure all samples have been loaded before trying to iterate over the
   // map.
-  ImportSamples(/*until_value=*/std::nullopt);
-  return std::make_unique<ExtractingPersistentSampleMapIterator>(
+  ImportSamples();
+  return std::make_unique<SampleMapIterator<SampleToCountMap, true>>(
       sample_counts_);
 }
 
@@ -182,7 +117,7 @@ PersistentMemoryAllocator::Reference
 PersistentSampleMap::GetNextPersistentRecord(
     PersistentMemoryAllocator::Iterator& iterator,
     uint64_t* sample_map_id,
-    Sample* value) {
+    Sample32* value) {
   const SampleRecord* record = iterator.GetNextOfObject<SampleRecord>();
   if (!record) {
     return 0;
@@ -198,67 +133,67 @@ PersistentMemoryAllocator::Reference
 PersistentSampleMap::CreatePersistentRecord(
     PersistentMemoryAllocator* allocator,
     uint64_t sample_map_id,
-    Sample value) {
+    Sample32 value) {
   SampleRecord* record = allocator->New<SampleRecord>();
-  if (!record) {
-    if (!allocator->IsFull()) {
-#if !BUILDFLAG(IS_NACL)
-      // TODO(crbug.com/40064026): Remove these. They are used to investigate
-      // unexpected failures.
-      SCOPED_CRASH_KEY_BOOL("PersistentSampleMap", "corrupted",
-                            allocator->IsCorrupt());
-#endif  // !BUILDFLAG(IS_NACL)
-      DUMP_WILL_BE_NOTREACHED() << "corrupt=" << allocator->IsCorrupt();
-    }
-    return 0;
+  if (record) {
+    record->id = sample_map_id;
+    record->value = value;
+    record->count = 0;
+    PersistentMemoryAllocator::Reference ref =
+        allocator->GetAsReference(record);
+    allocator->MakeIterable(ref);
+    return ref;
   }
 
-  record->id = sample_map_id;
-  record->value = value;
-  record->count = 0;
-
-  PersistentMemoryAllocator::Reference ref = allocator->GetAsReference(record);
-  allocator->MakeIterable(ref);
-  return ref;
+  if (!allocator->IsFull()) {
+    const bool corrupt = allocator->IsCorrupt();
+#if !BUILDFLAG(IS_NACL)
+    // TODO(crbug.com/40064026): Remove.
+    SCOPED_CRASH_KEY_BOOL("PersistentSampleMap", "corrupted", corrupt);
+#endif  // !BUILDFLAG(IS_NACL)
+    DUMP_WILL_BE_NOTREACHED() << "corrupt=" << corrupt;
+  }
+  return 0;
 }
 
 bool PersistentSampleMap::AddSubtractImpl(SampleCountIterator* iter,
                                           Operator op) {
-  Sample min;
+  Sample32 min;
   int64_t max;
-  Count count;
+  Count32 count;
   for (; !iter->Done(); iter->Next()) {
     iter->Get(&min, &max, &count);
-    if (count == 0)
+    if (count == 0) {
       continue;
-    if (strict_cast<int64_t>(min) + 1 != max)
+    }
+    if (int64_t{min} + 1 != max) {
       return false;  // SparseHistogram only supports bucket with size 1.
+    }
 
     // We have to do the following atomically, because even if the caller is
     // using a lock, a separate process (that is not aware of this lock) may
     // concurrently modify the value.
-    subtle::Barrier_AtomicIncrement(
-        GetOrCreateSampleCountStorage(min),
-        (op == HistogramSamples::ADD) ? count : -count);
+    GetOrCreateSampleCountStorage(min)->fetch_add(
+        (op == HistogramSamples::ADD) ? count : -count,
+        std::memory_order_seq_cst);
   }
   return true;
 }
 
-Count* PersistentSampleMap::GetSampleCountStorage(Sample value) {
+std::atomic<Count32>* PersistentSampleMap::GetSampleCountStorage(
+    Sample32 value) const {
   // If |value| is already in the map, just return that.
-  auto it = sample_counts_.find(value);
-  if (it != sample_counts_.end())
-    return it->second;
-
-  // Import any new samples from persistent memory looking for the value.
-  return ImportSamples(/*until_value=*/value);
+  const auto it = sample_counts_.find(value);
+  return (it == sample_counts_.end()) ? ImportSamples(value) : it->second.get();
 }
 
-Count* PersistentSampleMap::GetOrCreateSampleCountStorage(Sample value) {
+std::atomic<Count32>* PersistentSampleMap::GetOrCreateSampleCountStorage(
+    Sample32 value) {
   // Get any existing count storage.
-  Count* count_pointer = GetSampleCountStorage(value);
-  if (count_pointer)
+  std::atomic<Count32>* count_pointer = GetSampleCountStorage(value);
+  if (count_pointer) {
     return count_pointer;
+  }
 
   // Create a new record in persistent memory for the value. |records_| will
   // have been initialized by the GetSampleCountStorage() call above.
@@ -269,7 +204,7 @@ Count* PersistentSampleMap::GetOrCreateSampleCountStorage(Sample value) {
     // full or corrupt. Instead, allocate the counter from the heap. This
     // sample will not be persistent, will not be shared, and will leak...
     // but it's better than crashing.
-    count_pointer = new Count(0);
+    count_pointer = new std::atomic<Count32>(0);
     sample_counts_[value] = count_pointer;
     return count_pointer;
   }
@@ -283,12 +218,12 @@ Count* PersistentSampleMap::GetOrCreateSampleCountStorage(Sample value) {
   // Thread-safety within a process where multiple threads use the same
   // histogram object is delegated to the controlling histogram object which,
   // for sparse histograms, is a lock object.
-  count_pointer = ImportSamples(/*until_value=*/value);
+  count_pointer = ImportSamples(value);
   DCHECK(count_pointer);
   return count_pointer;
 }
 
-PersistentSampleMapRecords* PersistentSampleMap::GetRecords() {
+PersistentSampleMapRecords* PersistentSampleMap::GetRecords() const {
   // The |records_| pointer is lazily fetched from the |allocator_| only on
   // first use. Sometimes duplicate histograms are created by race conditions
   // and if both were to grab the records object, there would be a conflict.
@@ -300,7 +235,8 @@ PersistentSampleMapRecords* PersistentSampleMap::GetRecords() {
   return records_.get();
 }
 
-Count* PersistentSampleMap::ImportSamples(std::optional<Sample> until_value) {
+std::atomic<Count32>* PersistentSampleMap::ImportSamples(
+    std::optional<Sample32> until_value) const {
   std::vector<PersistentMemoryAllocator::Reference> refs;
   PersistentSampleMapRecords* records = GetRecords();
   while (!(refs = records->GetNextRecords(until_value)).empty()) {
@@ -308,7 +244,7 @@ Count* PersistentSampleMap::ImportSamples(std::optional<Sample> until_value) {
     // map. Iterate through them all and store them internally. Note that if
     // |until_value| was found, it will be the last element in |refs|.
     for (auto ref : refs) {
-      SampleRecord* record = records->GetAsObject<SampleRecord>(ref);
+      SampleRecord* const record = records->GetAsObject<SampleRecord>(ref);
       if (!record) {
         continue;
       }
@@ -316,10 +252,8 @@ Count* PersistentSampleMap::ImportSamples(std::optional<Sample> until_value) {
       DCHECK_EQ(id(), record->id);
 
       // Check if the record's value is already known.
-      if (!Contains(sample_counts_, record->value)) {
-        // No: Add it to map of known values.
-        sample_counts_[record->value] = &record->count;
-      } else {
+      const auto ret = sample_counts_.insert({record->value, &record->count});
+      if (!ret.second) {
         // Yes: Ignore it; it's a duplicate caused by a race condition -- see
         // code & comment in GetOrCreateSampleCountStorage() for details.
         // Check that nothing ever operated on the duplicate record.
