@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,38 +6,39 @@
 
 #include <crt_externs.h>
 #include <mach/mach.h>
+#include <os/availability.h>
 #include <spawn.h>
 #include <string.h>
-#include <sys/syscall.h>
 #include <sys/wait.h>
 
+#include "base/apple/mach_port_rendezvous.h"
 #include "base/command_line.h"
 #include "base/files/scoped_file.h"
 #include "base/logging.h"
-#include "base/mac/availability.h"
+#include "base/memory/raw_ptr.h"
 #include "base/posix/eintr_wrapper.h"
 #include "base/process/environment_internal.h"
 #include "base/threading/scoped_blocking_call.h"
 #include "base/threading/thread_restrictions.h"
-#include "base/trace_event/trace_event.h"
+#include "base/trace_event/base_tracing.h"
 
-// Changes the current thread's directory to a path or directory file
-// descriptor. libpthread only exposes a syscall wrapper starting in
-// macOS 10.12, but the system call dates back to macOS 10.5. On older OSes,
-// the syscall is issued directly.
 extern "C" {
-int pthread_chdir_np(const char*) API_AVAILABLE(macosx(10.12));
-int pthread_fchdir_np(int fd) API_AVAILABLE(macosx(10.12));
-}
+// Changes the current thread's directory to a path or directory file
+// descriptor.
+int pthread_chdir_np(const char* dir);
+
+int pthread_fchdir_np(int fd);
+
+int responsibility_spawnattrs_setdisclaim(posix_spawnattr_t attrs,
+                                          int disclaim);
+}  // extern "C"
 
 namespace base {
 
-// Friend and derived class of ScopedAllowBaseSyncPrimitives which allows
-// GetAppOutputInternal() to join a process. GetAppOutputInternal() can't itself
-// be a friend of ScopedAllowBaseSyncPrimitives because it is in the anonymous
-// namespace.
-class GetAppOutputScopedAllowBaseSyncPrimitives
-    : public base::ScopedAllowBaseSyncPrimitives {};
+void CheckPThreadStackMinIsSafe() {
+  static_assert(__builtin_constant_p(PTHREAD_STACK_MIN),
+                "Always constant on mac");
+}
 
 namespace {
 
@@ -68,6 +69,9 @@ class PosixSpawnFileActions {
     DPSXCHECK(posix_spawn_file_actions_init(&file_actions_));
   }
 
+  PosixSpawnFileActions(const PosixSpawnFileActions&) = delete;
+  PosixSpawnFileActions& operator=(const PosixSpawnFileActions&) = delete;
+
   ~PosixSpawnFileActions() {
     DPSXCHECK(posix_spawn_file_actions_destroy(&file_actions_));
   }
@@ -86,43 +90,43 @@ class PosixSpawnFileActions {
     DPSXCHECK(posix_spawn_file_actions_addinherit_np(&file_actions_, filedes));
   }
 
+#if BUILDFLAG(IS_MAC)
+  void Chdir(const char* path) {
+    DPSXCHECK(posix_spawn_file_actions_addchdir_np(&file_actions_, path));
+  }
+#endif
+
   const posix_spawn_file_actions_t* get() const { return &file_actions_; }
 
  private:
   posix_spawn_file_actions_t file_actions_;
-
-  DISALLOW_COPY_AND_ASSIGN(PosixSpawnFileActions);
 };
 
+#if !BUILDFLAG(IS_MAC)
 int ChangeCurrentThreadDirectory(const char* path) {
-  if (__builtin_available(macOS 10.12, *)) {
-    return pthread_chdir_np(path);
-  } else {
-    return syscall(SYS___pthread_chdir, path);
-  }
+  return pthread_chdir_np(path);
 }
 
 // The recommended way to unset a per-thread cwd is to set a new value to an
 // invalid file descriptor, per libpthread-218.1.3/private/private.h.
 int ResetCurrentThreadDirectory() {
-  if (__builtin_available(macOS 10.12, *)) {
-    return pthread_fchdir_np(-1);
-  } else {
-    return syscall(SYS___pthread_fchdir, -1);
-  }
+  return pthread_fchdir_np(-1);
 }
+#endif
 
 struct GetAppOutputOptions {
   // Whether to pipe stderr to stdout in |output|.
   bool include_stderr = false;
   // Caller-supplied string poiter for the output.
-  std::string* output = nullptr;
+  raw_ptr<std::string> output = nullptr;
   // Result exit code of Process::Wait().
   int exit_code = 0;
 };
 
 bool GetAppOutputInternal(const std::vector<std::string>& argv,
                           GetAppOutputOptions* gao_options) {
+  TRACE_EVENT0("base", "GetAppOutput");
+
   ScopedFD read_fd, write_fd;
   {
     int pipefds[2];
@@ -141,6 +145,9 @@ bool GetAppOutputInternal(const std::vector<std::string>& argv,
   }
 
   Process process = LaunchProcess(argv, launch_options);
+  if (!process.IsValid()) {
+    return false;
+  }
 
   // Close the parent process' write descriptor, so that EOF is generated in
   // read loop below.
@@ -159,13 +166,15 @@ bool GetAppOutputInternal(const std::vector<std::string>& argv,
     read_this_pass = HANDLE_EINTR(
         read(read_fd.get(), &(*output)[total_bytes_read], kBufferSize));
     if (read_this_pass >= 0) {
-      total_bytes_read += read_this_pass;
+      total_bytes_read += static_cast<size_t>(read_this_pass);
       output->resize(total_bytes_read);
     }
   } while (read_this_pass > 0);
 
-  // Reap the child process.
-  GetAppOutputScopedAllowBaseSyncPrimitives allow_wait;
+  // It is okay to allow this process to wait on the launched process as a
+  // process launched with GetAppOutput*() shouldn't wait back on the process
+  // that launched it.
+  internal::GetAppOutputScopedAllowBaseSyncPrimitives allow_wait;
   if (!process.WaitForExit(&gao_options->exit_code)) {
     return false;
   }
@@ -225,58 +234,99 @@ Process LaunchProcess(const std::vector<std::string>& argv,
     file_actions.Inherit(STDERR_FILENO);
   }
 
+#if BUILDFLAG(IS_MAC)
+  if (options.disclaim_responsibility) {
+    DPSXCHECK(responsibility_spawnattrs_setdisclaim(attr.get(), 1));
+  }
+
+  EnvironmentMap new_environment_map = options.environment;
+  MachPortRendezvousServerMac::AddFeatureStateToEnvironment(
+      new_environment_map);
+#else
+  const EnvironmentMap& new_environment_map = options.environment;
+#endif
+
   std::vector<char*> argv_cstr;
   argv_cstr.reserve(argv.size() + 1);
-  for (const auto& arg : argv)
+  for (const auto& arg : argv) {
     argv_cstr.push_back(const_cast<char*>(arg.c_str()));
+  }
   argv_cstr.push_back(nullptr);
 
-  std::unique_ptr<char*[]> owned_environ;
+  base::HeapArray<char*> owned_environ;
   char* empty_environ = nullptr;
   char** new_environ =
       options.clear_environment ? &empty_environ : *_NSGetEnviron();
-  if (!options.environment.empty()) {
+  if (!new_environment_map.empty()) {
     owned_environ =
-        internal::AlterEnvironment(new_environ, options.environment);
-    new_environ = owned_environ.get();
+        internal::AlterEnvironment(new_environ, new_environment_map);
+    new_environ = owned_environ.data();
   }
 
   const char* executable_path = !options.real_path.empty()
                                     ? options.real_path.value().c_str()
                                     : argv_cstr[0];
 
-  // If the new program has specified its PWD, change the thread-specific
-  // working directory. The new process will inherit it during posix_spawnp().
+  if (options.enable_cpu_security_mitigations) {
+    DPSXCHECK(posix_spawnattr_set_csm_np(attr.get(), POSIX_SPAWN_NP_CSM_ALL));
+  }
+
   if (!options.current_directory.empty()) {
-    int rv =
-        ChangeCurrentThreadDirectory(options.current_directory.value().c_str());
+    const char* chdir_str = options.current_directory.value().c_str();
+#if BUILDFLAG(IS_MAC)
+    file_actions.Chdir(chdir_str);
+#else
+    // If the chdir posix_spawn_file_actions extension is not available,
+    // change the thread-specific working directory. The new process will
+    // inherit it during posix_spawnp().
+    int rv = ChangeCurrentThreadDirectory(chdir_str);
     if (rv != 0) {
       DPLOG(ERROR) << "pthread_chdir_np";
       return Process();
     }
+#endif
   }
 
   int rv;
   pid_t pid;
   {
-    // If |options.mach_ports_for_rendezvous| is specified : the server's lock
-    // must be held for the duration of posix_spawnp() so that new child's PID
-    // can be recorded with the set of ports.
-    const bool has_mac_ports_for_rendezvous =
+    const bool has_mach_ports_for_rendezvous =
         !options.mach_ports_for_rendezvous.empty();
-    AutoLockMaybe rendezvous_lock(
-        has_mac_ports_for_rendezvous
-            ? &MachPortRendezvousServer::GetInstance()->GetLock()
-            : nullptr);
+#if BUILDFLAG(IS_IOS)
+    // This code is only used for the iOS simulator to launch tests. We do not
+    // support setting MachPorts on launch. You should look at
+    // content::ChildProcessLauncherHelper (for iOS) if you are trying to spawn
+    // a non-test and need ports.
+    CHECK(!has_mach_ports_for_rendezvous);
+#else
+    // If `options.mach_ports_for_rendezvous` or `options.process_requirement`
+    // is specified : the server's lock must be held for the duration of
+    // posix_spawnp() so that new child's PID can be recorded with the set of
+    // ports or process requirement.
+    bool needs_rendezvous_lock =
+        has_mach_ports_for_rendezvous || options.process_requirement;
 
+    AutoLockMaybe rendezvous_lock(
+        needs_rendezvous_lock
+            ? &MachPortRendezvousServerMac::GetInstance()->GetLock()
+            : nullptr);
+#endif
     // Use posix_spawnp as some callers expect to have PATH consulted.
     rv = posix_spawnp(&pid, executable_path, file_actions.get(), attr.get(),
                       &argv_cstr[0], new_environ);
 
-    if (has_mac_ports_for_rendezvous) {
-      auto* rendezvous = MachPortRendezvousServer::GetInstance();
+#if !BUILDFLAG(IS_IOS)
+    if (needs_rendezvous_lock) {
       if (rv == 0) {
-        rendezvous->RegisterPortsForPid(pid, options.mach_ports_for_rendezvous);
+        MachPortRendezvousServerMac::GetInstance()->GetLock().AssertAcquired();
+        if (has_mach_ports_for_rendezvous) {
+          MachPortRendezvousServerMac::GetInstance()->RegisterPortsForPid(
+              pid, options.mach_ports_for_rendezvous);
+        }
+        if (options.process_requirement) {
+          MachPortRendezvousServerMac::GetInstance()
+              ->SetProcessRequirementForPid(pid, *options.process_requirement);
+        }
       } else {
         // Because |options| is const-ref, the collection has to be copied here.
         // The caller expects to relinquish ownership of any strong rights if
@@ -288,12 +338,15 @@ Process LaunchProcess(const std::vector<std::string>& argv,
         }
       }
     }
+#endif
   }
 
+#if !BUILDFLAG(IS_MAC)
   // Restore the thread's working directory if it was changed.
   if (!options.current_directory.empty()) {
     ResetCurrentThreadDirectory();
   }
+#endif
 
   if (rv != 0) {
     DLOG(ERROR) << "posix_spawnp(" << executable_path << "): -" << rv << " "
