@@ -1,4 +1,4 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,30 +7,30 @@
 #include <stdint.h>
 
 #include <memory>
+#include <string_view>
 #include <utility>
 #include <vector>
 
 #include "base/allocator/buildflags.h"
+#include "base/at_exit.h"
 #include "base/base_switches.h"
-#include "base/bind.h"
-#include "base/callback.h"
 #include "base/command_line.h"
-#include "base/macros.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/run_loop.h"
-#include "base/single_thread_task_runner.h"
 #include "base/synchronization/waitable_event.h"
-#include "base/task/post_task.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/task/sequenced_task_runner.h"
+#include "base/task/single_thread_task_runner.h"
+#include "base/task/thread_pool.h"
+#include "base/test/task_environment.h"
 #include "base/test/test_io_thread.h"
 #include "base/threading/platform_thread.h"
-#include "base/threading/sequenced_task_runner_handle.h"
 #include "base/threading/thread.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "base/trace_event/memory_dump_manager_test_utils.h"
 #include "base/trace_event/memory_dump_provider.h"
 #include "base/trace_event/memory_dump_request_args.h"
 #include "base/trace_event/memory_dump_scheduler.h"
-#include "base/trace_event/memory_infra_background_whitelist.h"
+#include "base/trace_event/memory_infra_background_allowlist.h"
 #include "base/trace_event/process_memory_dump.h"
 #include "build/build_config.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -42,23 +42,30 @@ using testing::Between;
 using testing::Invoke;
 using testing::Return;
 
-namespace base {
-namespace trace_event {
+namespace base::trace_event {
 
 // GTest matchers for MemoryDumpRequestArgs arguments.
 MATCHER(IsDetailedDump, "") {
-  return arg.level_of_detail == MemoryDumpLevelOfDetail::DETAILED;
+  return arg.level_of_detail == MemoryDumpLevelOfDetail::kDetailed;
 }
 
 MATCHER(IsLightDump, "") {
-  return arg.level_of_detail == MemoryDumpLevelOfDetail::LIGHT;
+  return arg.level_of_detail == MemoryDumpLevelOfDetail::kLight;
+}
+
+MATCHER(IsDeterministicDump, "") {
+  return arg.determinism == MemoryDumpDeterminism::kForceGc;
+}
+
+MATCHER(IsNotDeterministicDump, "") {
+  return arg.determinism == MemoryDumpDeterminism::kNone;
 }
 
 namespace {
 
-const char* kMDPName = "TestDumpProvider";
-const char* kWhitelistedMDPName = "WhitelistedTestDumpProvider";
-const char* const kTestMDPWhitelist[] = {kWhitelistedMDPName, nullptr};
+constexpr char kMDPName[] = "TestDumpProvider";
+constexpr char kAllowlistedMDPName[] = "AllowlistedTestDumpProvider";
+constexpr std::string_view kTestMDPAllowlist[] = {kAllowlistedMDPName};
 
 void RegisterDumpProvider(
     MemoryDumpProvider* mdp,
@@ -108,7 +115,7 @@ class MockMemoryDumpProvider : public MemoryDumpProvider {
   MOCK_METHOD2(OnMemoryDump,
                bool(const MemoryDumpArgs& args, ProcessMemoryDump* pmd));
 
-  MockMemoryDumpProvider() : enable_mock_destructor(false) {
+  MockMemoryDumpProvider() {
     ON_CALL(*this, OnMemoryDump(_, _))
         .WillByDefault(
             Invoke([](const MemoryDumpArgs&, ProcessMemoryDump* pmd) -> bool {
@@ -116,11 +123,12 @@ class MockMemoryDumpProvider : public MemoryDumpProvider {
             }));
   }
   ~MockMemoryDumpProvider() override {
-    if (enable_mock_destructor)
+    if (enable_mock_destructor) {
       Destructor();
+    }
   }
 
-  bool enable_mock_destructor;
+  bool enable_mock_destructor = false;
 };
 
 class TestSequencedTaskRunner : public SequencedTaskRunner {
@@ -134,7 +142,6 @@ class TestSequencedTaskRunner : public SequencedTaskRunner {
                                   OnceClosure task,
                                   TimeDelta delay) override {
     NOTREACHED();
-    return false;
   }
 
   bool PostDelayedTask(const Location& from_here,
@@ -155,7 +162,7 @@ class TestSequencedTaskRunner : public SequencedTaskRunner {
   ~TestSequencedTaskRunner() override = default;
 
   const scoped_refptr<SequencedTaskRunner> task_runner_ =
-      CreateSequencedTaskRunner({ThreadPool()});
+      ThreadPool::CreateSequencedTaskRunner({});
   bool enabled_ = true;
   unsigned num_of_post_tasks_ = 0;
 };
@@ -164,23 +171,26 @@ class TestSequencedTaskRunner : public SequencedTaskRunner {
 
 class MemoryDumpManagerTest : public testing::Test {
  public:
-  MemoryDumpManagerTest(bool is_coordinator = false)
+  explicit MemoryDumpManagerTest(bool is_coordinator = false)
       : is_coordinator_(is_coordinator) {}
 
+  MemoryDumpManagerTest(const MemoryDumpManagerTest&) = delete;
+  MemoryDumpManagerTest& operator=(const MemoryDumpManagerTest&) = delete;
+
   void SetUp() override {
     // Bring up and initialize MemoryDumpManager while single-threaded (before
-    // instantiating ScopedTaskEnvironment) to avoid data races if worker
+    // instantiating TaskEnvironment) to avoid data races if worker
     // threads use tracing globals early.
     mdm_ = MemoryDumpManager::CreateInstanceForTesting();
     ASSERT_EQ(mdm_.get(), MemoryDumpManager::GetInstance());
 
     InitializeMemoryDumpManagerForInProcessTesting(is_coordinator_);
 
-    scoped_task_environment_ = std::make_unique<test::ScopedTaskEnvironment>();
+    task_environment_ = std::make_unique<test::TaskEnvironment>();
   }
 
   void TearDown() override {
-    scoped_task_environment_.reset();
+    task_environment_.reset();
 
     // Tear down the MemoryDumpManager while single-threaded to mirror logic in
     // SetUp().
@@ -193,12 +203,14 @@ class MemoryDumpManagerTest : public testing::Test {
   // memory dump is complete. Returns:
   // - return value: the |success| from the CreateProcessDump() callback.
   bool RequestProcessDumpAndWait(MemoryDumpType dump_type,
-                                 MemoryDumpLevelOfDetail level_of_detail) {
+                                 MemoryDumpLevelOfDetail level_of_detail,
+                                 MemoryDumpDeterminism determinism) {
     RunLoop run_loop;
     bool success = false;
     static uint64_t test_guid = 1;
     test_guid++;
-    MemoryDumpRequestArgs request_args{test_guid, dump_type, level_of_detail};
+    MemoryDumpRequestArgs request_args{test_guid, dump_type, level_of_detail,
+                                       determinism};
 
     // The signature of the callback delivered by MemoryDumpManager is:
     // void ProcessMemoryDumpCallback(
@@ -215,7 +227,7 @@ class MemoryDumpManagerTest : public testing::Test {
            std::unique_ptr<ProcessMemoryDump> pmd) {
           *curried_success = success;
           EXPECT_EQ(curried_expected_guid, dump_guid);
-          ThreadTaskRunnerHandle::Get()->PostTask(
+          SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
               FROM_HERE, std::move(curried_quit_closure));
         },
         Unretained(&success), run_loop.QuitClosure(), test_guid);
@@ -229,7 +241,7 @@ class MemoryDumpManagerTest : public testing::Test {
     mdm_->SetupForTracing(TraceConfig::MemoryDumpConfig());
   }
 
-  void EnableForTracingWithTraceConfig(const std::string trace_config_string) {
+  void EnableForTracingWithTraceConfig(const std::string& trace_config_string) {
     TraceConfig trace_config(trace_config_string);
     mdm_->SetupForTracing(trace_config.memory_dump_config());
   }
@@ -247,36 +259,37 @@ class MemoryDumpManagerTest : public testing::Test {
   // To tear down the singleton instance after each test.
   ShadowingAtExitManager at_exit_manager_;
 
-  std::unique_ptr<test::ScopedTaskEnvironment> scoped_task_environment_;
+  std::unique_ptr<test::TaskEnvironment> task_environment_;
 
   // Whether the test MemoryDumpManager should be initialized as the
   // coordinator.
   const bool is_coordinator_;
-
-  DISALLOW_COPY_AND_ASSIGN(MemoryDumpManagerTest);
 };
 
 class MemoryDumpManagerTestAsCoordinator : public MemoryDumpManagerTest {
  public:
   MemoryDumpManagerTestAsCoordinator() : MemoryDumpManagerTest(true) {}
 
- private:
-  DISALLOW_COPY_AND_ASSIGN(MemoryDumpManagerTestAsCoordinator);
+  MemoryDumpManagerTestAsCoordinator(
+      const MemoryDumpManagerTestAsCoordinator&) = delete;
+  MemoryDumpManagerTestAsCoordinator& operator=(
+      const MemoryDumpManagerTestAsCoordinator&) = delete;
 };
 
 // Basic sanity checks. Registers a memory dump provider and checks that it is
 // called.
 TEST_F(MemoryDumpManagerTest, SingleDumper) {
   MockMemoryDumpProvider mdp;
-  RegisterDumpProvider(&mdp, ThreadTaskRunnerHandle::Get());
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
 
   // Now repeat enabling the memory category and check that the dumper is
   // invoked this time.
   EnableForTracing();
   EXPECT_CALL(mdp, OnMemoryDump(_, _)).Times(3);
   for (int i = 0; i < 3; ++i) {
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
   }
   DisableTracing();
 
@@ -287,8 +300,9 @@ TEST_F(MemoryDumpManagerTest, SingleDumper) {
   EnableForTracing();
   EXPECT_CALL(mdp, OnMemoryDump(_, _)).Times(0);
   for (int i = 0; i < 3; ++i) {
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
   }
   DisableTracing();
 }
@@ -298,21 +312,49 @@ TEST_F(MemoryDumpManagerTest, SingleDumper) {
 TEST_F(MemoryDumpManagerTest, CheckMemoryDumpArgs) {
   MockMemoryDumpProvider mdp;
 
-  RegisterDumpProvider(&mdp, ThreadTaskRunnerHandle::Get());
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
   EnableForTracing();
   EXPECT_CALL(mdp, OnMemoryDump(IsDetailedDump(), _));
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
   DisableTracing();
   mdm_->UnregisterDumpProvider(&mdp);
 
   // Check that requesting dumps with low level of detail actually propagates to
   // OnMemoryDump() call on dump providers.
-  RegisterDumpProvider(&mdp, ThreadTaskRunnerHandle::Get());
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
   EnableForTracing();
   EXPECT_CALL(mdp, OnMemoryDump(IsLightDump(), _));
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::LIGHT));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kLight,
+                                        MemoryDumpDeterminism::kNone));
+  DisableTracing();
+  mdm_->UnregisterDumpProvider(&mdp);
+}
+
+// Checks that requesting deterministic dumps actually propagates
+// the deterministic option properly to OnMemoryDump() call on dump providers.
+TEST_F(MemoryDumpManagerTest, CheckMemoryDumpArgsDeterministic) {
+  MockMemoryDumpProvider mdp;
+
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
+  EnableForTracing();
+  EXPECT_CALL(mdp, OnMemoryDump(IsDeterministicDump(), _));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kForceGc));
+  DisableTracing();
+  mdm_->UnregisterDumpProvider(&mdp);
+
+  // Check that requesting dumps with deterministic option set to false
+  // actually propagates to OnMemoryDump() call on dump providers.
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
+  EnableForTracing();
+  EXPECT_CALL(mdp, OnMemoryDump(IsNotDeterministicDump(), _));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kLight,
+                                        MemoryDumpDeterminism::kNone));
   DisableTracing();
   mdm_->UnregisterDumpProvider(&mdp);
 }
@@ -323,12 +365,13 @@ TEST_F(MemoryDumpManagerTest, MultipleDumpers) {
   MockMemoryDumpProvider mdp2;
 
   // Enable only mdp1.
-  RegisterDumpProvider(&mdp1, ThreadTaskRunnerHandle::Get());
+  RegisterDumpProvider(&mdp1, SingleThreadTaskRunner::GetCurrentDefault());
   EnableForTracing();
   EXPECT_CALL(mdp1, OnMemoryDump(_, _));
   EXPECT_CALL(mdp2, OnMemoryDump(_, _)).Times(0);
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
   DisableTracing();
 
   // Invert: enable mdp2 and disable mdp1.
@@ -337,8 +380,9 @@ TEST_F(MemoryDumpManagerTest, MultipleDumpers) {
   EnableForTracing();
   EXPECT_CALL(mdp1, OnMemoryDump(_, _)).Times(0);
   EXPECT_CALL(mdp2, OnMemoryDump(_, _));
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
   DisableTracing();
 
   // Enable both mdp1 and mdp2.
@@ -346,15 +390,16 @@ TEST_F(MemoryDumpManagerTest, MultipleDumpers) {
   EnableForTracing();
   EXPECT_CALL(mdp1, OnMemoryDump(_, _));
   EXPECT_CALL(mdp2, OnMemoryDump(_, _));
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
   DisableTracing();
 }
 
 // Checks that the dump provider invocations depend only on the current
 // registration state and not on previous registrations and dumps.
 // Flaky on iOS, see crbug.com/706874
-#if defined(OS_IOS)
+#if BUILDFLAG(IS_IOS)
 #define MAYBE_RegistrationConsistency DISABLED_RegistrationConsistency
 #else
 #define MAYBE_RegistrationConsistency RegistrationConsistency
@@ -362,13 +407,14 @@ TEST_F(MemoryDumpManagerTest, MultipleDumpers) {
 TEST_F(MemoryDumpManagerTest, MAYBE_RegistrationConsistency) {
   MockMemoryDumpProvider mdp;
 
-  RegisterDumpProvider(&mdp, ThreadTaskRunnerHandle::Get());
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
 
   {
     EXPECT_CALL(mdp, OnMemoryDump(_, _));
     EnableForTracing();
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
     DisableTracing();
   }
 
@@ -377,31 +423,34 @@ TEST_F(MemoryDumpManagerTest, MAYBE_RegistrationConsistency) {
   {
     EXPECT_CALL(mdp, OnMemoryDump(_, _)).Times(0);
     EnableForTracing();
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
     DisableTracing();
   }
 
-  RegisterDumpProvider(&mdp, ThreadTaskRunnerHandle::Get());
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
   mdm_->UnregisterDumpProvider(&mdp);
 
   {
     EXPECT_CALL(mdp, OnMemoryDump(_, _)).Times(0);
     EnableForTracing();
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
     DisableTracing();
   }
 
-  RegisterDumpProvider(&mdp, ThreadTaskRunnerHandle::Get());
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
   mdm_->UnregisterDumpProvider(&mdp);
-  RegisterDumpProvider(&mdp, ThreadTaskRunnerHandle::Get());
+  RegisterDumpProvider(&mdp, SingleThreadTaskRunner::GetCurrentDefault());
 
   {
     EXPECT_CALL(mdp, OnMemoryDump(_, _));
     EnableForTracing();
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
     DisableTracing();
   }
 }
@@ -438,8 +487,9 @@ TEST_F(MemoryDumpManagerTest, RespectTaskRunnerAffinity) {
   EnableForTracing();
 
   while (!threads.empty()) {
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
 
     // Unregister a MDP and destroy one thread at each iteration to check the
     // live unregistration logic. The unregistration needs to happen on the same
@@ -484,14 +534,16 @@ TEST_F(MemoryDumpManagerTest, PostTaskForSequencedTaskRunner) {
   EnableForTracing();
 
   task_runner1->set_enabled(false);
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
   EXPECT_EQ(1u, task_runner1->no_of_post_tasks());
   EXPECT_EQ(1u, task_runner2->no_of_post_tasks());
 
   task_runner1->set_enabled(true);
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
   EXPECT_EQ(2u, task_runner1->no_of_post_tasks());
   EXPECT_EQ(2u, task_runner2->no_of_post_tasks());
   DisableTracing();
@@ -521,8 +573,9 @@ TEST_F(MemoryDumpManagerTest, DisableFailingDumpers) {
 
   const int kNumDumps = 2 * GetMaxConsecutiveFailuresCount();
   for (int i = 0; i < kNumDumps; i++) {
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
   }
 
   DisableTracing();
@@ -552,8 +605,9 @@ TEST_F(MemoryDumpManagerTest, RegisterDumperWhileDumping) {
   EXPECT_CALL(mdp2, OnMemoryDump(_, _)).Times(Between(2, 3));
 
   for (int i = 0; i < 4; i++) {
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
   }
 
   DisableTracing();
@@ -564,8 +618,10 @@ TEST_F(MemoryDumpManagerTest, UnregisterDumperWhileDumping) {
   MockMemoryDumpProvider mdp1;
   MockMemoryDumpProvider mdp2;
 
-  RegisterDumpProvider(&mdp1, ThreadTaskRunnerHandle::Get(), kDefaultOptions);
-  RegisterDumpProvider(&mdp2, ThreadTaskRunnerHandle::Get(), kDefaultOptions);
+  RegisterDumpProvider(&mdp1, SingleThreadTaskRunner::GetCurrentDefault(),
+                       kDefaultOptions);
+  RegisterDumpProvider(&mdp2, SingleThreadTaskRunner::GetCurrentDefault(),
+                       kDefaultOptions);
   EnableForTracing();
 
   EXPECT_CALL(mdp1, OnMemoryDump(_, _))
@@ -583,8 +639,9 @@ TEST_F(MemoryDumpManagerTest, UnregisterDumperWhileDumping) {
   EXPECT_CALL(mdp2, OnMemoryDump(_, _)).Times(Between(1, 2));
 
   for (int i = 0; i < 4; i++) {
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
   }
 
   DisableTracing();
@@ -631,8 +688,9 @@ TEST_F(MemoryDumpManagerTest, UnregisterDumperFromThreadWhileDumping) {
   }
 
   EnableForTracing();
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
   ASSERT_EQ(1, on_memory_dump_call_count);
 
   DisableTracing();
@@ -660,7 +718,7 @@ TEST_F(MemoryDumpManagerTest, TearDownThreadWhileDumping) {
     TestIOThread* other_thread = threads[other_idx].get();
     // TestIOThread isn't thread-safe and must be stopped on the |main_runner|.
     scoped_refptr<SequencedTaskRunner> main_runner =
-        SequencedTaskRunnerHandle::Get();
+        SequencedTaskRunner::GetCurrentDefault();
     auto on_dump = [other_thread, main_runner, &on_memory_dump_call_count](
                        const MemoryDumpArgs& args, ProcessMemoryDump* pmd) {
       PostTaskAndWait(
@@ -678,8 +736,9 @@ TEST_F(MemoryDumpManagerTest, TearDownThreadWhileDumping) {
   }
 
   EnableForTracing();
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
   ASSERT_EQ(1, on_memory_dump_call_count);
 
   DisableTracing();
@@ -691,23 +750,25 @@ TEST_F(MemoryDumpManagerTest, TriggerDumpWithoutTracing) {
   MockMemoryDumpProvider mdp;
   RegisterDumpProvider(&mdp, nullptr);
   EXPECT_CALL(mdp, OnMemoryDump(_, _));
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
 }
 
-TEST_F(MemoryDumpManagerTest, BackgroundWhitelisting) {
-  SetDumpProviderWhitelistForTesting(kTestMDPWhitelist);
+TEST_F(MemoryDumpManagerTest, BackgroundAllowlisting) {
+  SetDumpProviderAllowlistForTesting(kTestMDPAllowlist);
 
   // Standard provider with default options (create dump for current process).
   MockMemoryDumpProvider backgroundMdp;
   RegisterDumpProvider(&backgroundMdp, nullptr, kDefaultOptions,
-                       kWhitelistedMDPName);
+                       kAllowlistedMDPName);
 
   EnableForTracing();
 
   EXPECT_CALL(backgroundMdp, OnMemoryDump(_, _)).Times(1);
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::SUMMARY_ONLY,
-                                        MemoryDumpLevelOfDetail::BACKGROUND));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kSummaryOnly,
+                                        MemoryDumpLevelOfDetail::kBackground,
+                                        MemoryDumpDeterminism::kNone));
   DisableTracing();
 }
 
@@ -720,8 +781,9 @@ TEST_F(MemoryDumpManagerTest, UnregisterAndDeleteDumpProviderSoon) {
   for (int i = 0; i < kNumProviders; ++i) {
     std::unique_ptr<MockMemoryDumpProvider> mdp(new MockMemoryDumpProvider);
     mdp->enable_mock_destructor = true;
-    EXPECT_CALL(*mdp, Destructor())
-        .WillOnce(Invoke([&dtor_count]() { dtor_count++; }));
+    EXPECT_CALL(*mdp, Destructor()).WillOnce(Invoke([&dtor_count] {
+      dtor_count++;
+    }));
     RegisterDumpProvider(mdp.get(), nullptr, kDefaultOptions);
     mdps.push_back(std::move(mdp));
   }
@@ -745,8 +807,8 @@ TEST_F(MemoryDumpManagerTest, UnregisterAndDeleteDumpProviderSoonDuringDump) {
   RegisterDumpProvider(mdp.get(), nullptr, kDefaultOptions);
 
   base::PlatformThreadRef thread_ref;
-  auto self_unregister_from_another_thread = [&mdp, &thread_ref](
-      const MemoryDumpArgs&, ProcessMemoryDump*) -> bool {
+  auto self_unregister_from_another_thread =
+      [&mdp, &thread_ref](const MemoryDumpArgs&, ProcessMemoryDump*) -> bool {
     thread_ref = PlatformThread::CurrentRef();
     TestIOThread thread_for_unregistration(TestIOThread::kAutoStart);
     PostTaskAndWait(
@@ -760,16 +822,15 @@ TEST_F(MemoryDumpManagerTest, UnregisterAndDeleteDumpProviderSoonDuringDump) {
   EXPECT_CALL(*mdp, OnMemoryDump(_, _))
       .Times(1)
       .WillOnce(Invoke(self_unregister_from_another_thread));
-  EXPECT_CALL(*mdp, Destructor())
-      .Times(1)
-      .WillOnce(Invoke([&thread_ref]() {
-        EXPECT_EQ(thread_ref, PlatformThread::CurrentRef());
-      }));
+  EXPECT_CALL(*mdp, Destructor()).Times(1).WillOnce(Invoke([&thread_ref] {
+    EXPECT_EQ(thread_ref, PlatformThread::CurrentRef());
+  }));
 
   EnableForTracing();
   for (int i = 0; i < 2; ++i) {
-    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                          MemoryDumpLevelOfDetail::DETAILED));
+    EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                          MemoryDumpLevelOfDetail::kDetailed,
+                                          MemoryDumpDeterminism::kNone));
   }
   DisableTracing();
 }
@@ -780,8 +841,8 @@ TEST_F(MemoryDumpManagerTest, UnregisterAndDeleteDumpProviderSoonDuringDump) {
 // NoStackOverflowWithTooManyMDPs test.
 class SimpleMockMemoryDumpProvider : public MemoryDumpProvider {
  public:
-  SimpleMockMemoryDumpProvider(int expected_num_dump_calls)
-      : expected_num_dump_calls_(expected_num_dump_calls), num_dump_calls_(0) {}
+  explicit SimpleMockMemoryDumpProvider(int expected_num_dump_calls)
+      : expected_num_dump_calls_(expected_num_dump_calls) {}
 
   ~SimpleMockMemoryDumpProvider() override {
     EXPECT_EQ(expected_num_dump_calls_, num_dump_calls_);
@@ -795,11 +856,11 @@ class SimpleMockMemoryDumpProvider : public MemoryDumpProvider {
 
  private:
   int expected_num_dump_calls_;
-  int num_dump_calls_;
+  int num_dump_calls_ = 0;
 };
 
 TEST_F(MemoryDumpManagerTest, NoStackOverflowWithTooManyMDPs) {
-  SetDumpProviderWhitelistForTesting(kTestMDPWhitelist);
+  SetDumpProviderAllowlistForTesting(kTestMDPAllowlist);
 
   int kMDPCount = 1000;
   std::vector<std::unique_ptr<SimpleMockMemoryDumpProvider>> mdps;
@@ -810,24 +871,26 @@ TEST_F(MemoryDumpManagerTest, NoStackOverflowWithTooManyMDPs) {
   for (int i = 0; i < kMDPCount; ++i) {
     mdps.push_back(std::make_unique<SimpleMockMemoryDumpProvider>(3));
     RegisterDumpProvider(mdps.back().get(), nullptr, kDefaultOptions,
-                         kWhitelistedMDPName);
+                         kAllowlistedMDPName);
   }
   std::unique_ptr<Thread> stopped_thread(new Thread("test thread"));
   stopped_thread->Start();
   for (int i = 0; i < kMDPCount; ++i) {
     mdps.push_back(std::make_unique<SimpleMockMemoryDumpProvider>(0));
     RegisterDumpProvider(mdps.back().get(), stopped_thread->task_runner(),
-                         kDefaultOptions, kWhitelistedMDPName);
+                         kDefaultOptions, kAllowlistedMDPName);
   }
   stopped_thread->Stop();
 
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::DETAILED));
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::EXPLICITLY_TRIGGERED,
-                                        MemoryDumpLevelOfDetail::BACKGROUND));
-  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::SUMMARY_ONLY,
-                                        MemoryDumpLevelOfDetail::BACKGROUND));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kDetailed,
+                                        MemoryDumpDeterminism::kNone));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kExplicitlyTriggered,
+                                        MemoryDumpLevelOfDetail::kBackground,
+                                        MemoryDumpDeterminism::kNone));
+  EXPECT_TRUE(RequestProcessDumpAndWait(MemoryDumpType::kSummaryOnly,
+                                        MemoryDumpLevelOfDetail::kBackground,
+                                        MemoryDumpDeterminism::kNone));
 }
 
-}  // namespace trace_event
-}  // namespace base
+}  // namespace base::trace_event
