@@ -10,11 +10,13 @@
 #include <algorithm>
 #include <map>
 #include <memory>
+#include <set>
 #include <string>
 #include <vector>
 
 #include "base/memory/ptr_util.h"
 #include "base/memory/raw_ptr.h"
+#include "base/memory/raw_ptr_exclusion.h"
 #include "cc/cc_export.h"
 #include "cc/layers/layer.h"
 #include "cc/layers/layer_impl.h"
@@ -24,6 +26,8 @@
 #include "cc/raster/lcd_text_disallowed_reason.h"
 #include "cc/tiles/picture_layer_tiling.h"
 #include "cc/tiles/picture_layer_tiling_set.h"
+#include "cc/tiles/tile_index.h"
+#include "cc/tiles/tile_priority.h"
 #include "cc/tiles/tiling_set_eviction_queue.h"
 #include "cc/trees/image_animation_controller.h"
 
@@ -48,12 +52,16 @@ class CC_EXPORT PictureLayerImpl
   PictureLayerImpl& operator=(const PictureLayerImpl&) = delete;
 
   void SetIsBackdropFilterMask(bool is_backdrop_filter_mask) {
+    if (is_backdrop_filter_mask_ == is_backdrop_filter_mask) {
+      return;
+    }
     is_backdrop_filter_mask_ = is_backdrop_filter_mask;
+    SetNeedsPushProperties();
   }
   bool is_backdrop_filter_mask() const { return is_backdrop_filter_mask_; }
 
   // LayerImpl overrides.
-  const char* LayerTypeAsString() const override;
+  mojom::LayerType GetLayerType() const override;
   std::unique_ptr<LayerImpl> CreateLayerImpl(
       LayerTreeImpl* tree_impl) const override;
   void PushPropertiesTo(LayerImpl* layer) override;
@@ -70,6 +78,7 @@ class CC_EXPORT PictureLayerImpl
   Region GetInvalidationRegionForDebugging() override;
   gfx::Rect GetEnclosingVisibleRectInTargetSpace() const override;
   gfx::ContentColorUsage GetContentColorUsage() const override;
+  DamageReasonSet GetDamageReasons() const override;
 
   // PictureLayerTilingClient overrides.
   std::unique_ptr<Tile> CreateTile(const Tile::CreateInfo& info) override;
@@ -80,26 +89,31 @@ class CC_EXPORT PictureLayerImpl
   bool HasValidTilePriorities() const override;
   bool RequiresHighResToDraw() const override;
   const PaintWorkletRecordMap& GetPaintWorkletRecords() const override;
-  bool IsDirectlyCompositedImage() const override;
-  bool ScrollInteractionInProgress() const override;
-  bool CurrentScrollCheckerboardsDueToNoRecording() const override;
+  std::vector<const DrawImage*> GetDiscardableImagesInRect(
+      const gfx::Rect& rect) const override;
+  ScrollOffsetMap GetRasterInducingScrollOffsets() const override;
+  const GlobalStateThatImpactsTilePriority& global_tile_state() const override;
 
   // ImageAnimationController::AnimationDriver overrides.
   bool ShouldAnimate(PaintImage::Id paint_image_id) const override;
 
   void set_gpu_raster_max_texture_size(gfx::Size gpu_raster_max_texture_size) {
+    if (gpu_raster_max_texture_size_ == gpu_raster_max_texture_size) {
+      return;
+    }
     gpu_raster_max_texture_size_ = gpu_raster_max_texture_size;
+    SetNeedsPushProperties();
   }
 
   gfx::Size gpu_raster_max_texture_size() {
     return gpu_raster_max_texture_size_;
   }
 
-  void UpdateRasterSource(
-      scoped_refptr<RasterSource> raster_source,
-      Region* new_invalidation,
-      const PictureLayerTilingSet* pending_set,
-      const PaintWorkletRecordMap* pending_paint_worklet_records);
+  void UpdateRasterSource(scoped_refptr<RasterSource> raster_source,
+                          Region* new_invalidation);
+  void SetRasterSourceForTesting(scoped_refptr<RasterSource> raster_source,
+                                 const Region& invalidation = Region());
+  void RegenerateDiscardableImageMap();
   bool UpdateTiles();
 
   // Mask-related functions.
@@ -107,13 +121,6 @@ class CC_EXPORT PictureLayerImpl
                              gfx::Size* resource_size,
                              gfx::SizeF* resource_uv_size) const override;
 
-  void SetNearestNeighbor(bool nearest_neighbor);
-
-  void SetDirectlyCompositedImageDefaultRasterScale(
-      const gfx::Vector2dF& scale);
-  // TODO(crbug.com/1196414): Support 2D scales in directly composited images.
-  void SetDirectlyCompositedImageDefaultRasterScale(float scale);
-
   size_t GPUMemoryUsageInBytes() const override;
 
   void RunMicroBenchmark(MicroBenchmarkImpl* benchmark) override;
@@ -143,6 +150,9 @@ class CC_EXPORT PictureLayerImpl
   ImageInvalidationResult InvalidateRegionForImages(
       const PaintImageIdFlatSet& images_to_invalidate);
 
+  void InvalidateRasterInducingScrolls(
+      const base::flat_set<ElementId>& scrolls_to_invalidate);
+
   bool can_use_lcd_text() const {
     return lcd_text_disallowed_reason_ == LCDTextDisallowedReason::kNone;
   }
@@ -164,12 +174,12 @@ class CC_EXPORT PictureLayerImpl
     return paint_worklet_records_;
   }
 
-  gfx::Size content_bounds() { return content_bounds_; }
-
   // Invalidates all PaintWorklets in this layer who depend on the given
   // property to be painted. Used when the value for the property is changed by
   // an animation, at which point the PaintWorklet must be re-painted.
-  void InvalidatePaintWorklets(const PaintWorkletInput::PropertyKey& key);
+  void InvalidatePaintWorklets(const PaintWorkletInput::PropertyKey& key,
+                               const PaintWorkletInput::PropertyValue& prev,
+                               const PaintWorkletInput::PropertyValue& next);
 
   void SetContentsScaleForTesting(float scale) {
     ideal_contents_scale_ = raster_contents_scale_ =
@@ -180,6 +190,17 @@ class CC_EXPORT PictureLayerImpl
     last_append_quads_tilings_.push_back(tiling);
   }
 
+  void set_has_non_animated_image_update_rect() {
+    has_non_animated_image_update_rect_ = true;
+  }
+
+  // Returns the set of tiles which have been updated since the last call to
+  // this method. This returns tile indices for each updated tile, grouped by
+  // the scale key of their respective tiling. Beware that this is not pruned,
+  // so tilings or tiles identified within may no longer exist.
+  using TileUpdateSet = std::map<float, std::set<TileIndex>>;
+  TileUpdateSet TakeUpdatedTiles();
+
  protected:
   friend class RasterizeAndRecordBenchmarkImpl;
 
@@ -198,7 +219,8 @@ class CC_EXPORT PictureLayerImpl
   // Returns false if raster translation is not applicable.
   bool CalculateRasterTranslation(gfx::Vector2dF& raster_translation) const;
   void CleanUpTilingsOnActiveLayer(
-      const std::vector<PictureLayerTiling*>& used_tilings);
+      const std::vector<raw_ptr<PictureLayerTiling, VectorExperimental>>&
+          used_tilings);
   float MinimumContentsScale() const;
   float MaximumContentsScale() const;
   void UpdateViewportRectForTilePriorityInContentSpace();
@@ -212,6 +234,16 @@ class CC_EXPORT PictureLayerImpl
   // their contents scale.
   float CalculateDirectlyCompositedImageRasterScale() const;
 
+  void UpdateRasterSourceInternal(
+      scoped_refptr<RasterSource> raster_source,
+      Region* new_invalidation,
+      const PictureLayerTilingSet* pending_set,
+      const PaintWorkletRecordMap* pending_paint_worklet_records,
+      const DiscardableImageMap* pending_discardable_image_map);
+
+  bool IsDirectlyCompositedImage() const;
+  void UpdateDirectlyCompositedImageFromRasterSource();
+
   void SanityCheckTilingState() const;
 
   void GetDebugBorderProperties(SkColor4f* color, float* width) const override;
@@ -229,8 +261,7 @@ class CC_EXPORT PictureLayerImpl
   // Set the collection of PaintWorkletInput as well as their PaintImageId that
   // are part of this layer.
   void SetPaintWorkletInputs(
-      const std::vector<DiscardableImageMap::PaintWorkletInputWithImageId>&
-          inputs);
+      const DiscardableImageMap::PaintWorkletInputs& inputs);
 
   LCDTextDisallowedReason ComputeLCDTextDisallowedReason(
       bool raster_translation_aligns_pixels) const;
@@ -240,12 +271,14 @@ class CC_EXPORT PictureLayerImpl
   // node, has a will-change hint for one of the transform properties.
   bool AffectedByWillChangeTransformHint() const;
 
-  raw_ptr<PictureLayerImpl> twin_layer_ = nullptr;
+  // RAW_PTR_EXCLUSION: Performance reasons (based on analysis of speedometer3).
+  RAW_PTR_EXCLUSION PictureLayerImpl* twin_layer_ = nullptr;
 
   std::unique_ptr<PictureLayerTilingSet> tilings_ =
       CreatePictureLayerTilingSet();
   scoped_refptr<RasterSource> raster_source_;
   Region invalidation_;
+  scoped_refptr<const DiscardableImageMap> discardable_image_map_;
 
   // Ideal scales are calcuated from the transforms applied to the layer. They
   // represent the best known scale from the layer to the final output.
@@ -280,19 +313,24 @@ class CC_EXPORT PictureLayerImpl
     return std::max(raster_contents_scale_.x(), raster_contents_scale_.y());
   }
 
-  bool is_backdrop_filter_mask_ : 1;
+  bool is_backdrop_filter_mask_ : 1 = false;
 
-  bool was_screen_space_transform_animating_ : 1;
-  bool only_used_low_res_last_append_quads_ : 1;
+  bool was_screen_space_transform_animating_ : 1 = false;
+  bool only_used_low_res_last_append_quads_ : 1 = false;
 
-  bool nearest_neighbor_ : 1;
+  bool nearest_neighbor_ : 1 = false;
 
   // This is set by UpdateRasterSource() on change of raster source size. It's
   // used to recalculate raster scale for will-chagne:transform. It's reset to
   // false after raster scale update.
-  bool raster_source_size_changed_ : 1;
+  bool raster_source_size_changed_ : 1 = false;
 
-  bool directly_composited_image_default_raster_scale_changed_ : 1;
+  bool directly_composited_image_default_raster_scale_changed_ : 1 = false;
+
+  // Keep track of if a non-empty update_rect is due to animated image or other
+  // reasons.
+  bool has_animated_image_update_rect_ : 1 = false;
+  bool has_non_animated_image_update_rect_ : 1 = false;
 
   LCDTextDisallowedReason lcd_text_disallowed_reason_ =
       LCDTextDisallowedReason::kNoText;
@@ -304,7 +342,7 @@ class CC_EXPORT PictureLayerImpl
   // To avoid re-raster on scale changes, this may be different than the used
   // raster scale, see: |RecalculateRasterScales()| and
   // |CalculateDirectlyCompositedImageRasterScale()|.
-  // TODO(crbug.com/1196414): Support 2D scales in directly composited images.
+  // TODO(crbug.com/40176440): Support 2D scales in directly composited images.
   float directly_composited_image_default_raster_scale_ = 0;
 
   // Use this instead of |visible_layer_rect()| for tiling calculations. This
@@ -318,7 +356,8 @@ class CC_EXPORT PictureLayerImpl
   // drawn. Note that accessing this vector should only be done in the context
   // of comparing pointers, since objects pointed to are not guaranteed to
   // exist.
-  std::vector<PictureLayerTiling*> last_append_quads_tilings_;
+  std::vector<raw_ptr<PictureLayerTiling, VectorExperimental>>
+      last_append_quads_tilings_;
 
   // The set of PaintWorkletInputs that are part of this PictureLayerImpl, and
   // their painted results (if any). During commit, Blink hands us a set of
@@ -327,12 +366,14 @@ class CC_EXPORT PictureLayerImpl
   // |LayerTreeHostImpl::UpdateSyncTreeAfterCommitOrImplSideInvalidation|.
   PaintWorkletRecordMap paint_worklet_records_;
 
-  gfx::Size content_bounds_;
   TileSizeCalculator tile_size_calculator_{this};
 
   // Denotes an area that is damaged and needs redraw. This is in the layer's
   // space.
   gfx::Rect damage_rect_;
+
+  // Tracks tiles changed since the last call to TakeUpdatedTiles().
+  TileUpdateSet updated_tiles_;
 };
 
 }  // namespace cc
