@@ -1,28 +1,35 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef CC_METRICS_COMPOSITOR_FRAME_REPORTER_H_
 #define CC_METRICS_COMPOSITOR_FRAME_REPORTER_H_
 
+#include <array>
 #include <bitset>
+#include <deque>
 #include <memory>
+#include <optional>
 #include <queue>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "base/memory/raw_ptr.h"
+#include "base/memory/raw_ptr_exclusion.h"
 #include "base/time/default_tick_clock.h"
 #include "base/time/time.h"
 #include "cc/base/devtools_instrumentation.h"
 #include "cc/cc_export.h"
 #include "cc/metrics/begin_main_frame_metrics.h"
 #include "cc/metrics/event_metrics.h"
+#include "cc/metrics/frame_info.h"
 #include "cc/metrics/frame_sequence_metrics.h"
+#include "cc/metrics/predictor_jank_tracker.h"
+#include "cc/metrics/scroll_jank_dropped_frame_tracker.h"
 #include "cc/scheduler/scheduler.h"
 #include "components/viz/common/frame_sinks/begin_frame_args.h"
 #include "components/viz/common/frame_timing_details.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace viz {
 struct FrameTimingDetails;
@@ -30,8 +37,24 @@ struct FrameTimingDetails;
 
 namespace cc {
 class DroppedFrameCounter;
+class EventLatencyTracker;
+class FrameSequenceTrackerCollection;
 class LatencyUkmReporter;
 
+struct GlobalMetricsTrackers {
+  // RAW_PTR_EXCLUSION: Renderer performance: visible in sampling profiler
+  // stacks.
+  RAW_PTR_EXCLUSION DroppedFrameCounter* dropped_frame_counter = nullptr;
+  RAW_PTR_EXCLUSION LatencyUkmReporter* latency_ukm_reporter = nullptr;
+  RAW_PTR_EXCLUSION FrameSequenceTrackerCollection* frame_sequence_trackers =
+      nullptr;
+  RAW_PTR_EXCLUSION EventLatencyTracker* event_latency_tracker = nullptr;
+  RAW_PTR_EXCLUSION PredictorJankTracker* predictor_jank_tracker = nullptr;
+  RAW_PTR_EXCLUSION ScrollJankDroppedFrameTracker*
+      scroll_jank_dropped_frame_tracker = nullptr;
+  RAW_PTR_EXCLUSION ScrollJankUkmReporter* scroll_jank_ukm_reporter = nullptr;
+};
+
 // This is used for tracing and reporting the duration of pipeline stages within
 // a single frame.
 //
@@ -91,20 +114,29 @@ class CC_EXPORT CompositorFrameReporter {
     kStageTypeCount
   };
 
+  // Note that the values of `VizBreakdown` enum should be defined in order,
+  // (i.e. a breakdown that happens earlier in the pipeline should appear
+  // earlier in `VizBreakdown`) for traces to record them correctly. The only
+  // exception is `kSwapStartToSwapEnd` and its breakdowns as we either record
+  // the former or the latter in a trace, but not both.
+  // These values are persisted to logs. Entries should not be renumbered and
+  // numeric values should never be reused.
   enum class VizBreakdown {
     kSubmitToReceiveCompositorFrame = 0,
     kReceivedCompositorFrameToStartDraw = 1,
     kStartDrawToSwapStart = 2,
     kSwapStartToSwapEnd = 3,
-    kSwapEndToPresentationCompositorFrame = 4,
 
     // This is a breakdown of SwapStartToSwapEnd stage which is optionally
     // recorded if querying these timestamps is supported by the platform.
-    kSwapStartToBufferAvailable = 5,
-    kBufferAvailableToBufferReady = 6,
-    kBufferReadyToLatch = 7,
-    kLatchToSwapEnd = 8,
-    kBreakdownCount
+    kSwapStartToBufferAvailable = 4,
+    kBufferAvailableToBufferReady = 5,
+    kBufferReadyToLatch = 6,
+    kLatchToSwapEnd = 7,
+
+    kSwapEndToPresentationCompositorFrame = 8,
+    kBreakdownCount,
+    kMaxValue = kBreakdownCount
   };
 
   enum class BlinkBreakdown {
@@ -112,9 +144,9 @@ class CC_EXPORT CompositorFrameReporter {
     kAnimate = 1,
     kStyleUpdate = 2,
     kLayoutUpdate = 3,
-    kPrepaint = 4,
-    kCompositingInputs = 5,
-    kCompositingAssignments = 6,
+    kAccessibility = 4,
+    kPrepaint = 5,
+    kCompositingInputs = 6,
     kPaint = 7,
     kCompositeCommit = 8,
     kUpdateLayers = 9,
@@ -122,6 +154,23 @@ class CC_EXPORT CompositorFrameReporter {
     kBreakdownCount
   };
 
+  // These numbers are used for indexing UMA histograms. The order should be
+  // preserved, and entries should not be deleted.
+  //
+  // These represent ratios of stages in EventMetrics::DispatchStage to the
+  // VSync time when the event originally arrived. This can be different than
+  // the frame where this event was eventually presented.
+  enum class VSyncRatioType {
+    kArrivedInRendererVsVSyncRatioAfterVSync = 0,
+    kArrivedInRendererVsVSyncRatioBeforeVSync = 1,
+    kGenerationVsVsyncRatioAfterVSync = 2,
+    kGenerationVsVsyncRatioBeforeVSync = 3,
+    kVSyncRatioTypeCount
+  };
+
+  // To distinguish between impl and main reporter
+  enum class ReporterType { kImpl = 0, kMain = 1 };
+
   struct CC_EXPORT StageData {
     StageType stage_type;
     base::TimeTicks start_time;
@@ -134,13 +183,21 @@ class CC_EXPORT CompositorFrameReporter {
     ~StageData();
   };
 
-  enum SmoothThread {
-    kSmoothNone,
-    kSmoothCompositor,
-    kSmoothMain,
-    kSmoothBoth
+  struct CC_EXPORT EventLatencyInfo {
+    std::vector<base::TimeDelta> dispatch_durations;
+    base::TimeDelta transition_duration;
+    std::vector<base::TimeDelta> compositor_durations;
+    base::TimeDelta total_duration;
+    std::string transition_name;
+    EventLatencyInfo(const int num_dispatch_stages,
+                     const int num_compositor_stages);
+    EventLatencyInfo(const EventLatencyInfo&);
+    ~EventLatencyInfo();
   };
 
+  using SmoothThread = FrameInfo::SmoothThread;
+  using SmoothEffectDrivingThread = FrameInfo::SmoothEffectDrivingThread;
+
   // Holds a processed list of Blink breakdowns with an `Iterator` class to
   // easily iterator over them.
   class CC_EXPORT ProcessedBlinkBreakdown {
@@ -156,7 +213,9 @@ class CC_EXPORT CompositorFrameReporter {
       base::TimeDelta GetLatency() const;
 
      private:
-      const ProcessedBlinkBreakdown* owner_;
+      // RAW_PTR_EXCLUSION: Renderer performance: visible in sampling profiler
+      // stacks.
+      RAW_PTR_EXCLUSION const ProcessedBlinkBreakdown* owner_;
 
       size_t index_ = 0;
     };
@@ -173,7 +232,9 @@ class CC_EXPORT CompositorFrameReporter {
     Iterator CreateIterator() const;
 
    private:
-    base::TimeDelta list_[static_cast<int>(BlinkBreakdown::kBreakdownCount)];
+    std::array<base::TimeDelta,
+               static_cast<size_t>(BlinkBreakdown::kBreakdownCount)>
+        list_;
   };
 
   // Holds a processed list of Viz breakdowns with an `Iterator` class to easily
@@ -194,7 +255,12 @@ class CC_EXPORT CompositorFrameReporter {
       base::TimeDelta GetDuration() const;
 
      private:
-      const ProcessedVizBreakdown* owner_;
+      bool HasValue() const;
+      void SkipBreakdownsIfNecessary();
+
+      // RAW_PTR_EXCLUSION: Renderer performance: visible in sampling profiler
+      // stacks.
+      RAW_PTR_EXCLUSION const ProcessedVizBreakdown* owner_;
       const bool skip_swap_start_to_swap_end_;
 
       size_t index_ = 0;
@@ -216,27 +282,55 @@ class CC_EXPORT CompositorFrameReporter {
     base::TimeTicks swap_start() const { return swap_start_; }
 
    private:
-    absl::optional<std::pair<base::TimeTicks, base::TimeTicks>>
-        list_[static_cast<int>(VizBreakdown::kBreakdownCount)];
+    std::array<std::optional<std::pair<base::TimeTicks, base::TimeTicks>>,
+               static_cast<size_t>(VizBreakdown::kBreakdownCount)>
+        list_;
 
     bool buffer_ready_available_ = false;
     base::TimeTicks swap_start_;
   };
 
+  // Wrapper for all level of breakdown stages' prediction
+  struct CC_EXPORT CompositorLatencyInfo {
+    CompositorLatencyInfo();
+    explicit CompositorLatencyInfo(base::TimeDelta init_value);
+    ~CompositorLatencyInfo();
+
+    std::vector<base::TimeDelta> top_level_stages;
+    std::vector<base::TimeDelta> blink_breakdown_stages;
+    std::vector<base::TimeDelta> viz_breakdown_stages;
+
+    base::TimeDelta total_latency;
+    base::TimeDelta total_blink_latency;
+    base::TimeDelta total_viz_latency;
+  };
+
   CompositorFrameReporter(const ActiveTrackers& active_trackers,
                           const viz::BeginFrameArgs& args,
-                          LatencyUkmReporter* latency_ukm_reporter,
-                          bool should_report_metrics,
+                          bool should_report_histograms,
                           SmoothThread smooth_thread,
-                          FrameSequenceMetrics::ThreadType scrolling_thread,
+                          FrameInfo::SmoothEffectDrivingThread scrolling_thread,
                           int layer_tree_host_id,
-                          DroppedFrameCounter* dropped_frame_counter);
+                          const GlobalMetricsTrackers& trackers);
   ~CompositorFrameReporter();
 
   CompositorFrameReporter(const CompositorFrameReporter& reporter) = delete;
   CompositorFrameReporter& operator=(const CompositorFrameReporter& reporter) =
       delete;
 
+  // Name for `CompositorFrameReporter::StageType`, possibly suffixed with the
+  // name of the appropriate breakdown.
+  static const char* GetStageName(
+      StageType stage_type,
+      std::optional<VizBreakdown> viz_breakdown = std::nullopt,
+      std::optional<BlinkBreakdown> blink_breakdown = std::nullopt,
+      bool impl_only = false);
+
+  // Name for the viz breakdowns which are shown in traces as substages under
+  // PipelineReporter -> SubmitCompositorFrameToPresentationCompositorFrame or
+  // EventLatency -> SubmitCompositorFrameToPresentationCompositorFrame.
+  static const char* GetVizBreakdownName(VizBreakdown breakdown);
+
   // Creates and returns a clone of the reporter, only if it is currently in the
   // 'begin impl frame' stage. For any other state, it returns null.
   // This is used only when there is a partial update. So the cloned reporter
@@ -254,8 +348,13 @@ class CC_EXPORT CompositorFrameReporter {
   void SetVizBreakdown(const viz::FrameTimingDetails& viz_breakdown);
 
   void AddEventsMetrics(EventMetrics::List events_metrics);
+
+  // Erase and return all EventMetrics objects from our list.
   EventMetrics::List TakeEventsMetrics();
 
+  void set_normalized_invalidated_area(
+      std::optional<float> normalized_invalidated_area);
+
   size_t stage_history_size_for_testing() const {
     return stage_history_.size();
   }
@@ -290,8 +389,15 @@ class CC_EXPORT CompositorFrameReporter {
     tick_clock_ = tick_clock;
   }
 
-  void set_has_missing_content(bool has_missing_content) {
-    has_missing_content_ = has_missing_content;
+  void set_checkerboarded_needs_raster(bool checkerboarded_needs_raster) {
+    checkerboarded_needs_raster_ = checkerboarded_needs_raster;
+  }
+  void set_checkerboarded_needs_record(bool checkerboarded_needs_record) {
+    checkerboarded_needs_record_ = checkerboarded_needs_record;
+  }
+
+  void set_top_controls_moved(bool top_controls_moved) {
+    top_controls_moved_ = top_controls_moved;
   }
 
   void SetPartialUpdateDecider(CompositorFrameReporter* decider);
@@ -310,6 +416,14 @@ class CC_EXPORT CompositorFrameReporter {
         is_accompanied_by_main_thread_update;
   }
 
+  void set_is_forked(bool is_forked) { is_forked_ = is_forked; }
+  void set_is_backfill(bool is_backfill) { is_backfill_ = is_backfill; }
+  void set_created_new_tree(bool new_tree) { created_new_tree_ = new_tree; }
+  void set_want_new_tree(bool want_new_tree) { want_new_tree_ = want_new_tree; }
+  void set_invalidate_raster_scroll(bool invalidate_raster_scroll) {
+    invalidate_raster_scroll_ = invalidate_raster_scroll;
+  }
+
   const viz::BeginFrameId& frame_id() const { return args_.frame_id; }
 
   // Adopts |cloned_reporter|, i.e. keeps |cloned_reporter| alive until after
@@ -317,12 +431,53 @@ class CC_EXPORT CompositorFrameReporter {
   // created from this reporter using |CopyReporterAtBeginImplStage()|.
   void AdoptReporter(std::unique_ptr<CompositorFrameReporter> cloned_reporter);
 
+  // Called after the frame corresponding to this reporter was successfully
+  // presented. It doesn't get called when the frame is dropped or not submitted
+  // at all.
+  void DidSuccessfullyPresentFrame();
+
   // If this is a cloned reporter, then this returns a weak-ptr to the original
   // reporter this was cloned from (using |CopyReporterAtBeginImplStage()|).
 
   CompositorFrameReporter* partial_update_decider() const {
     return partial_update_decider_.get();
   }
+  using FrameReportTypes =
+      std::bitset<static_cast<size_t>(FrameReportType::kMaxValue) + 1>;
+
+  // This function is called to calculate breakdown stage duration's prediction
+  // based on the `previous_predictions` and update the `previous_predictions`
+  // to the new prediction calculated.
+  void CalculateCompositorLatencyPrediction(
+      CompositorLatencyInfo& previous_predictions,
+      base::TimeDelta prediction_deviation_threshold);
+
+  // Sets EventLatency stage duration predictions based on previous trace
+  // durations using exponentially weighted averages.
+  void CalculateEventLatencyPrediction(
+      CompositorFrameReporter::EventLatencyInfo& predicted_event_latency,
+      base::TimeDelta prediction_deviation_threshold);
+
+  ReporterType get_reporter_type() { return reporter_type_; }
+
+  void set_reporter_type_to_impl() { reporter_type_ = ReporterType::kImpl; }
+  void set_reporter_type_to_main() { reporter_type_ = ReporterType::kMain; }
+
+  const std::vector<std::string>& high_latency_substages_for_testing() {
+    return high_latency_substages_;
+  }
+
+  void ClearHighLatencySubstagesForTesting() {
+    high_latency_substages_.clear();
+  }
+
+  std::vector<std::unique_ptr<EventMetrics>>& events_metrics_for_testing() {
+    return events_metrics_;
+  }
+
+  // Erase and return only the EventMetrics objects which depend on main thread
+  // updates (see comments on EventMetrics::requires_main_thread_update_).
+  EventMetrics::List TakeMainBlockedEventsMetrics();
 
  protected:
   void set_has_partial_update(bool has_partial_update) {
@@ -333,7 +488,7 @@ class CC_EXPORT CompositorFrameReporter {
   void TerminateReporter();
   void EndCurrentStage(base::TimeTicks end_time);
 
-  void ReportCompositorLatencyHistograms() const;
+  void ReportCompositorLatencyMetrics() const;
   void ReportStageHistogramWithBreakdown(
       const StageData& stage,
       FrameSequenceTrackerType frame_sequence_tracker_type =
@@ -344,13 +499,17 @@ class CC_EXPORT CompositorFrameReporter {
       FrameSequenceTrackerType frame_sequence_tracker_type) const;
   void ReportCompositorLatencyHistogram(
       FrameSequenceTrackerType intraction_type,
-      const int stage_type_index,
+      StageType stage_type,
+      std::optional<VizBreakdown> viz_breakdown,
+      std::optional<BlinkBreakdown> blink_breakdown,
       base::TimeDelta time_delta) const;
 
-  void ReportEventLatencyHistograms() const;
-
-  void ReportCompositorLatencyTraceEvents() const;
+  void ReportEventLatencyMetrics() const;
+  void ReportCompositorLatencyTraceEvents(const FrameInfo& info) const;
   void ReportEventLatencyTraceEvents() const;
+  void ReportScrollJankMetrics() const;
+
+  void ReportPaintMetric() const;
 
   void EnableReportType(FrameReportType report_type) {
     report_types_.set(static_cast<size_t>(report_type));
@@ -367,11 +526,22 @@ class CC_EXPORT CompositorFrameReporter {
 
   base::TimeTicks Now() const;
 
-  bool IsDroppedFrameAffectingSmoothness() const;
+  FrameInfo GenerateFrameInfo() const;
 
   base::WeakPtr<CompositorFrameReporter> GetWeakPtr();
 
-  const bool should_report_metrics_;
+  void FindHighLatencyAttribution(
+      CompositorLatencyInfo& previous_predictions,
+      CompositorLatencyInfo& current_stage_durations);
+
+  void FindEventLatencyAttribution(
+      EventMetrics* event_metrics,
+      CompositorFrameReporter::EventLatencyInfo& predicted_event_latency,
+      CompositorFrameReporter::EventLatencyInfo& actual_event_latency);
+
+  // Whether UMA histograms should be reported or not.
+  const bool should_report_histograms_;
+
   const viz::BeginFrameArgs args_;
 
   StageData current_stage_;
@@ -392,8 +562,11 @@ class CC_EXPORT CompositorFrameReporter {
   // List of metrics for events affecting this frame.
   EventMetrics::List events_metrics_;
 
-  std::bitset<static_cast<size_t>(FrameReportType::kMaxValue) + 1>
-      report_types_;
+  // Total invalidated (repainted) area of a frame, normalized by the frame's
+  // output size.
+  std::optional<float> paint_metric_;
+
+  FrameReportTypes report_types_;
 
   base::TimeTicks frame_termination_time_;
   base::TimeTicks begin_main_frame_start_;
@@ -401,9 +574,7 @@ class CC_EXPORT CompositorFrameReporter {
       FrameTerminationStatus::kUnknown;
 
   const ActiveTrackers active_trackers_;
-  const FrameSequenceMetrics::ThreadType scrolling_thread_;
-
-  LatencyUkmReporter* latency_ukm_reporter_;
+  const FrameInfo::SmoothEffectDrivingThread scrolling_thread_;
 
   // Indicates if work on Impl frame is finished.
   bool did_finish_impl_frame_ = false;
@@ -413,13 +584,13 @@ class CC_EXPORT CompositorFrameReporter {
 
   // The timestamp of when the frame was marked as not having produced a frame
   // (through a call to DidNotProduceFrame()).
-  absl::optional<base::TimeTicks> did_not_produce_frame_time_;
-  absl::optional<FrameSkippedReason> frame_skip_reason_;
-  absl::optional<base::TimeTicks> main_frame_abort_time_;
+  std::optional<base::TimeTicks> did_not_produce_frame_time_;
+  std::optional<FrameSkippedReason> frame_skip_reason_;
+  std::optional<base::TimeTicks> main_frame_abort_time_;
 
-  const base::TickClock* tick_clock_ = base::DefaultTickClock::GetInstance();
+  raw_ptr<const base::TickClock> tick_clock_ =
+      base::DefaultTickClock::GetInstance();
 
-  DroppedFrameCounter* dropped_frame_counter_ = nullptr;
   bool has_partial_update_ = false;
 
   // If the submitted frame has update from main thread
@@ -428,9 +599,21 @@ class CC_EXPORT CompositorFrameReporter {
   const SmoothThread smooth_thread_;
   const int layer_tree_host_id_;
 
-  // Indicates whether the submitted frame had any missing content (i.e. content
-  // with checkerboarding).
-  bool has_missing_content_ = false;
+  // Indicates whether the submitted frame had any missing or incomplete
+  // content (i.e. content with checkerboarding), due to rasterization and
+  // recording, respectively.
+  bool checkerboarded_needs_raster_ = false;
+  bool checkerboarded_needs_record_ = false;
+
+  bool top_controls_moved_ = false;
+
+  // Indicates whether the frame is forked (i.e. a PipelineReporter event starts
+  // at the same frame sequence as another PipelineReporter).
+  bool is_forked_ = false;
+
+  // Indicates whether the frame is backfill (i.e. dropped frames when there are
+  // no partial compositor updates).
+  bool is_backfill_ = false;
 
   // For a reporter A, if the main-thread takes a long time to respond
   // to a begin-main-frame, then all reporters created (and terminated) until
@@ -440,9 +623,8 @@ class CC_EXPORT CompositorFrameReporter {
   // In such cases, |partial_update_dependents_| for A contains all the frames
   // that depend on A for deciding whether they had partial updates or not, and
   // |partial_update_decider_| is set to A for all these reporters.
-  std::queue<base::WeakPtr<CompositorFrameReporter>> partial_update_dependents_;
+  std::deque<base::WeakPtr<CompositorFrameReporter>> partial_update_dependents_;
   base::WeakPtr<CompositorFrameReporter> partial_update_decider_;
-  uint32_t discarded_partial_update_dependents_count_ = 0;
 
   // From the above example, it may be necessary for A to keep all the
   // dependents alive until A terminates, so that the dependents can set their
@@ -451,9 +633,22 @@ class CC_EXPORT CompositorFrameReporter {
   std::queue<std::unique_ptr<CompositorFrameReporter>>
       owned_partial_update_dependents_;
 
+  // Indicates whether or not an impl-side invalidation was necessary for a
+  // raster-dependent effect, and whether or not it occurred.
+  bool want_new_tree_ = false;
+  bool created_new_tree_ = false;
+
+  bool invalidate_raster_scroll_ = false;
+
+  const GlobalMetricsTrackers global_trackers_;
+
+  std::vector<std::string> high_latency_substages_;
+
+  ReporterType reporter_type_;
+
   base::WeakPtrFactory<CompositorFrameReporter> weak_factory_{this};
 };
 
 }  // namespace cc
 
-#endif  // CC_METRICS_COMPOSITOR_FRAME_REPORTER_H_"
+#endif  // CC_METRICS_COMPOSITOR_FRAME_REPORTER_H_
