@@ -1,24 +1,31 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/40285824): Remove this and convert code to safer constructs.
+#pragma allow_unsafe_buffers
+#endif
+
 #include "chrome/browser/devtools/device/adb/mock_adb_server.h"
 
 #include <stddef.h>
 #include <stdint.h>
 
+#include <string_view>
+
+#include "base/containers/span.h"
+#include "base/functional/bind.h"
 #include "base/location.h"
-#include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/memory/weak_ptr.h"
-#include "base/message_loop/message_loop.h"
+#include "base/run_loop.h"
 #include "base/sequence_checker.h"
-#include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/task/single_thread_task_runner.h"
+#include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/test_utils.h"
@@ -29,6 +36,7 @@
 #include "net/log/net_log_source.h"
 #include "net/socket/stream_socket.h"
 #include "net/socket/tcp_server_socket.h"
+#include "net/traffic_annotation/network_traffic_annotation_test_helper.h"
 
 using content::BrowserThread;
 
@@ -73,7 +81,9 @@ const char kSampleOpenedUnixSockets[] =
     "00000000: 00000002 00000000"
     " 00010000 0001 01 20894 @chrome_devtools_remote_1002\n"
     "00000000: 00000002 00000000"
-    " 00010000 0001 01 20895 @noprocess_devtools_remote\n";
+    " 00010000 0001 01 20895 @noprocess_devtools_remote\n"
+    "00000000: 00000002 00000000"
+    " 00010000 0001 01 20895 @node_devtools_remote\n";
 
 const char kSampleListProcesses[] =
     "USER    PID  PPID VSIZE  RSS    WCHAN    PC         NAME\n"
@@ -123,6 +133,11 @@ char kSampleWebViewVersion[] = "{\n"
     "   \"WebKit-Version\": \"537.36 (@157588)\"\n"
     "}";
 
+char kSampleNodeVersion[] = "{\n"
+    "   \"Browser\": \"node.js/v10.15.3\","
+    "   \"Protocol-Version\": \"1.1\""
+    "}";
+
 char kSampleChromePages[] = "[ {\n"
     "   \"description\": \"\",\n"
     "   \"devtoolsFrontendUrl\": \"/devtools/devtools.html?"
@@ -176,29 +191,51 @@ char kSampleWebViewPages[] = "[ {\n"
     "44681551-ADFD-2411-076B-3AB14C1C60E2\"\n"
     "}]";
 
-static const int kBufferSize = 16*1024;
-static const uint16_t kAdbPort = 5037;
+char kSampleNodePage[] = "[ {\n"
+    "   \"description\": \"\","
+    "   \"devtoolsFrontendUrl\": \"chrome-devtools://devtools/bundled/"
+    "js_app.html?experiments=true&v8only=true&ws=192.168.86.1:33279/"
+    "148b8b92-8ca0-43fd-b8c8-a351864644f8\","
+    "   \"faviconUrl\": \"https://nodejs.org/static/favicon.ico\","
+    "   \"id\": \"148b8b92-8ca0-43fd-b8c8-a351864644f8\","
+    "   \"title\": \"a-node-process\","
+    "   \"type\": \"node\","
+    "   \"url\": \"about:blank\",\n"
+    "   \"webSocketDebuggerUrl\": \"ws://192.168.86.1:33279/"
+    "148b8b92-8ca0-43fd-b8c8-a351864644f8\""
+    "} ]";
 
-static const int kAdbMessageHeaderSize = 4;
+static constexpr int kBufferSize = 16 * 1024;
+static constexpr uint16_t kAdbPort = 5037;
+
+static constexpr size_t kAdbMessageHeaderSize = 4;
 
 class SimpleHttpServer {
  public:
   class Parser {
    public:
-    virtual int Consume(const char* data, int size) = 0;
-    virtual ~Parser() {}
+    virtual size_t Consume(const char* data, size_t size) = 0;
+    virtual ~Parser() = default;
   };
 
-  using SendCallback = base::Callback<void(const std::string&)>;
-  using ParserFactory = base::Callback<Parser*(const SendCallback&)>;
+  using SendCallback = base::RepeatingCallback<void(const std::string&)>;
+  using ParserFactory = base::RepeatingCallback<Parser*(const SendCallback&)>;
 
   SimpleHttpServer(const ParserFactory& factory, net::IPEndPoint endpoint);
+
+  SimpleHttpServer(const SimpleHttpServer&) = delete;
+  SimpleHttpServer& operator=(const SimpleHttpServer&) = delete;
+
   virtual ~SimpleHttpServer();
 
  private:
   class Connection {
    public:
     Connection(net::StreamSocket* socket, const ParserFactory& factory);
+
+    Connection(const Connection&) = delete;
+    Connection& operator=(const Connection&) = delete;
+
     virtual ~Connection();
 
    private:
@@ -210,38 +247,35 @@ class SimpleHttpServer {
 
     std::unique_ptr<net::StreamSocket> socket_;
     std::unique_ptr<Parser> parser_;
-    scoped_refptr<net::GrowableIOBuffer> input_buffer_;
-    scoped_refptr<net::GrowableIOBuffer> output_buffer_;
-    int bytes_to_write_;
-    bool read_closed_;
+    scoped_refptr<net::GrowableIOBuffer> input_buffer_ =
+        base::MakeRefCounted<net::GrowableIOBuffer>();
+    scoped_refptr<net::GrowableIOBuffer> output_buffer_ =
+        base::MakeRefCounted<net::GrowableIOBuffer>();
+    size_t bytes_to_write_ = 0;
+    bool read_closed_ = false;
 
     SEQUENCE_CHECKER(sequence_checker_);
 
-    base::WeakPtrFactory<Connection> weak_factory_;
-
-    DISALLOW_COPY_AND_ASSIGN(Connection);
+    base::WeakPtrFactory<Connection> weak_factory_{this};
   };
 
   void OnConnect();
   void OnAccepted(int result);
 
   ParserFactory factory_;
-  std::unique_ptr<net::TCPServerSocket> socket_;
+  std::unique_ptr<net::TCPServerSocket> socket_ =
+      std::make_unique<net::TCPServerSocket>(nullptr, net::NetLogSource());
   std::unique_ptr<net::StreamSocket> client_socket_;
 
   SEQUENCE_CHECKER(sequence_checker_);
 
-  base::WeakPtrFactory<SimpleHttpServer> weak_factory_;
-
-  DISALLOW_COPY_AND_ASSIGN(SimpleHttpServer);
+  base::WeakPtrFactory<SimpleHttpServer> weak_factory_{this};
 };
 
 SimpleHttpServer::SimpleHttpServer(const ParserFactory& factory,
                                    net::IPEndPoint endpoint)
-    : factory_(factory),
-      socket_(new net::TCPServerSocket(nullptr, net::NetLogSource())),
-      weak_factory_(this) {
-  socket_->Listen(endpoint, 5);
+    : factory_(factory) {
+  socket_->Listen(endpoint, 5, /*ipv6_only=*/std::nullopt);
   OnConnect();
 }
 
@@ -252,13 +286,8 @@ SimpleHttpServer::~SimpleHttpServer() {
 SimpleHttpServer::Connection::Connection(net::StreamSocket* socket,
                                          const ParserFactory& factory)
     : socket_(socket),
-      parser_(factory.Run(base::Bind(&Connection::Send,
-                                     base::Unretained(this)))),
-      input_buffer_(new net::GrowableIOBuffer()),
-      output_buffer_(new net::GrowableIOBuffer()),
-      bytes_to_write_(0),
-      read_closed_(false),
-      weak_factory_(this) {
+      parser_(factory.Run(
+          base::BindRepeating(&Connection::Send, base::Unretained(this)))) {
   input_buffer_->SetCapacity(kBufferSize);
   ReadData();
 }
@@ -269,29 +298,32 @@ SimpleHttpServer::Connection::~Connection() {
 
 void SimpleHttpServer::Connection::Send(const std::string& message) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  const char* data = message.c_str();
-  int size = message.size();
-
-  if ((output_buffer_->offset() + bytes_to_write_ + size) >
-      output_buffer_->capacity()) {
+  const size_t size = message.size();
+  const size_t total_size = bytes_to_write_ + size;
+  const auto old_offset = base::checked_cast<size_t>(output_buffer_->offset());
+  const auto old_capacity =
+      base::checked_cast<size_t>(output_buffer_->capacity());
+  if ((old_offset + total_size) > old_capacity) {
     // If not enough space without relocation
-    if (output_buffer_->capacity() < (bytes_to_write_ + size)) {
+    if (old_capacity < total_size) {
       // If even buffer is not enough
-      int new_size = std::max(output_buffer_->capacity() * 2, size * 2);
-      output_buffer_->SetCapacity(new_size);
+      output_buffer_->SetCapacity(
+          base::checked_cast<int>(std::max(old_capacity * 2, size * 2)));
     }
-    memmove(output_buffer_->StartOfBuffer(),
-            output_buffer_->data(),
-            bytes_to_write_);
     output_buffer_->set_offset(0);
+    output_buffer_->span().copy_prefix_from(
+        output_buffer_->span().subspan(old_offset, bytes_to_write_));
   }
 
-  memcpy(output_buffer_->data() + bytes_to_write_, data, size);
-  bytes_to_write_ += size;
+  output_buffer_->span()
+      .subspan(bytes_to_write_, size)
+      .copy_from(base::as_byte_span(message));
+  bytes_to_write_ = total_size;
 
-  if (bytes_to_write_ == size)
+  if (total_size == size) {
     // If write loop wasn't yet started, then start it
     WriteData();
+  }
 }
 
 void SimpleHttpServer::Connection::ReadData() {
@@ -301,9 +333,8 @@ void SimpleHttpServer::Connection::ReadData() {
     input_buffer_->SetCapacity(input_buffer_->capacity() * 2);
 
   int read_result = socket_->Read(
-      input_buffer_.get(),
-      input_buffer_->RemainingCapacity(),
-      base::Bind(&Connection::OnDataRead, base::Unretained(this)));
+      input_buffer_.get(), input_buffer_->RemainingCapacity(),
+      base::BindOnce(&Connection::OnDataRead, base::Unretained(this)));
 
   if (read_result != net::ERR_IO_PENDING)
     OnDataRead(read_result);
@@ -319,33 +350,37 @@ void SimpleHttpServer::Connection::OnDataRead(int count) {
     return;
   }
   input_buffer_->set_offset(input_buffer_->offset() + count);
-  int bytes_processed;
+  size_t bytes_processed;
 
   do {
-    char* data = input_buffer_->StartOfBuffer();
-    int data_size = input_buffer_->offset();
-    bytes_processed = parser_->Consume(data, data_size);
+    base::span<uint8_t> data_buffer = input_buffer_->span_before_offset();
+    base::span<char> data_chars = base::as_writable_chars(data_buffer);
+    bytes_processed = parser_->Consume(data_chars.data(), data_chars.size());
 
     if (bytes_processed) {
-      memmove(data, data + bytes_processed, data_size - bytes_processed);
-      input_buffer_->set_offset(data_size - bytes_processed);
+      const size_t unprocessed_size = data_buffer.size() - bytes_processed;
+      input_buffer_->everything().copy_prefix_from(
+          data_buffer.subspan(bytes_processed));
+      input_buffer_->set_offset(unprocessed_size);
     }
   } while (bytes_processed);
   // Posting to avoid deep recursion in case of synchronous IO
-  base::ThreadTaskRunnerHandle::Get()->PostTask(
+  base::SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
       FROM_HERE,
       base::BindOnce(&Connection::ReadData, weak_factory_.GetWeakPtr()));
 }
 
 void SimpleHttpServer::Connection::WriteData() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
+  const auto bytes_to_write_i = base::checked_cast<int>(bytes_to_write_);
   CHECK_GE(output_buffer_->capacity(),
-           output_buffer_->offset() + bytes_to_write_) << "Overflow";
+           output_buffer_->offset() + bytes_to_write_i)
+      << "Overflow";
 
   int write_result = socket_->Write(
-      output_buffer_.get(),
-      bytes_to_write_,
-      base::Bind(&Connection::OnDataWritten, base::Unretained(this)));
+      output_buffer_.get(), bytes_to_write_i,
+      base::BindOnce(&Connection::OnDataWritten, base::Unretained(this)),
+      TRAFFIC_ANNOTATION_FOR_TESTS);
 
   if (write_result != net::ERR_IO_PENDING)
     OnDataWritten(write_result);
@@ -358,29 +393,34 @@ void SimpleHttpServer::Connection::OnDataWritten(int count) {
     return;
   }
   CHECK_GT(count, 0);
+  const auto bytes_to_write_i = base::checked_cast<int>(bytes_to_write_);
+  CHECK_LE(count, bytes_to_write_i);
   CHECK_GE(output_buffer_->capacity(),
-           output_buffer_->offset() + bytes_to_write_) << "Overflow";
+           output_buffer_->offset() + bytes_to_write_i)
+      << "Overflow";
 
   bytes_to_write_ -= count;
   output_buffer_->set_offset(output_buffer_->offset() + count);
 
-  if (bytes_to_write_ != 0)
+  if (bytes_to_write_ != 0) {
     // Posting to avoid deep recursion in case of synchronous IO
-    base::ThreadTaskRunnerHandle::Get()->PostTask(
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
         FROM_HERE,
         base::BindOnce(&Connection::WriteData, weak_factory_.GetWeakPtr()));
-  else if (read_closed_)
+  } else if (read_closed_) {
     delete this;
+  }
 }
 
 void SimpleHttpServer::OnConnect() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
-  int accept_result = socket_->Accept(&client_socket_,
-      base::Bind(&SimpleHttpServer::OnAccepted, base::Unretained(this)));
+  int accept_result = socket_->Accept(
+      &client_socket_,
+      base::BindOnce(&SimpleHttpServer::OnAccepted, base::Unretained(this)));
 
   if (accept_result != net::ERR_IO_PENDING)
-    base::ThreadTaskRunnerHandle::Get()->PostTask(
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostTask(
         FROM_HERE, base::BindOnce(&SimpleHttpServer::OnAccepted,
                                   weak_factory_.GetWeakPtr(), accept_result));
 }
@@ -409,7 +449,7 @@ class AdbParser : public SimpleHttpServer::Parser,
         callback_(callback) {
   }
 
-  int Consume(const char* data, int size) override {
+  size_t Consume(const char* data, size_t size) override {
     DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
     if (mock_connection_) {
       mock_connection_->Receive(std::string(data, size));
@@ -417,9 +457,9 @@ class AdbParser : public SimpleHttpServer::Parser,
     }
     if (size >= kAdbMessageHeaderSize) {
       std::string message_header(data, kAdbMessageHeaderSize);
-      int message_size;
+      uint32_t message_size;
 
-      EXPECT_TRUE(base::HexStringToInt(message_header, &message_size));
+      EXPECT_TRUE(base::HexStringToUInt(message_header, &message_size));
 
       if (size >= message_size + kAdbMessageHeaderSize) {
         std::string message_body(data + kAdbMessageHeaderSize, message_size);
@@ -444,7 +484,7 @@ class AdbParser : public SimpleHttpServer::Parser,
       Send("FAIL", "device offline (x)");
     } else {
       mock_connection_ =
-          base::MakeUnique<MockAndroidConnection>(this, serial_, command);
+          std::make_unique<MockAndroidConnection>(this, serial_, command);
     }
   }
 
@@ -465,11 +505,10 @@ class AdbParser : public SimpleHttpServer::Parser,
         buffer = std::string();
     }
 
-    int size = response.size();
-    if (size > 0) {
-      static const char kHexChars[] = "0123456789ABCDEF";
-      for (int i = 3; i >= 0; i--)
-        buffer += kHexChars[ (size >> 4*i) & 0x0f ];
+    if (size_t size = response.size(); size > 0) {
+      CHECK_LE(size, 0xffffu);
+      base::AppendHexEncodedByte(static_cast<uint8_t>(size >> 8), buffer);
+      base::AppendHexEncodedByte(static_cast<uint8_t>(size), buffer);
       if (flush_mode_ == FlushWithSize) {
           callback_.Run(buffer);
           buffer = std::string();
@@ -489,21 +528,21 @@ class AdbParser : public SimpleHttpServer::Parser,
   SEQUENCE_CHECKER(sequence_checker_);
 };
 
-static SimpleHttpServer* mock_adb_server_ = NULL;
+static SimpleHttpServer* mock_adb_server_ = nullptr;
 
 void StartMockAdbServerOnIOThread(FlushMode flush_mode) {
   CHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  CHECK(mock_adb_server_ == NULL);
+  CHECK(mock_adb_server_ == nullptr);
   net::IPEndPoint endpoint(net::IPAddress(127, 0, 0, 1), kAdbPort);
   mock_adb_server_ = new SimpleHttpServer(
-      base::Bind(&AdbParser::Create, flush_mode), endpoint);
+      base::BindRepeating(&AdbParser::Create, flush_mode), endpoint);
 }
 
 void StopMockAdbServerOnIOThread() {
   CHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  CHECK(mock_adb_server_ != NULL);
+  CHECK(mock_adb_server_ != nullptr);
   delete mock_adb_server_;
-  mock_adb_server_ = NULL;
+  mock_adb_server_ = nullptr;
 }
 
 } // namespace
@@ -517,8 +556,7 @@ MockAndroidConnection::MockAndroidConnection(
   ProcessCommand(command);
 }
 
-MockAndroidConnection::~MockAndroidConnection() {
-}
+MockAndroidConnection::~MockAndroidConnection() = default;
 
 void MockAndroidConnection::Receive(const std::string& data) {
   request_ += data;
@@ -527,9 +565,11 @@ void MockAndroidConnection::Receive(const std::string& data) {
     return;
 
   std::string request(request_.substr(0, request_end_pos));
-  std::vector<std::string> tokens =
-      base::SplitString(request, " ", base::KEEP_WHITESPACE,
-                        base::SPLIT_WANT_NONEMPTY);
+  std::vector<std::string_view> lines = base::SplitStringPieceUsingSubstr(
+      request, "\r\n", base::KEEP_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
+  CHECK_GE(2U, lines.size());
+  std::vector<std::string> tokens = base::SplitString(
+      lines[0], " ", base::KEEP_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
   CHECK_EQ(3U, tokens.size());
   CHECK_EQ("GET", tokens[0]);
   CHECK_EQ("HTTP/1.1", tokens[2]);
@@ -541,14 +581,14 @@ void MockAndroidConnection::Receive(const std::string& data) {
   if (socket_name_ == "chrome_devtools_remote") {
     if (path == kJsonVersionPath)
       SendHTTPResponse(kSampleChromeVersion);
-    else if (path == kJsonListPath)
+    else if (base::StartsWith(path, kJsonListPath))
       SendHTTPResponse(kSampleChromePages);
     else
       NOTREACHED() << "Unknown command " << request;
   } else if (socket_name_ == "chrome_devtools_remote_1002") {
     if (path == kJsonVersionPath)
       SendHTTPResponse(kSampleChromeBetaVersion);
-    else if (path == kJsonListPath)
+    else if (base::StartsWith(path, kJsonListPath))
       SendHTTPResponse(kSampleChromeBetaPages);
     else
       NOTREACHED() << "Unknown command " << request;
@@ -556,17 +596,24 @@ void MockAndroidConnection::Receive(const std::string& data) {
                               base::CompareCase::SENSITIVE)) {
     if (path == kJsonVersionPath)
       SendHTTPResponse("{}");
-    else if (path == kJsonListPath)
+    else if (base::StartsWith(path, kJsonListPath))
       SendHTTPResponse("[]");
     else
       NOTREACHED() << "Unknown command " << request;
   } else if (socket_name_ == "webview_devtools_remote_2425") {
     if (path == kJsonVersionPath)
       SendHTTPResponse(kSampleWebViewVersion);
-    else if (path == kJsonListPath)
+    else if (base::StartsWith(path, kJsonListPath))
       SendHTTPResponse(kSampleWebViewPages);
     else
       NOTREACHED() << "Unknown command " << request;
+  } else if (socket_name_ == "node_devtools_remote") {
+    if (path == kJsonVersionPath)
+      SendHTTPResponse(kSampleNodeVersion);
+    else if (base::StartsWith(path, kJsonListPath))
+      SendHTTPResponse(kSampleNodePage);
+    else
+      NOTREACHED() << "Unknown command " << request;
   } else {
     NOTREACHED() << "Unknown socket " << socket_name_;
   }
@@ -619,16 +666,17 @@ void MockAndroidConnection::SendHTTPResponse(const std::string& body) {
 }
 
 void StartMockAdbServer(FlushMode flush_mode) {
-  BrowserThread::PostTaskAndReply(
-      BrowserThread::IO, FROM_HERE,
-      base::BindOnce(&StartMockAdbServerOnIOThread, flush_mode),
-      base::MessageLoop::QuitWhenIdleClosure());
-  content::RunMessageLoop();
+  base::RunLoop run_loop;
+  content::GetIOThreadTaskRunner({})->PostTaskAndReply(
+      FROM_HERE, base::BindOnce(&StartMockAdbServerOnIOThread, flush_mode),
+      run_loop.QuitClosure());
+  run_loop.Run();
 }
 
 void StopMockAdbServer() {
-  BrowserThread::PostTaskAndReply(BrowserThread::IO, FROM_HERE,
-                                  base::BindOnce(&StopMockAdbServerOnIOThread),
-                                  base::MessageLoop::QuitWhenIdleClosure());
-  content::RunMessageLoop();
+  base::RunLoop run_loop;
+  content::GetIOThreadTaskRunner({})->PostTaskAndReply(
+      FROM_HERE, base::BindOnce(&StopMockAdbServerOnIOThread),
+      run_loop.QuitClosure());
+  run_loop.Run();
 }
