@@ -1,7 +1,7 @@
 // Copyright 2021 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
-#include "chrome/browser/extensions/api/enterprise_reporting_private/enterprise_reporting_private_api.h"
+#include <string_view>
 
 #include "base/command_line.h"
 #include "base/test/scoped_feature_list.h"
@@ -11,6 +11,7 @@
 #include "chrome/browser/content_settings/host_content_settings_map_factory.h"
 #include "chrome/browser/enterprise/connectors/test/deep_scanning_test_utils.h"
 #include "chrome/browser/enterprise/util/managed_browser_utils.h"
+#include "chrome/browser/extensions/api/enterprise_reporting_private/enterprise_reporting_private_api.h"
 #include "chrome/browser/net/profile_network_context_service.h"
 #include "chrome/browser/net/profile_network_context_service_factory.h"
 #include "chrome/browser/policy/chrome_browser_policy_connector.h"
@@ -32,6 +33,7 @@
 #include "content/public/common/content_switches.h"
 #include "content/public/test/browser_test.h"
 #include "extensions/browser/api_test_utils.h"
+#include "net/base/features.h"
 #include "net/cert/x509_certificate.h"
 #include "net/cert/x509_util.h"
 #include "net/ssl/client_cert_identity_test_util.h"
@@ -40,16 +42,17 @@
 #include "net/test/test_data_directory.h"
 
 #if BUILDFLAG(IS_CHROMEOS)
-#include "chrome/browser/policy/dm_token_utils.h"
-#endif
-
-#if BUILDFLAG(IS_CHROMEOS_ASH)
 #include "chrome/browser/ash/policy/core/browser_policy_connector_ash.h"
 #include "chrome/browser/ash/policy/core/user_cloud_policy_manager_ash.h"
 #include "chrome/browser/browser_process_platform_part.h"
+#include "chrome/browser/policy/dm_token_utils.h"
 #include "chromeos/dbus/constants/dbus_switches.h"
 #endif
 
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX)
+#include "components/device_signals/core/common/signals_features.h"
+#endif
+
 namespace enterprise_reporting_private =
     ::extensions::api::enterprise_reporting_private;
 
@@ -109,8 +112,9 @@ class MockClientCertStore : public net::ClientCertStore {
   explicit MockClientCertStore(net::ClientCertIdentityList certs)
       : certs_(std::move(certs)) {}
 
-  void GetClientCerts(const net::SSLCertRequestInfo& cert_request_info,
-                      ClientCertListCallback callback) override {
+  void GetClientCerts(
+      scoped_refptr<const net::SSLCertRequestInfo> cert_request_info,
+      ClientCertListCallback callback) override {
     std::move(callback).Run(std::move(certs_));
   }
 
@@ -125,7 +129,7 @@ class MockClientCertStore : public net::ClientCertStore {
 class EnterpriseReportingPrivateGetContextInfoBaseBrowserTest
     : public InProcessBrowserTest {
  public:
-#if !BUILDFLAG(GOOGLE_CHROME_BRANDING) && !BUILDFLAG(IS_CHROMEOS_ASH)
+#if !BUILDFLAG(GOOGLE_CHROME_BRANDING) && !BUILDFLAG(IS_CHROMEOS)
   void SetUpDefaultCommandLine(base::CommandLine* command_line) override {
     InProcessBrowserTest::SetUpDefaultCommandLine(command_line);
     command_line->AppendSwitch(::switches::kEnableChromeBrowserCloudManagement);
@@ -162,7 +166,7 @@ class EnterpriseReportingPrivateGetContextInfoBrowserTest
     if (browser_managed()) {
       SetupDMToken();
     }
-    feature_list_.InitAndEnableFeature(features::kAsyncDns);
+    feature_list_.InitAndEnableFeature(net::features::kAsyncDns);
   }
 
   bool browser_managed() const { return testing::get<0>(GetParam()); }
@@ -174,7 +178,7 @@ class EnterpriseReportingPrivateGetContextInfoBrowserTest
         SetUpOnMainThread();
 
     if (browser_managed()) {
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
       auto* browser_policy_manager = g_browser_process->platform_part()
                                          ->browser_policy_connector_ash()
                                          ->GetDeviceCloudPolicyManager();
@@ -192,7 +196,7 @@ class EnterpriseReportingPrivateGetContextInfoBrowserTest
     }
 
     if (profile_managed()) {
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
       auto* profile_policy_manager =
           browser()->profile()->GetUserCloudPolicyManagerAsh();
       profile_policy_manager->core()->client()->SetupRegistration(
@@ -251,24 +255,24 @@ IN_PROC_BROWSER_TEST_P(
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_TRUE(info.browser_affiliation_ids.empty());
-  EXPECT_TRUE(info.profile_affiliation_ids.empty());
-  EXPECT_TRUE(info.on_file_attached_providers.empty());
-  EXPECT_TRUE(info.on_file_downloaded_providers.empty());
-  EXPECT_TRUE(info.on_bulk_data_entry_providers.empty());
-  EXPECT_TRUE(info.on_print_providers.empty());
-  EXPECT_EQ(enterprise_reporting_private::REALTIME_URL_CHECK_MODE_DISABLED,
-            info.realtime_url_check_mode);
-  EXPECT_TRUE(info.on_security_event_providers.empty());
-  EXPECT_EQ(version_info::GetVersionNumber(), info.browser_version);
-  EXPECT_EQ(site_isolation_enabled(), info.site_isolation_enabled);
+  EXPECT_TRUE(info->browser_affiliation_ids.empty());
+  EXPECT_TRUE(info->profile_affiliation_ids.empty());
+  EXPECT_TRUE(info->on_file_attached_providers.empty());
+  EXPECT_TRUE(info->on_file_downloaded_providers.empty());
+  EXPECT_TRUE(info->on_bulk_data_entry_providers.empty());
+  EXPECT_TRUE(info->on_print_providers.empty());
+  EXPECT_EQ(enterprise_reporting_private::RealtimeUrlCheckMode::kDisabled,
+            info->realtime_url_check_mode);
+  EXPECT_TRUE(info->on_security_event_providers.empty());
+  EXPECT_EQ(version_info::GetVersionNumber(), info->browser_version);
+  EXPECT_EQ(site_isolation_enabled(), info->site_isolation_enabled);
 }
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
 class EnterpriseReportingPrivateGetContextInfoChromeOSFirewallTest
     : public EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
       public testing::WithParamInterface<bool> {
@@ -313,34 +317,33 @@ IN_PROC_BROWSER_TEST_P(
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_TRUE(info.browser_affiliation_ids.empty());
-  EXPECT_TRUE(info.profile_affiliation_ids.empty());
-  EXPECT_TRUE(info.on_file_attached_providers.empty());
-  EXPECT_TRUE(info.on_file_downloaded_providers.empty());
-  EXPECT_TRUE(info.on_bulk_data_entry_providers.empty());
-  EXPECT_TRUE(info.on_print_providers.empty());
-  EXPECT_EQ(enterprise_reporting_private::REALTIME_URL_CHECK_MODE_DISABLED,
-            info.realtime_url_check_mode);
-  EXPECT_TRUE(info.on_security_event_providers.empty());
-  EXPECT_EQ(version_info::GetVersionNumber(), info.browser_version);
-  EXPECT_EQ(enterprise_reporting_private::SAFE_BROWSING_LEVEL_STANDARD,
-            info.safe_browsing_protection_level);
+  EXPECT_TRUE(info->browser_affiliation_ids.empty());
+  EXPECT_TRUE(info->profile_affiliation_ids.empty());
+  EXPECT_TRUE(info->on_file_attached_providers.empty());
+  EXPECT_TRUE(info->on_file_downloaded_providers.empty());
+  EXPECT_TRUE(info->on_bulk_data_entry_providers.empty());
+  EXPECT_TRUE(info->on_print_providers.empty());
+  EXPECT_EQ(enterprise_reporting_private::RealtimeUrlCheckMode::kDisabled,
+            info->realtime_url_check_mode);
+  EXPECT_TRUE(info->on_security_event_providers.empty());
+  EXPECT_EQ(version_info::GetVersionNumber(), info->browser_version);
+  EXPECT_EQ(enterprise_reporting_private::SafeBrowsingLevel::kStandard,
+            info->safe_browsing_protection_level);
   EXPECT_EQ(BuiltInDnsClientPlatformDefault(),
-            info.built_in_dns_client_enabled);
+            info->built_in_dns_client_enabled);
   EXPECT_EQ(
-      enterprise_reporting_private::PASSWORD_PROTECTION_TRIGGER_POLICY_UNSET,
-      info.password_protection_warning_trigger);
-  EXPECT_FALSE(info.chrome_remote_desktop_app_blocked);
-  ExpectDefaultThirdPartyBlockingEnabled(info);
-  EXPECT_EQ(dev_mode_enabled() ? api::enterprise_reporting_private::
-                                     SettingValue::SETTING_VALUE_UNKNOWN
-                               : api::enterprise_reporting_private::
-                                     SettingValue::SETTING_VALUE_ENABLED,
-            info.os_firewall);
+      enterprise_reporting_private::PasswordProtectionTrigger::kPolicyUnset,
+      info->password_protection_warning_trigger);
+  EXPECT_FALSE(info->chrome_remote_desktop_app_blocked);
+  ExpectDefaultThirdPartyBlockingEnabled(*info);
+  EXPECT_EQ(dev_mode_enabled()
+                ? api::enterprise_reporting_private::SettingValue::kUnknown
+                : api::enterprise_reporting_private::SettingValue::kEnabled,
+            info->os_firewall);
 }
 
 INSTANTIATE_TEST_SUITE_P(
@@ -349,14 +352,8 @@ INSTANTIATE_TEST_SUITE_P(
     testing::Bool());
 #endif
 
-// crbug.com/1230268 not working on Lacros.
-#if BUILDFLAG(IS_CHROMEOS_LACROS)
-#define MAYBE_AffiliationIDs DISABLED_AffiliationIDs
-#else
-#define MAYBE_AffiliationIDs AffiliationIDs
-#endif
 IN_PROC_BROWSER_TEST_P(EnterpriseReportingPrivateGetContextInfoBrowserTest,
-                       MAYBE_AffiliationIDs) {
+                       AffiliationIDs) {
   auto function =
       base::MakeRefCounted<EnterpriseReportingPrivateGetContextInfoFunction>();
   auto context_info_value = api_test_utils::RunFunctionAndReturnSingleResult(
@@ -365,45 +362,45 @@ IN_PROC_BROWSER_TEST_P(EnterpriseReportingPrivateGetContextInfoBrowserTest,
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
   if (profile_managed()) {
-    EXPECT_EQ(2u, info.profile_affiliation_ids.size());
-    EXPECT_EQ(kProfileID1, info.profile_affiliation_ids[0]);
-    EXPECT_EQ(kProfileID2, info.profile_affiliation_ids[1]);
+    EXPECT_EQ(2u, info->profile_affiliation_ids.size());
+    EXPECT_EQ(kProfileID1, info->profile_affiliation_ids[0]);
+    EXPECT_EQ(kProfileID2, info->profile_affiliation_ids[1]);
   } else {
-    EXPECT_TRUE(info.profile_affiliation_ids.empty());
+    EXPECT_TRUE(info->profile_affiliation_ids.empty());
   }
 
   if (browser_managed()) {
-    EXPECT_EQ(2u, info.browser_affiliation_ids.size());
-    EXPECT_EQ(kBrowserID1, info.browser_affiliation_ids[0]);
-    EXPECT_EQ(kBrowserID2, info.browser_affiliation_ids[1]);
+    EXPECT_EQ(2u, info->browser_affiliation_ids.size());
+    EXPECT_EQ(kBrowserID1, info->browser_affiliation_ids[0]);
+    EXPECT_EQ(kBrowserID2, info->browser_affiliation_ids[1]);
   } else {
-    EXPECT_TRUE(info.browser_affiliation_ids.empty());
+    EXPECT_TRUE(info->browser_affiliation_ids.empty());
   }
 
-  EXPECT_TRUE(info.on_file_attached_providers.empty());
-  EXPECT_TRUE(info.on_file_downloaded_providers.empty());
-  EXPECT_TRUE(info.on_bulk_data_entry_providers.empty());
-  EXPECT_TRUE(info.on_print_providers.empty());
-  EXPECT_EQ(enterprise_reporting_private::REALTIME_URL_CHECK_MODE_DISABLED,
-            info.realtime_url_check_mode);
-  EXPECT_TRUE(info.on_security_event_providers.empty());
-  EXPECT_EQ(version_info::GetVersionNumber(), info.browser_version);
-  EXPECT_EQ(enterprise_reporting_private::SAFE_BROWSING_LEVEL_STANDARD,
-            info.safe_browsing_protection_level);
-  EXPECT_TRUE(info.built_in_dns_client_enabled);
+  EXPECT_TRUE(info->on_file_attached_providers.empty());
+  EXPECT_TRUE(info->on_file_downloaded_providers.empty());
+  EXPECT_TRUE(info->on_bulk_data_entry_providers.empty());
+  EXPECT_TRUE(info->on_print_providers.empty());
+  EXPECT_EQ(enterprise_reporting_private::RealtimeUrlCheckMode::kDisabled,
+            info->realtime_url_check_mode);
+  EXPECT_TRUE(info->on_security_event_providers.empty());
+  EXPECT_EQ(version_info::GetVersionNumber(), info->browser_version);
+  EXPECT_EQ(enterprise_reporting_private::SafeBrowsingLevel::kStandard,
+            info->safe_browsing_protection_level);
+  EXPECT_TRUE(info->built_in_dns_client_enabled);
   EXPECT_EQ(
-      enterprise_reporting_private::PASSWORD_PROTECTION_TRIGGER_POLICY_UNSET,
-      info.password_protection_warning_trigger);
-  EXPECT_FALSE(info.chrome_remote_desktop_app_blocked);
+      enterprise_reporting_private::PasswordProtectionTrigger::kPolicyUnset,
+      info->password_protection_warning_trigger);
+  EXPECT_FALSE(info->chrome_remote_desktop_app_blocked);
 #if BUILDFLAG(IS_WIN) && BUILDFLAG(GOOGLE_CHROME_BRANDING)
-  EXPECT_TRUE(*info.third_party_blocking_enabled);
+  EXPECT_TRUE(*info->third_party_blocking_enabled);
 #else
-  EXPECT_FALSE(info.third_party_blocking_enabled.has_value());
+  EXPECT_FALSE(info->third_party_blocking_enabled.has_value());
 #endif
 }
 
@@ -422,16 +419,16 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_EQ(0UL, info.on_file_downloaded_providers.size());
-  EXPECT_EQ(0UL, info.on_bulk_data_entry_providers.size());
-  EXPECT_EQ(0UL, info.on_print_providers.size());
+  EXPECT_EQ(0UL, info->on_file_downloaded_providers.size());
+  EXPECT_EQ(0UL, info->on_bulk_data_entry_providers.size());
+  EXPECT_EQ(0UL, info->on_print_providers.size());
 
-  EXPECT_EQ(1UL, info.on_file_attached_providers.size());
-  EXPECT_EQ("google", info.on_file_attached_providers[0]);
+  EXPECT_EQ(1UL, info->on_file_attached_providers.size());
+  EXPECT_EQ("google", info->on_file_attached_providers[0]);
 }
 
 IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
@@ -449,16 +446,16 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_EQ(0UL, info.on_file_attached_providers.size());
-  EXPECT_EQ(0UL, info.on_bulk_data_entry_providers.size());
-  EXPECT_EQ(0UL, info.on_print_providers.size());
+  EXPECT_EQ(0UL, info->on_file_attached_providers.size());
+  EXPECT_EQ(0UL, info->on_bulk_data_entry_providers.size());
+  EXPECT_EQ(0UL, info->on_print_providers.size());
 
-  EXPECT_EQ(1UL, info.on_file_downloaded_providers.size());
-  EXPECT_EQ("google", info.on_file_downloaded_providers[0]);
+  EXPECT_EQ(1UL, info->on_file_downloaded_providers.size());
+  EXPECT_EQ("google", info->on_file_downloaded_providers[0]);
 }
 
 IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
@@ -476,16 +473,16 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_EQ(0UL, info.on_file_downloaded_providers.size());
-  EXPECT_EQ(0UL, info.on_file_attached_providers.size());
-  EXPECT_EQ(0UL, info.on_print_providers.size());
+  EXPECT_EQ(0UL, info->on_file_downloaded_providers.size());
+  EXPECT_EQ(0UL, info->on_file_attached_providers.size());
+  EXPECT_EQ(0UL, info->on_print_providers.size());
 
-  EXPECT_EQ(1UL, info.on_bulk_data_entry_providers.size());
-  EXPECT_EQ("google", info.on_bulk_data_entry_providers[0]);
+  EXPECT_EQ(1UL, info->on_bulk_data_entry_providers.size());
+  EXPECT_EQ("google", info->on_bulk_data_entry_providers[0]);
 }
 
 IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
@@ -503,16 +500,16 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_EQ(0UL, info.on_file_downloaded_providers.size());
-  EXPECT_EQ(0UL, info.on_file_attached_providers.size());
-  EXPECT_EQ(0UL, info.on_bulk_data_entry_providers.size());
+  EXPECT_EQ(0UL, info->on_file_downloaded_providers.size());
+  EXPECT_EQ(0UL, info->on_file_attached_providers.size());
+  EXPECT_EQ(0UL, info->on_bulk_data_entry_providers.size());
 
-  EXPECT_EQ(1UL, info.on_print_providers.size());
-  EXPECT_EQ("google", info.on_print_providers[0]);
+  EXPECT_EQ(1UL, info->on_print_providers.size());
+  EXPECT_EQ("google", info->on_print_providers[0]);
 }
 
 IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
@@ -539,21 +536,21 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_EQ(1UL, info.on_bulk_data_entry_providers.size());
-  EXPECT_EQ("google", info.on_bulk_data_entry_providers[0]);
+  EXPECT_EQ(1UL, info->on_bulk_data_entry_providers.size());
+  EXPECT_EQ("google", info->on_bulk_data_entry_providers[0]);
 
-  EXPECT_EQ(1UL, info.on_file_attached_providers.size());
-  EXPECT_EQ("other", info.on_file_attached_providers[0]);
+  EXPECT_EQ(1UL, info->on_file_attached_providers.size());
+  EXPECT_EQ("other", info->on_file_attached_providers[0]);
 
-  EXPECT_EQ(1UL, info.on_file_downloaded_providers.size());
-  EXPECT_EQ("another", info.on_file_downloaded_providers[0]);
+  EXPECT_EQ(1UL, info->on_file_downloaded_providers.size());
+  EXPECT_EQ("another", info->on_file_downloaded_providers[0]);
 
-  EXPECT_EQ(1UL, info.on_print_providers.size());
-  EXPECT_EQ("and_another", info.on_print_providers[0]);
+  EXPECT_EQ(1UL, info->on_print_providers.size());
+  EXPECT_EQ("and_another", info->on_print_providers[0]);
 }
 
 IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
@@ -568,11 +565,11 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_EQ(0UL, info.on_security_event_providers.size());
+  EXPECT_EQ(0UL, info->on_security_event_providers.size());
 }
 
 IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
@@ -590,17 +587,27 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetContextInfoBaseBrowserTest,
   ASSERT_TRUE(context_info_value);
   ASSERT_TRUE(context_info_value->is_dict());
 
-  enterprise_reporting_private::ContextInfo info;
-  ASSERT_TRUE(enterprise_reporting_private::ContextInfo::Populate(
-      context_info_value->GetDict(), info));
+  auto info = enterprise_reporting_private::ContextInfo::FromValue(
+      context_info_value->GetDict());
+  ASSERT_TRUE(info);
 
-  EXPECT_EQ(1UL, info.on_security_event_providers.size());
+  EXPECT_EQ(1UL, info->on_security_event_providers.size());
   // test::SetOnSecurityEventReporting sets the provider name to google
-  EXPECT_EQ("google", info.on_security_event_providers[0]);
+  EXPECT_EQ("google", info->on_security_event_providers[0]);
 }
 
-class EnterpriseReportingPrivateGetCertificateTest : public policy::PolicyTest {
+class EnterpriseReportingPrivateGetCertificateTest
+    : public policy::PolicyTest,
+      public testing::WithParamInterface<bool> {
  public:
+  EnterpriseReportingPrivateGetCertificateTest() {
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX)
+    feature_list_.InitWithFeatureState(
+        enterprise_signals::features::kAllowClientCertificateReportingForUsers,
+        GetParam());
+#endif
+  }
+
   void SetUpOnMainThread() override {
     ProfileNetworkContextServiceFactory::GetForContext(browser()->profile())
         ->set_client_cert_store_factory_for_testing(base::BindRepeating(
@@ -609,7 +616,7 @@ class EnterpriseReportingPrivateGetCertificateTest : public policy::PolicyTest {
   }
 
   void SetPolicyValue(const std::string& policy_value) {
-    EXPECT_FALSE(chrome::enterprise_util::IsMachinePolicyPref(
+    EXPECT_FALSE(enterprise_util::IsMachinePolicyPref(
         prefs::kManagedAutoSelectCertificateForUrls));
 
     base::Value::List list;
@@ -622,10 +629,24 @@ class EnterpriseReportingPrivateGetCertificateTest : public policy::PolicyTest {
                  nullptr);
     UpdateProviderPolicy(policies);
 
-    EXPECT_TRUE(chrome::enterprise_util::IsMachinePolicyPref(
+    EXPECT_TRUE(enterprise_util::IsMachinePolicyPref(
         prefs::kManagedAutoSelectCertificateForUrls));
   }
 
+  void SetUserPolicyValue(const std::string& policy_value) {
+    EXPECT_FALSE(enterprise_util::IsMachinePolicyPref(
+        prefs::kManagedAutoSelectCertificateForUrls));
+    base::Value::List list;
+    list.Append(policy_value);
+
+    policy::PolicyMap policies;
+    policies.Set(policy::key::kAutoSelectCertificateForUrls,
+                 policy::POLICY_LEVEL_MANDATORY, policy::POLICY_SCOPE_USER,
+                 policy::POLICY_SOURCE_CLOUD, base::Value(std::move(list)),
+                 nullptr);
+    UpdateProviderPolicy(policies);
+  }
+
   enterprise_reporting_private::Certificate GetCertificate() {
     auto function = base::MakeRefCounted<
         EnterpriseReportingPrivateGetCertificateFunction>();
@@ -639,11 +660,12 @@ class EnterpriseReportingPrivateGetCertificateTest : public policy::PolicyTest {
     EXPECT_TRUE(certificate_value);
     EXPECT_TRUE(certificate_value->is_dict());
 
-    enterprise_reporting_private::Certificate cert;
-    EXPECT_TRUE(enterprise_reporting_private::Certificate::Populate(
-        certificate_value->GetDict(), cert));
+    auto cert = enterprise_reporting_private::Certificate::FromValue(
+        certificate_value->GetDict());
+    EXPECT_TRUE(cert);
 
-    return cert;
+    return std::move(cert).value_or(
+        enterprise_reporting_private::Certificate());
   }
 
   void SetupDefaultClientCertList() {
@@ -666,17 +688,18 @@ class EnterpriseReportingPrivateGetCertificateTest : public policy::PolicyTest {
   }
 
   std::vector<scoped_refptr<net::X509Certificate>> client_certs_;
+  base::test::ScopedFeatureList feature_list_;
 };
 
-IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
+IN_PROC_BROWSER_TEST_P(EnterpriseReportingPrivateGetCertificateTest,
                        TestPolicyUnset) {
   auto cert = GetCertificate();
 
-  EXPECT_EQ(enterprise_reporting_private::CERTIFICATE_STATUS_POLICY_UNSET,
+  EXPECT_EQ(enterprise_reporting_private::CertificateStatus::kPolicyUnset,
             cert.status);
 }
 
-IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
+IN_PROC_BROWSER_TEST_P(EnterpriseReportingPrivateGetCertificateTest,
                        TestPolicySet) {
   constexpr char kPolicyValue[] = R"({
       "pattern": "https://www.example.com",
@@ -690,11 +713,11 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
 
   auto cert = GetCertificate();
 
-  EXPECT_EQ(enterprise_reporting_private::CERTIFICATE_STATUS_OK, cert.status);
+  EXPECT_EQ(enterprise_reporting_private::CertificateStatus::kOk, cert.status);
   EXPECT_FALSE(cert.encoded_certificate.has_value());
 }
 
-IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
+IN_PROC_BROWSER_TEST_P(EnterpriseReportingPrivateGetCertificateTest,
                        TestPolicySetCertsPresentButNotMatching) {
   SetupDefaultClientCertList();
 
@@ -710,11 +733,11 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
 
   auto cert = GetCertificate();
 
-  EXPECT_EQ(enterprise_reporting_private::CERTIFICATE_STATUS_OK, cert.status);
+  EXPECT_EQ(enterprise_reporting_private::CertificateStatus::kOk, cert.status);
   EXPECT_FALSE(cert.encoded_certificate.has_value());
 }
 
-IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
+IN_PROC_BROWSER_TEST_P(EnterpriseReportingPrivateGetCertificateTest,
                        TestPolicySetCertsPresentUrlNotMatching) {
   SetupDefaultClientCertList();
 
@@ -730,11 +753,11 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
 
   auto cert = GetCertificate();
 
-  EXPECT_EQ(enterprise_reporting_private::CERTIFICATE_STATUS_OK, cert.status);
+  EXPECT_EQ(enterprise_reporting_private::CertificateStatus::kOk, cert.status);
   EXPECT_FALSE(cert.encoded_certificate.has_value());
 }
 
-IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
+IN_PROC_BROWSER_TEST_P(EnterpriseReportingPrivateGetCertificateTest,
                        TestPolicySetCertsPresentAndMatching) {
   SetupDefaultClientCertList();
 
@@ -750,14 +773,48 @@ IN_PROC_BROWSER_TEST_F(EnterpriseReportingPrivateGetCertificateTest,
 
   auto cert = GetCertificate();
 
-  EXPECT_EQ(enterprise_reporting_private::CERTIFICATE_STATUS_OK, cert.status);
+  EXPECT_EQ(enterprise_reporting_private::CertificateStatus::kOk, cert.status);
   EXPECT_TRUE(cert.encoded_certificate.has_value());
 
-  base::StringPiece der_cert = net::x509_util::CryptoBufferAsStringPiece(
+  std::string_view der_cert = net::x509_util::CryptoBufferAsStringPiece(
       client_certs()[0]->cert_buffer());
   std::vector<uint8_t> expected_der_bytes(der_cert.begin(), der_cert.end());
 
   EXPECT_EQ(expected_der_bytes, *cert.encoded_certificate);
 }
 
+IN_PROC_BROWSER_TEST_P(EnterpriseReportingPrivateGetCertificateTest,
+                       TestUserPolicySetCertsPresentAndMatching) {
+  SetupDefaultClientCertList();
+
+  constexpr char kPolicyValue[] = R"({
+      "pattern": "https://www.example.com",
+      "filter": {
+        "ISSUER": {
+          "CN": "B CA"
+        }
+      }
+    })";
+  SetUserPolicyValue(kPolicyValue);
+
+  auto cert = GetCertificate();
+
+  EXPECT_EQ(enterprise_reporting_private::CertificateStatus::kOk, cert.status);
+  EXPECT_TRUE(cert.encoded_certificate.has_value());
+
+  std::string_view der_cert = net::x509_util::CryptoBufferAsStringPiece(
+      client_certs()[0]->cert_buffer());
+  std::vector<uint8_t> expected_der_bytes(der_cert.begin(), der_cert.end());
+
+  EXPECT_EQ(expected_der_bytes, *cert.encoded_certificate);
+}
+
+INSTANTIATE_TEST_SUITE_P(,
+                         EnterpriseReportingPrivateGetCertificateTest,
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX)
+                         testing::Bool());
+#else
+                         testing::Values(false));
+#endif
+
 }  // namespace extensions
