@@ -1,58 +1,79 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/api/image_writer_private/operation.h"
 
+#include <string_view>
 #include <utility>
 
-#include "base/bind.h"
+#include "base/containers/heap_array.h"
+#include "base/containers/span.h"
 #include "base/files/file_util.h"
-#include "base/task/post_task.h"
+#include "base/functional/bind.h"
+#include "base/task/thread_pool.h"
 #include "build/build_config.h"
-#include "chrome/browser/extensions/api/image_writer_private/error_messages.h"
+#include "build/chromeos_buildflags.h"
+#include "chrome/browser/extensions/api/image_writer_private/error_constants.h"
+#include "chrome/browser/extensions/api/image_writer_private/extraction_properties.h"
+#include "chrome/browser/extensions/api/image_writer_private/image_writer_utility_client.h"
 #include "chrome/browser/extensions/api/image_writer_private/operation_manager.h"
-#include "chrome/browser/extensions/api/image_writer_private/unzip_helper.h"
+#include "chrome/browser/extensions/api/image_writer_private/tar_extractor.h"
+#include "chrome/browser/extensions/api/image_writer_private/xz_extractor.h"
+#include "chrome/browser/extensions/api/image_writer_private/zip_extractor.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
-#include "services/service_manager/public/cpp/connector.h"
 
 namespace extensions {
 namespace image_writer {
 
-using content::BrowserThread;
-
 namespace {
 
 const int kMD5BufferSize = 1024;
 
+// Returns true if the file at |image_path| is an archived image.
+bool IsArchive(const base::FilePath& image_path) {
+  return ZipExtractor::IsZipFile(image_path) ||
+         TarExtractor::IsTarFile(image_path) ||
+         XzExtractor::IsXzFile(image_path);
+}
+
+// Extracts the archive at |image_path| using to |temp_dir_path| using a proper
+// extractor.
+void ExtractArchive(ExtractionProperties properties) {
+  if (ZipExtractor::IsZipFile(properties.image_path)) {
+    ZipExtractor::Extract(std::move(properties));
+  } else if (TarExtractor::IsTarFile(properties.image_path)) {
+    TarExtractor::Extract(std::move(properties));
+  } else if (XzExtractor::IsXzFile(properties.image_path)) {
+    XzExtractor::Extract(std::move(properties));
+  } else {
+    NOTREACHED();
+  }
+}
+
 }  // namespace
 
 Operation::Operation(base::WeakPtr<OperationManager> manager,
-                     std::unique_ptr<service_manager::Connector> connector,
                      const ExtensionId& extension_id,
                      const std::string& device_path,
                      const base::FilePath& download_folder)
     : manager_(manager),
       extension_id_(extension_id),
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
       device_path_(base::FilePath::FromUTF8Unsafe(device_path)),
 #else
       device_path_(device_path),
 #endif
       temp_dir_(std::make_unique<base::ScopedTempDir>()),
-      connector_(std::move(connector)),
-      stage_(image_writer_api::STAGE_UNKNOWN),
+      stage_(image_writer_api::Stage::kUnknown),
       progress_(0),
       download_folder_(download_folder),
       task_runner_(
-          base::CreateSequencedTaskRunnerWithTraits(blocking_task_traits())) {
+          base::ThreadPool::CreateSequencedTaskRunner(blocking_task_traits())) {
 }
 
 Operation::~Operation() {
-  // The connector_ is bound to the |task_runner_| and must be deleted there.
-  task_runner_->DeleteSoon(FROM_HERE, std::move(connector_));
-
   // base::ScopedTempDir must be destroyed on a thread that allows blocking IO
   // because it will try delete the directory if a call to Delete() hasn't been
   // made or was unsuccessful.
@@ -62,7 +83,7 @@ Operation::~Operation() {
 void Operation::Cancel() {
   DCHECK(IsRunningInCorrectSequence());
 
-  stage_ = image_writer_api::STAGE_NONE;
+  stage_ = image_writer_api::Stage::kNone;
 
   CleanUp();
 }
@@ -86,7 +107,7 @@ void Operation::PostTask(base::OnceClosure task) {
 
 void Operation::Start() {
   DCHECK(IsRunningInCorrectSequence());
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS)
   if (download_folder_.empty() ||
       !temp_dir_->CreateUniqueTempDirUnderPath(download_folder_)) {
 #else
@@ -103,30 +124,36 @@ void Operation::Start() {
   StartImpl();
 }
 
-void Operation::OnUnzipOpenComplete(const base::FilePath& image_path) {
+void Operation::OnExtractOpenComplete(const base::FilePath& image_path) {
   DCHECK(IsRunningInCorrectSequence());
   image_path_ = image_path;
 }
 
-void Operation::Unzip(const base::Closure& continuation) {
+void Operation::Extract(base::OnceClosure continuation) {
   DCHECK(IsRunningInCorrectSequence());
   if (IsCancelled()) {
     return;
   }
 
-  if (image_path_.Extension() != FILE_PATH_LITERAL(".zip")) {
-    PostTask(continuation);
-    return;
+  if (IsArchive(image_path_)) {
+    SetStage(image_writer_api::Stage::kUnzip);
+
+    ExtractionProperties properties;
+    properties.image_path = image_path_;
+    properties.temp_dir_path = temp_dir_->GetPath();
+    properties.open_callback =
+        base::BindOnce(&Operation::OnExtractOpenComplete, this);
+    properties.complete_callback = base::BindOnce(
+        &Operation::CompleteAndContinue, this, std::move(continuation));
+    properties.failure_callback =
+        base::BindOnce(&Operation::OnExtractFailure, this);
+    properties.progress_callback =
+        base::BindRepeating(&Operation::OnExtractProgress, this);
+
+    ExtractArchive(std::move(properties));
+  } else {
+    PostTask(std::move(continuation));
   }
-
-  SetStage(image_writer_api::STAGE_UNZIP);
-
-  auto unzip_helper = base::MakeRefCounted<UnzipHelper>(
-      task_runner(), base::Bind(&Operation::OnUnzipOpenComplete, this),
-      base::Bind(&Operation::CompleteAndContinue, this, continuation),
-      base::Bind(&Operation::OnUnzipFailure, this),
-      base::Bind(&Operation::OnUnzipProgress, this));
-  unzip_helper->Unzip(image_path_, temp_dir_->GetPath());
 }
 
 void Operation::Finish() {
@@ -134,16 +161,16 @@ void Operation::Finish() {
 
   CleanUp();
 
-  base::PostTaskWithTraits(
-      FROM_HERE, {BrowserThread::UI},
+  content::GetUIThreadTaskRunner({})->PostTask(
+      FROM_HERE,
       base::BindOnce(&OperationManager::OnComplete, manager_, extension_id_));
 }
 
 void Operation::Error(const std::string& error_message) {
   DCHECK(IsRunningInCorrectSequence());
 
-  base::PostTaskWithTraits(
-      FROM_HERE, {BrowserThread::UI},
+  content::GetUIThreadTaskRunner({})->PostTask(
+      FROM_HERE,
       base::BindOnce(&OperationManager::OnError, manager_, extension_id_,
                      stage_, progress_, error_message));
 
@@ -163,10 +190,9 @@ void Operation::SetProgress(int progress) {
 
   progress_ = progress;
 
-  base::PostTaskWithTraits(
-      FROM_HERE, {BrowserThread::UI},
-      base::BindOnce(&OperationManager::OnProgress, manager_, extension_id_,
-                     stage_, progress_));
+  content::GetUIThreadTaskRunner({})->PostTask(
+      FROM_HERE, base::BindOnce(&OperationManager::OnProgress, manager_,
+                                extension_id_, stage_, progress_));
 }
 
 void Operation::SetStage(image_writer_api::Stage stage) {
@@ -178,16 +204,15 @@ void Operation::SetStage(image_writer_api::Stage stage) {
   stage_ = stage;
   progress_ = 0;
 
-  base::PostTaskWithTraits(
-      FROM_HERE, {BrowserThread::UI},
-      base::BindOnce(&OperationManager::OnProgress, manager_, extension_id_,
-                     stage_, progress_));
+  content::GetUIThreadTaskRunner({})->PostTask(
+      FROM_HERE, base::BindOnce(&OperationManager::OnProgress, manager_,
+                                extension_id_, stage_, progress_));
 }
 
 bool Operation::IsCancelled() {
   DCHECK(IsRunningInCorrectSequence());
 
-  return stage_ == image_writer_api::STAGE_NONE;
+  return stage_ == image_writer_api::Stage::kNone;
 }
 
 void Operation::AddCleanUpFunction(base::OnceClosure callback) {
@@ -195,19 +220,17 @@ void Operation::AddCleanUpFunction(base::OnceClosure callback) {
   cleanup_functions_.push_back(std::move(callback));
 }
 
-void Operation::CompleteAndContinue(const base::Closure& continuation) {
+void Operation::CompleteAndContinue(base::OnceClosure continuation) {
   DCHECK(IsRunningInCorrectSequence());
   SetProgress(kProgressComplete);
-  PostTask(continuation);
+  PostTask(std::move(continuation));
 }
 
-#if !defined(OS_CHROMEOS)
+#if !BUILDFLAG(IS_CHROMEOS)
 void Operation::StartUtilityClient() {
   DCHECK(IsRunningInCorrectSequence());
   if (!image_writer_client_.get()) {
-    // connector_ can be null in tests.
-    image_writer_client_ = ImageWriterUtilityClient::Create(
-        task_runner_, connector_ ? connector_->Clone() : nullptr);
+    image_writer_client_ = ImageWriterUtilityClient::Create(task_runner_);
     AddCleanUpFunction(base::BindOnce(&Operation::StopUtilityClient, this));
   }
 }
@@ -280,7 +303,6 @@ void Operation::MD5Chunk(
 
   CHECK_LE(bytes_processed, bytes_total);
 
-  std::unique_ptr<char[]> buffer(new char[kMD5BufferSize]);
   int read_size = std::min(bytes_total - bytes_processed,
                            static_cast<int64_t>(kMD5BufferSize));
 
@@ -290,11 +312,14 @@ void Operation::MD5Chunk(
     base::MD5Final(&digest, &md5_context_);
     std::move(callback).Run(base::MD5DigestToBase16(digest));
   } else {
-    int len = file.Read(bytes_processed, buffer.get(), read_size);
+    auto buffer = base::HeapArray<char>::Uninit(kMD5BufferSize);
+    int len =
+        file.Read(bytes_processed, base::as_writable_bytes(buffer.as_span()))
+            .value_or(0);
 
     if (len == read_size) {
       // Process data.
-      base::MD5Update(&md5_context_, base::StringPiece(buffer.get(), len));
+      base::MD5Update(&md5_context_, std::string_view(buffer.data(), len));
       int percent_curr =
           ((bytes_processed + len) * progress_scale) / bytes_total +
           progress_offset;
@@ -312,12 +337,12 @@ void Operation::MD5Chunk(
   }
 }
 
-void Operation::OnUnzipFailure(const std::string& error) {
+void Operation::OnExtractFailure(const std::string& error) {
   DCHECK(IsRunningInCorrectSequence());
   Error(error);
 }
 
-void Operation::OnUnzipProgress(int64_t total_bytes, int64_t progress_bytes) {
+void Operation::OnExtractProgress(int64_t total_bytes, int64_t progress_bytes) {
   DCHECK(IsRunningInCorrectSequence());
 
   int progress_percent = kProgressComplete * progress_bytes / total_bytes;
