@@ -1,23 +1,31 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#ifdef UNSAFE_BUFFERS_BUILD
+// TODO(crbug.com/390223051): Remove C-library calls to fix the errors.
+#pragma allow_unsafe_libc_calls
+#endif
+
 #include <memory>
 
-#include "base/macros.h"
 #include "base/memory/scoped_refptr.h"
-#include "base/test/bind_test_util.h"
+#include "base/test/bind.h"
+#include "base/test/test_future.h"
 #include "chrome/browser/extensions/extension_service_test_base.h"
 #include "chrome/test/base/testing_profile.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/test/test_storage_partition.h"
 #include "extensions/browser/api/socket/tcp_socket.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "mojo/public/cpp/bindings/remote.h"
 #include "net/base/address_list.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
-#include "net/base/test_completion_callback.h"
 #include "net/socket/socket_test_util.h"
 #include "net/traffic_annotation/network_traffic_annotation.h"
+#include "net/url_request/url_request_context.h"
+#include "net/url_request/url_request_context_builder.h"
 #include "net/url_request/url_request_test_util.h"
 #include "services/network/network_context.h"
 #include "services/network/public/mojom/network_context.mojom.h"
@@ -31,11 +39,30 @@ const int kTestMsgLength = strlen(kTestMsg);
 
 const char FAKE_ID[] = "abcdefghijklmnopqrst";
 
+using AcceptFuture = base::test::TestFuture<
+    int32_t,
+    mojo::PendingRemote<network::mojom::TCPConnectedSocket>,
+    const std::optional<net::IPEndPoint>&,
+    mojo::ScopedDataPipeConsumerHandle,
+    mojo::ScopedDataPipeProducerHandle>;
+
+using BoolFuture = base::test::TestFuture<bool>;
+
+using ConnectFuture = base::test::TestFuture<int32_t>;
+
+using ListenFuture = base::test::TestFuture<int32_t, const std::string&>;
+
+using ReadFuture =
+    base::test::TestFuture<int32_t, scoped_refptr<net::IOBuffer>, bool>;
+
+using WriteFuture = base::test::TestFuture<int32_t>;
+
 class TCPSocketUnitTestBase : public extensions::ExtensionServiceTestBase {
  public:
   TCPSocketUnitTestBase()
-      : url_request_context_(true /* delay_initialization */) {}
-  ~TCPSocketUnitTestBase() override {}
+      : url_request_context_builder_(
+            net::CreateTestURLRequestContextBuilder()) {}
+  ~TCPSocketUnitTestBase() override = default;
 
   std::unique_ptr<TCPSocket> CreateSocket() {
     auto socket = std::make_unique<TCPSocket>(&profile_, FAKE_ID);
@@ -47,9 +74,9 @@ class TCPSocketUnitTestBase : public extensions::ExtensionServiceTestBase {
       const net::IPEndPoint& ip_end_point) {
     auto socket = CreateSocket();
     net::AddressList address(ip_end_point);
-    net::TestCompletionCallback callback;
-    socket->Connect(address, callback.callback());
-    EXPECT_EQ(net::OK, callback.WaitForResult());
+    ConnectFuture connect_future;
+    socket->Connect(address, connect_future.GetCallback());
+    EXPECT_EQ(net::OK, connect_future.Get());
     return socket;
   }
 
@@ -63,21 +90,15 @@ class TCPSocketUnitTestBase : public extensions::ExtensionServiceTestBase {
     std::string received_data;
     const int count = 512;
     while (true) {
-      base::RunLoop run_loop;
-      int net_error = net::ERR_FAILED;
-      socket->Read(count,
-                   base::BindLambdaForTesting(
-                       [&](int result, scoped_refptr<net::IOBuffer> io_buffer,
-                           bool socket_destroying) {
-                         net_error = result;
-                         EXPECT_FALSE(socket_destroying);
-                         if (result > 0)
-                           received_data.append(io_buffer->data(), result);
-                         run_loop.Quit();
-                       }));
-      run_loop.Run();
-      if (net_error <= 0)
+      ReadFuture read_future;
+      socket->Read(count, read_future.GetCallback());
+      auto [net_error, io_buffer, socket_destroying] = read_future.Take();
+      EXPECT_FALSE(socket_destroying);
+      if (net_error > 0) {
+        received_data.append(io_buffer->data(), net_error);
+      } else {
         break;
+      }
     }
     EXPECT_EQ(expected_data, received_data);
   }
@@ -87,21 +108,22 @@ class TCPSocketUnitTestBase : public extensions::ExtensionServiceTestBase {
   void SetUp() override { InitializeEmptyExtensionService(); }
 
   void Initialize() {
-    url_request_context_.Init();
+    url_request_context_ = url_request_context_builder_->Build();
     network_context_ = std::make_unique<network::NetworkContext>(
-        nullptr, mojo::MakeRequest(&network_context_ptr_),
-        &url_request_context_,
+        nullptr, network_context_remote_.BindNewPipeAndPassReceiver(),
+        url_request_context_.get(),
         /*cors_exempt_header_list=*/std::vector<std::string>());
-    partition_.set_network_context(network_context_ptr_.get());
+    partition_.set_network_context(network_context_remote_.get());
   }
 
-  net::TestURLRequestContext url_request_context_;
+  std::unique_ptr<net::URLRequestContextBuilder> url_request_context_builder_;
 
  private:
   TestingProfile profile_;
+  std::unique_ptr<net::URLRequestContext> url_request_context_;
   content::TestStoragePartition partition_;
   std::unique_ptr<network::NetworkContext> network_context_;
-  network::mojom::NetworkContextPtr network_context_ptr_;
+  mojo::Remote<network::mojom::NetworkContext> network_context_remote_;
 };
 
 }  // namespace
@@ -109,13 +131,13 @@ class TCPSocketUnitTestBase : public extensions::ExtensionServiceTestBase {
 class TCPSocketUnitTest : public TCPSocketUnitTestBase,
                           public ::testing::WithParamInterface<net::IoMode> {
  public:
-  TCPSocketUnitTest() : TCPSocketUnitTestBase() {
+  TCPSocketUnitTest() {
     mock_client_socket_factory_.set_enable_read_if_ready(true);
-    url_request_context_.set_client_socket_factory(
+    url_request_context_builder_->set_client_socket_factory_for_testing(
         &mock_client_socket_factory_);
     Initialize();
   }
-  ~TCPSocketUnitTest() override {}
+  ~TCPSocketUnitTest() override = default;
 
   net::MockClientSocketFactory* mock_client_socket_factory() {
     return &mock_client_socket_factory_;
@@ -125,7 +147,7 @@ class TCPSocketUnitTest : public TCPSocketUnitTestBase,
   net::MockClientSocketFactory mock_client_socket_factory_;
 };
 
-INSTANTIATE_TEST_SUITE_P(/* no prefix */,
+INSTANTIATE_TEST_SUITE_P(All,
                          TCPSocketUnitTest,
                          testing::Values(net::SYNCHRONOUS, net::ASYNC));
 
@@ -138,10 +160,10 @@ TEST_F(TCPSocketUnitTest, SocketConnectError) {
   mock_client_socket_factory()->AddSocketDataProvider(&data_provider);
   std::unique_ptr<TCPSocket> socket = CreateSocket();
 
-  net::AddressList address(ip_end_point);
-  net::TestCompletionCallback callback;
-  socket->Connect(address, callback.callback());
-  EXPECT_EQ(net::ERR_FAILED, callback.WaitForResult());
+  ConnectFuture connect_future;
+  socket->Connect(net::AddressList(std::move(ip_end_point)),
+                  connect_future.GetCallback());
+  EXPECT_EQ(net::ERR_FAILED, connect_future.Get());
 }
 
 TEST_P(TCPSocketUnitTest, SocketConnectAfterDisconnect) {
@@ -155,11 +177,12 @@ TEST_P(TCPSocketUnitTest, SocketConnectAfterDisconnect) {
   mock_client_socket_factory()->AddSocketDataProvider(&data_provider2);
   std::unique_ptr<TCPSocket> socket = CreateAndConnectSocket();
   socket->Disconnect(false /* socket_destroying */);
-  net::TestCompletionCallback callback2;
+
+  ConnectFuture connect_future;
   net::IPEndPoint ip_end_point(net::IPAddress::IPv4Localhost(), 1234);
-  net::AddressList address(ip_end_point);
-  socket->Connect(address, callback2.callback());
-  EXPECT_EQ(net::OK, callback2.WaitForResult());
+  socket->Connect(net::AddressList(std::move(ip_end_point)),
+                  connect_future.GetCallback());
+  EXPECT_EQ(net::OK, connect_future.Get());
 
   EXPECT_TRUE(data_provider1.AllReadDataConsumed());
   EXPECT_TRUE(data_provider1.AllWriteDataConsumed());
@@ -179,12 +202,12 @@ TEST_F(TCPSocketUnitTest, SocketConnectDisconnectRace) {
   std::unique_ptr<TCPSocket> socket = CreateSocket();
 
   net::AddressList address(ip_end_point);
-  net::TestCompletionCallback callback;
-  socket->Connect(address, callback.callback());
+  ConnectFuture connect_future;
+  socket->Connect(address, connect_future.GetCallback());
   socket->Disconnect(false /* socket_destroying */);
   base::RunLoop().RunUntilIdle();
 
-  EXPECT_FALSE(callback.have_result());
+  EXPECT_FALSE(connect_future.IsReady());
 }
 
 TEST_F(TCPSocketUnitTest, DestroyWhileReadPending) {
@@ -195,26 +218,18 @@ TEST_F(TCPSocketUnitTest, DestroyWhileReadPending) {
   mock_client_socket_factory()->AddSocketDataProvider(&data_provider);
   std::unique_ptr<TCPSocket> socket = CreateAndConnectSocket();
 
-  int net_result = net::ERR_FAILED;
-  base::RunLoop run_loop;
-  int count = 1;
   // Read one byte, and it should be pending.
-  socket->Read(count,
-               base::BindLambdaForTesting(
-                   [&](int result, scoped_refptr<net::IOBuffer> io_buffer,
-                       bool socket_destroying) {
-                     net_result = result;
-                     // |socket_destroying| should correctly denote that this
-                     // read callback is invoked through the destructor of
-                     // TCPSocket.
-                     EXPECT_TRUE(socket_destroying);
-                     run_loop.Quit();
-                   }));
+  ReadFuture read_future;
+  socket->Read(/*count=*/1, read_future.GetCallback());
   // Destroy socket.
   socket = nullptr;
   // Wait for read callback.
-  run_loop.Run();
-  EXPECT_EQ(net::ERR_CONNECTION_CLOSED, net_result);
+  auto [net_error, io_buffer, socket_destroying] = read_future.Take();
+  // |socket_destroying| should correctly denote that this
+  // read callback is invoked through the destructor of
+  // TCPSocket.
+  EXPECT_TRUE(socket_destroying);
+  EXPECT_EQ(net::ERR_CONNECTION_CLOSED, net_error);
 }
 
 TEST_P(TCPSocketUnitTest, Read) {
@@ -268,21 +283,15 @@ TEST_P(TCPSocketUnitTest, SocketPartialRead) {
   int count = 1;
   std::string received_data;
   while (true) {
-    int net_result = net::ERR_FAILED;
-    base::RunLoop run_loop;
-    socket->Read(count,
-                 base::BindLambdaForTesting(
-                     [&](int result, scoped_refptr<net::IOBuffer> io_buffer,
-                         bool socket_destroying) {
-                       net_result = result;
-                       EXPECT_FALSE(socket_destroying);
-                       if (result > 0)
-                         received_data.append(io_buffer->data(), result);
-                       run_loop.Quit();
-                     }));
-    run_loop.Run();
-    if (net_result <= 0)
+    ReadFuture read_future;
+    socket->Read(count, read_future.GetCallback());
+    auto [bytes_read, io_buffer, socket_destroying] = read_future.Take();
+    EXPECT_FALSE(socket_destroying);
+    if (bytes_read > 0) {
+      received_data.append(io_buffer->data(), bytes_read);
+    } else {
       break;
+    }
     // Double the read size in the next iteration.
     count *= 2;
   }
@@ -301,34 +310,27 @@ TEST_P(TCPSocketUnitTest, ReadError) {
   mock_client_socket_factory()->AddSocketDataProvider(&data_provider);
   std::unique_ptr<TCPSocket> socket = CreateAndConnectSocket();
 
-  const int count = 512;
-  int net_error = net::OK;
+  int net_error_out = net::OK;
   while (true) {
-    base::RunLoop run_loop;
-    socket->Read(count,
-                 base::BindLambdaForTesting(
-                     [&](int result, scoped_refptr<net::IOBuffer> io_buffer,
-                         bool socket_destroying) {
-                       net_error = result;
-                       EXPECT_FALSE(socket_destroying);
-                       if (result <= 0) {
-                         EXPECT_FALSE(socket->IsConnected());
-                         EXPECT_EQ(nullptr, io_buffer);
-                       } else {
-                         EXPECT_TRUE(socket->IsConnected());
-                       }
-                       run_loop.Quit();
-                     }));
-    run_loop.Run();
-    if (net_error <= 0)
+    ReadFuture read_future;
+    socket->Read(/*count=*/512, read_future.GetCallback());
+    auto [net_error, io_buffer, socket_destroying] = read_future.Take();
+    EXPECT_FALSE(socket_destroying);
+    if (net_error <= 0) {
+      EXPECT_FALSE(socket->IsConnected());
+      EXPECT_EQ(nullptr, io_buffer);
+      net_error_out = net_error;
       break;
+    } else {
+      EXPECT_TRUE(socket->IsConnected());
+    }
   }
   // Note that TCPSocket only detects that receive pipe is broken and propagates
   // it as 0 byte read. It doesn't know the specific net error code. To know the
   // specific net error code, it needs to register itself as a
   // network::mojom::SocketObserver. However, that gets tricky because of two
   // separate mojo pipes.
-  EXPECT_EQ(0, net_error);
+  EXPECT_EQ(0, net_error_out);
   EXPECT_TRUE(data_provider.AllReadDataConsumed());
   EXPECT_TRUE(data_provider.AllWriteDataConsumed());
 }
@@ -346,9 +348,9 @@ TEST_P(TCPSocketUnitTest, Write) {
   std::unique_ptr<TCPSocket> socket = CreateAndConnectSocket();
 
   auto io_buffer = base::MakeRefCounted<net::StringIOBuffer>(kTestMsg);
-  net::TestCompletionCallback write_callback;
-  socket->Write(io_buffer.get(), kTestMsgLength, write_callback.callback());
-  EXPECT_EQ(kTestMsgLength, write_callback.WaitForResult());
+  WriteFuture write_future;
+  socket->Write(io_buffer.get(), kTestMsgLength, write_future.GetCallback());
+  EXPECT_EQ(kTestMsgLength, write_future.Get());
 }
 
 // Tests the case where a message is split over two separate socket writes.
@@ -372,10 +374,10 @@ TEST_P(TCPSocketUnitTest, MultipleWrite) {
   auto drainable_io_buffer = base::MakeRefCounted<net::DrainableIOBuffer>(
       io_buffer.get(), kTestMsgLength);
   while (num_bytes_written < kTestMsgLength) {
-    net::TestCompletionCallback write_callback;
+    WriteFuture write_future;
     socket->Write(drainable_io_buffer.get(), kTestMsgLength - num_bytes_written,
-                  write_callback.callback());
-    int result = write_callback.WaitForResult();
+                  write_future.GetCallback());
+    int result = write_future.Get();
     ASSERT_GT(result, net::OK);
     drainable_io_buffer->DidConsume(result);
     num_bytes_written += result;
@@ -406,15 +408,15 @@ TEST_P(TCPSocketUnitTest, PartialWrite) {
   auto drainable_io_buffer = base::MakeRefCounted<net::DrainableIOBuffer>(
       io_buffer.get(), kTestMsgLength);
   while (num_bytes_written < kTestMsgLength) {
-    net::TestCompletionCallback write_callback;
+    WriteFuture write_future;
     socket->Write(
         drainable_io_buffer.get(),
         std::max(kTestMsgLength - num_bytes_written, num_bytes_to_write),
-        write_callback.callback());
-    int result = write_callback.WaitForResult();
-    ASSERT_GT(result, net::OK);
-    drainable_io_buffer->DidConsume(result);
-    num_bytes_written += result;
+        write_future.GetCallback());
+    int bytes_written = write_future.Get();
+    ASSERT_GT(bytes_written, 0);
+    drainable_io_buffer->DidConsume(bytes_written);
+    num_bytes_written += bytes_written;
     num_bytes_to_write *= 2;
     // Flushes the write.
     base::RunLoop().RunUntilIdle();
@@ -440,16 +442,14 @@ TEST_P(TCPSocketUnitTest, WriteError) {
   auto io_buffer = base::MakeRefCounted<net::StringIOBuffer>(kTestMsg);
   int net_error = net::OK;
   while (true) {
-    base::RunLoop run_loop;
-    socket->Write(io_buffer.get(), kTestMsgLength,
-                  base::BindLambdaForTesting([&](int result) {
-                    EXPECT_EQ(result > 0, socket->IsConnected());
-                    net_error = result;
-                    run_loop.Quit();
-                  }));
-    run_loop.Run();
-    if (net_error <= 0)
+    WriteFuture write_future;
+    socket->Write(io_buffer.get(), kTestMsgLength, write_future.GetCallback());
+    auto bytes_written = write_future.Get();
+    EXPECT_EQ(bytes_written > 0, socket->IsConnected());
+    if (bytes_written <= 0) {
+      net_error = bytes_written;
       break;
+    }
   }
   // Note that TCPSocket only detects that send pipe is broken and propagates
   // it as a net::ERR_FAILED. It doesn't know the specific net error code. To do
@@ -470,9 +470,14 @@ class ExtensionsMockClientSocket : public net::MockTCPClientSocket {
             nullptr /* netlog */,
             provider),
         success_(success) {
-    this->set_enable_read_if_ready(true);
+    set_enable_read_if_ready(true);
   }
-  ~ExtensionsMockClientSocket() override {}
+
+  ExtensionsMockClientSocket(const ExtensionsMockClientSocket&) = delete;
+  ExtensionsMockClientSocket& operator=(const ExtensionsMockClientSocket&) =
+      delete;
+
+  ~ExtensionsMockClientSocket() override = default;
 
   bool SetNoDelay(bool no_delay) override { return success_; }
   bool SetKeepAlive(bool enable, int delay) override { return success_; }
@@ -480,8 +485,6 @@ class ExtensionsMockClientSocket : public net::MockTCPClientSocket {
  private:
   // Whether to return success for SetNoDelay() and SetKeepAlive().
   const bool success_;
-
-  DISALLOW_COPY_AND_ASSIGN(ExtensionsMockClientSocket);
 };
 
 static const net::MockRead kMockReads[] = {net::MockRead(net::ASYNC, net::OK)};
@@ -491,6 +494,10 @@ static const net::MockRead kMockReads[] = {net::MockRead(net::ASYNC, net::OK)};
 class TestSocketFactory : public net::ClientSocketFactory {
  public:
   explicit TestSocketFactory(bool success) : success_(success) {}
+
+  TestSocketFactory(const TestSocketFactory&) = delete;
+  TestSocketFactory& operator=(const TestSocketFactory&) = delete;
+
   ~TestSocketFactory() override = default;
 
   std::unique_ptr<net::DatagramClientSocket> CreateDatagramClientSocket(
@@ -503,6 +510,7 @@ class TestSocketFactory : public net::ClientSocketFactory {
   std::unique_ptr<net::TransportClientSocket> CreateTransportClientSocket(
       const net::AddressList&,
       std::unique_ptr<net::SocketPerformanceWatcher>,
+      net::NetworkQualityEstimator* network_quality_estimator,
       net::NetLog*,
       const net::NetLogSource&) override {
     providers_.push_back(std::make_unique<net::StaticSocketDataProvider>(
@@ -518,28 +526,12 @@ class TestSocketFactory : public net::ClientSocketFactory {
     NOTIMPLEMENTED();
     return nullptr;
   }
-  std::unique_ptr<net::ProxyClientSocket> CreateProxyClientSocket(
-      std::unique_ptr<net::StreamSocket> stream_socket,
-      const std::string& user_agent,
-      const net::HostPortPair& endpoint,
-      const net::ProxyServer& proxy_server,
-      net::HttpAuthController* http_auth_controller,
-      bool tunnel,
-      bool using_spdy,
-      net::NextProto negotiated_protocol,
-      net::ProxyDelegate* proxy_delegate,
-      const net::NetworkTrafficAnnotationTag& traffic_annotation) override {
-    NOTIMPLEMENTED();
-    return nullptr;
-  }
 
  private:
   std::vector<std::unique_ptr<net::StaticSocketDataProvider>> providers_;
   // Whether to return success for net::TransportClientSocket::SetNoDelay() and
   // SetKeepAlive().
   const bool success_;
-
-  DISALLOW_COPY_AND_ASSIGN(TestSocketFactory);
 };
 
 }  // namespace
@@ -547,18 +539,18 @@ class TestSocketFactory : public net::ClientSocketFactory {
 class TCPSocketSettingsTest : public TCPSocketUnitTestBase,
                               public ::testing::WithParamInterface<bool> {
  public:
-  TCPSocketSettingsTest()
-      : TCPSocketUnitTestBase(), client_socket_factory_(GetParam()) {
-    url_request_context_.set_client_socket_factory(&client_socket_factory_);
+  TCPSocketSettingsTest() : client_socket_factory_(GetParam()) {
+    url_request_context_builder_->set_client_socket_factory_for_testing(
+        &client_socket_factory_);
     Initialize();
   }
-  ~TCPSocketSettingsTest() override {}
+  ~TCPSocketSettingsTest() override = default;
 
  private:
   TestSocketFactory client_socket_factory_;
 };
 
-INSTANTIATE_TEST_SUITE_P(/* no prefix */,
+INSTANTIATE_TEST_SUITE_P(All,
                          TCPSocketSettingsTest,
                          testing::Bool());
 
@@ -566,21 +558,15 @@ TEST_P(TCPSocketSettingsTest, SetNoDelay) {
   std::unique_ptr<TCPSocket> socket = CreateAndConnectSocket();
   bool expected_success = GetParam();
   {
-    base::RunLoop run_loop;
-    socket->SetNoDelay(true, base::BindLambdaForTesting([&](bool success) {
-                         EXPECT_EQ(expected_success, success);
-                         run_loop.Quit();
-                       }));
-    run_loop.Run();
+    BoolFuture no_delay_future;
+    socket->SetNoDelay(true, no_delay_future.GetCallback());
+    EXPECT_EQ(expected_success, no_delay_future.Get());
   }
 
   {
-    base::RunLoop run_loop;
-    socket->SetNoDelay(false, base::BindLambdaForTesting([&](bool success) {
-                         EXPECT_EQ(expected_success, success);
-                         run_loop.Quit();
-                       }));
-    run_loop.Run();
+    BoolFuture no_delay_future;
+    socket->SetNoDelay(false, no_delay_future.GetCallback());
+    EXPECT_EQ(expected_success, no_delay_future.Get());
   }
 }
 
@@ -588,30 +574,24 @@ TEST_P(TCPSocketSettingsTest, SetKeepAlive) {
   std::unique_ptr<TCPSocket> socket = CreateAndConnectSocket();
   bool expected_success = GetParam();
   {
-    base::RunLoop run_loop;
-    socket->SetKeepAlive(true /* enable */, 123 /* delay */,
-                         base::BindLambdaForTesting([&](bool success) {
-                           EXPECT_EQ(expected_success, success);
-                           run_loop.Quit();
-                         }));
-    run_loop.Run();
+    BoolFuture keep_alive_future;
+    socket->SetKeepAlive(/*enable=*/true, /*delay=*/123,
+                         keep_alive_future.GetCallback());
+    EXPECT_EQ(expected_success, keep_alive_future.Get());
   }
 
   {
-    base::RunLoop run_loop;
-    socket->SetKeepAlive(false /* enable */, 123 /* delay */,
-                         base::BindLambdaForTesting([&](bool success) {
-                           EXPECT_EQ(expected_success, success);
-                           run_loop.Quit();
-                         }));
-    run_loop.Run();
+    BoolFuture keep_alive_future;
+    socket->SetKeepAlive(/*enable=*/false, /*delay=*/123,
+                         keep_alive_future.GetCallback());
+    EXPECT_EQ(expected_success, keep_alive_future.Get());
   }
 }
 
 class TCPSocketServerTest : public TCPSocketUnitTestBase {
  public:
-  TCPSocketServerTest() : TCPSocketUnitTestBase() { Initialize(); }
-  ~TCPSocketServerTest() override {}
+  TCPSocketServerTest() { Initialize(); }
+  ~TCPSocketServerTest() override = default;
 
  private:
   net::MockClientSocketFactory mock_client_socket_factory_;
@@ -620,34 +600,24 @@ class TCPSocketServerTest : public TCPSocketUnitTestBase {
 TEST_F(TCPSocketServerTest, ListenAccept) {
   // Create a server socket.
   std::unique_ptr<TCPSocket> socket = CreateSocket();
-  net::TestCompletionCallback callback;
-  base::RunLoop run_loop;
-  socket->Listen(
-      "127.0.0.1", 0 /* port */, 1 /* backlog */,
-      base::BindLambdaForTesting([&](int result, const std::string& error_msg) {
-        EXPECT_EQ(net::OK, result);
-        run_loop.Quit();
-      }));
-  run_loop.Run();
+  {
+    ListenFuture listen_future;
+    socket->Listen("127.0.0.1", 0 /* port */, 1 /* backlog */,
+                   listen_future.GetCallback());
+    EXPECT_EQ(net::OK, listen_future.Get<int32_t>());
+  }
+
   net::IPEndPoint server_addr;
   EXPECT_TRUE(socket->GetLocalAddress(&server_addr));
 
-  base::RunLoop accept_run_loop;
-  net::IPEndPoint accept_client_addr;
-  socket->Accept(base::BindLambdaForTesting(
-      [&](int result, network::mojom::TCPConnectedSocketPtr accepted_socket,
-          const base::Optional<net::IPEndPoint>& remote_addr,
-          mojo::ScopedDataPipeConsumerHandle receive_handle,
-          mojo::ScopedDataPipeProducerHandle send_handle) {
-        EXPECT_EQ(net::OK, result);
-        accept_client_addr = remote_addr.value();
-        accept_run_loop.Quit();
-      }));
-
+  AcceptFuture accept_future;
+  socket->Accept(accept_future.GetCallback());
   // Create a client socket to talk to the server socket.
   auto client_socket = CreateAndConnectSocketWithAddress(server_addr);
-  accept_run_loop.Run();
+  auto [result, connected_socket, accept_client_addr, receive_handle,
+        send_handle] = accept_future.Take();
 
+  EXPECT_EQ(result, net::OK);
   net::IPEndPoint peer_addr;
   EXPECT_TRUE(client_socket->GetPeerAddress(&peer_addr));
   net::IPEndPoint client_addr;
@@ -659,7 +629,6 @@ TEST_F(TCPSocketServerTest, ListenAccept) {
 TEST_F(TCPSocketServerTest, ListenDisconnectRace) {
   // Create a server socket.
   std::unique_ptr<TCPSocket> socket = CreateSocket();
-  net::TestCompletionCallback callback;
   bool callback_ran = false;
   socket->Listen(
       "127.0.0.1", 0 /* port */, 1 /* backlog */,
@@ -674,82 +643,64 @@ TEST_F(TCPSocketServerTest, ListenDisconnectRace) {
 TEST_F(TCPSocketServerTest, ReadAndWrite) {
   // Create a server socket.
   std::unique_ptr<TCPSocket> socket = CreateSocket();
-  net::TestCompletionCallback callback;
-  base::RunLoop run_loop;
-  socket->Listen(
-      "127.0.0.1", 0 /* port */, 1 /* backlog */,
-      base::BindLambdaForTesting([&](int result, const std::string& error_msg) {
-        EXPECT_EQ(net::OK, result);
-        run_loop.Quit();
-      }));
-  run_loop.Run();
+  {
+    ListenFuture listen_future;
+    socket->Listen("127.0.0.1", 0 /* port */, 1 /* backlog */,
+                   listen_future.GetCallback());
+    EXPECT_EQ(net::OK, listen_future.Get<int32_t>());
+  }
   net::IPEndPoint server_addr;
   EXPECT_TRUE(socket->GetLocalAddress(&server_addr));
 
-  base::RunLoop accept_run_loop;
-  std::unique_ptr<TCPSocket> accepted_socket;
-
-  socket->Accept(base::BindLambdaForTesting(
-      [&](int result, network::mojom::TCPConnectedSocketPtr connected_socket,
-          const base::Optional<net::IPEndPoint>& remote_addr,
-          mojo::ScopedDataPipeConsumerHandle receive_handle,
-          mojo::ScopedDataPipeProducerHandle send_handle) {
-        EXPECT_EQ(net::OK, result);
-        accepted_socket = std::make_unique<TCPSocket>(
-            std::move(connected_socket), std::move(receive_handle),
-            std::move(send_handle), remote_addr, FAKE_ID);
-        accept_run_loop.Quit();
-      }));
+  AcceptFuture accept_future;
+  socket->Accept(accept_future.GetCallback());
 
   // Create a client socket to talk to the server socket.
   auto client_socket = CreateAndConnectSocketWithAddress(server_addr);
-  net::TestCompletionCallback connect_callback;
-  accept_run_loop.Run();
+
+  auto [result, connected_socket, remote_addr, receive_handle, send_handle] =
+      accept_future.Take();
+  EXPECT_EQ(net::OK, result);
+  auto accepted_socket = std::make_unique<TCPSocket>(
+      std::move(connected_socket), std::move(receive_handle),
+      std::move(send_handle), remote_addr, FAKE_ID);
 
   // Send data from the client to the server.
-  auto io_buffer = base::MakeRefCounted<net::StringIOBuffer>(kTestMsg);
-  net::TestCompletionCallback write_callback;
-  client_socket->Write(io_buffer.get(), kTestMsgLength,
-                       write_callback.callback());
-  EXPECT_EQ(kTestMsgLength, write_callback.WaitForResult());
+  auto io_buffer_write = base::MakeRefCounted<net::StringIOBuffer>(kTestMsg);
+  {
+    WriteFuture write_future;
+    client_socket->Write(io_buffer_write.get(), kTestMsgLength,
+                         write_future.GetCallback());
+    EXPECT_EQ(kTestMsgLength, write_future.Get());
+  }
 
   std::string received_contents;
   while (received_contents.size() < kTestMsgLength) {
-    base::RunLoop run_loop;
-    accepted_socket->Read(
-        kTestMsgLength,
-        base::BindLambdaForTesting([&](int result,
-                                       scoped_refptr<net::IOBuffer> io_buffer,
-                                       bool socket_destroying) {
-          ASSERT_GT(result, 0);
-          EXPECT_FALSE(socket_destroying);
-          received_contents.append(std::string(io_buffer->data(), result));
-          run_loop.Quit();
-        }));
-    run_loop.Run();
+    ReadFuture read_future;
+    accepted_socket->Read(kTestMsgLength, read_future.GetCallback());
+    auto [bytes_read, io_buffer_read, socket_destroying] = read_future.Take();
+    ASSERT_GT(bytes_read, 0);
+    EXPECT_FALSE(socket_destroying);
+    received_contents.append(std::string(io_buffer_read->data(), bytes_read));
   }
   EXPECT_EQ(kTestMsg, received_contents);
 
   // Send data from the server to the client.
-  net::TestCompletionCallback write_callback2;
-  accepted_socket->Write(io_buffer.get(), kTestMsgLength,
-                         write_callback2.callback());
-  EXPECT_EQ(kTestMsgLength, write_callback2.WaitForResult());
+  {
+    WriteFuture write_future;
+    accepted_socket->Write(io_buffer_write.get(), kTestMsgLength,
+                           write_future.GetCallback());
+    EXPECT_EQ(kTestMsgLength, write_future.Get());
+  }
 
   std::string sent_contents;
   while (sent_contents.size() < kTestMsgLength) {
-    base::RunLoop run_loop;
-    client_socket->Read(
-        kTestMsgLength,
-        base::BindLambdaForTesting([&](int result,
-                                       scoped_refptr<net::IOBuffer> io_buffer,
-                                       bool socket_destroying) {
-          ASSERT_GT(result, 0);
-          EXPECT_FALSE(socket_destroying);
-          sent_contents.append(std::string(io_buffer->data(), result));
-          run_loop.Quit();
-        }));
-    run_loop.Run();
+    ReadFuture read_future;
+    client_socket->Read(kTestMsgLength, read_future.GetCallback());
+    auto [bytes_read, io_buffer_read, socket_destroying] = read_future.Take();
+    ASSERT_GT(bytes_read, 0);
+    EXPECT_FALSE(socket_destroying);
+    sent_contents.append(std::string(io_buffer_read->data(), bytes_read));
   }
   EXPECT_EQ(kTestMsg, sent_contents);
 }
