@@ -347,13 +347,9 @@ void MediaStreamCaptureIndicator::WebContentsDeviceUsage::AddDevices(
     user_media_stop_callbacks_[stop_callback_id] = std::move(stop_callback);
   }
 
-  // TODO(crbug.com/40071631): Don't turn on this until related bugs are fixed.
-  // This may record the same stop_callback twice and lead to a crash if
-  // called later on.
-  // if (type == MediaType::kDisplayMedia) {
-  //     display_media_stop_callbacks_[stop_callback_id] =
-  //     std::move(stop_callback);
-  //   }
+  if (type == MediaType::kDisplayMedia) {
+    display_media_stop_callbacks_[stop_callback_id] = std::move(stop_callback);
+  }
 
   if (web_contents()) {
     web_contents()->NotifyNavigationStateChanged(content::INVALIDATE_TYPE_TAB);
@@ -679,8 +675,10 @@ void MediaStreamCaptureIndicator::MaybeDestroyStatusTrayIcon() {
 
   StatusTray* status_tray = g_browser_process->status_tray();
   if (status_tray != nullptr) {
-    status_tray->RemoveStatusIcon(status_icon_);
+    std::unique_ptr<StatusIcon> removed_icon =
+        status_tray->RemoveStatusIcon(status_icon_);
     status_icon_ = nullptr;
+    removed_icon.reset();
   }
 }
 
@@ -742,7 +740,7 @@ void MediaStreamCaptureIndicator::GetStatusTrayIconInfo(
     gfx::ImageSkia* image,
     std::u16string* tool_tip) {
 #if BUILDFLAG(IS_ANDROID)
-  NOTREACHED_IN_MIGRATION();
+  NOTREACHED();
 #else   // !BUILDFLAG(IS_ANDROID)
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   DCHECK(audio || video);
