@@ -2,29 +2,23 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import {assert} from 'chrome://resources/js/assert_ts.js';
+import {assert} from 'chrome://resources/js/assert.js';
 import {EventTracker} from 'chrome://resources/js/event_tracker.js';
 import {loadTimeData} from 'chrome://resources/js/load_time_data.js';
 
-import {CapabilitiesResponse, NativeLayer, NativeLayerImpl} from '../native_layer.js';
-// <if expr="is_chromeos">
-import {NativeLayerCros, NativeLayerCrosImpl, PrinterSetupResponse} from '../native_layer_cros.js';
+import type {CapabilitiesResponse, NativeLayer} from '../native_layer.js';
+import {NativeLayerImpl} from '../native_layer.js';
+import type {Cdd, MediaSizeOption} from './cdd.js';
+
+import type {RecentDestination} from './destination.js';
+import {createDestinationKey, createRecentDestinationKey, Destination, DestinationOrigin, GooglePromotedDestinationId, isPdfPrinter, PDF_DESTINATION_KEY, PrinterType} from './destination.js';
 
-// </if>
-import {Cdd, MediaSizeOption} from './cdd.js';
-import {createDestinationKey, createRecentDestinationKey, Destination, DestinationOrigin, GooglePromotedDestinationId, isPdfPrinter, PDF_DESTINATION_KEY, PrinterType, RecentDestination} from './destination.js';
-// <if expr="is_chromeos">
-import {DestinationProvisionalType} from './destination.js';
-// </if>
 import {DestinationMatch} from './destination_match.js';
-import {ExtensionDestinationInfo, LocalDestinationInfo, parseDestination} from './local_parsers.js';
-// <if expr="is_chromeos">
-import {parseExtensionDestination} from './local_parsers.js';
-// </if>
+import type {ExtensionDestinationInfo, LocalDestinationInfo} from './local_parsers.js';
+import {parseDestination} from './local_parsers.js';
 
 /**
  * Printer search statuses used by the destination store.
- * @enum {string}
  */
 enum DestinationStorePrinterSearchStatus {
   START = 'start',
@@ -143,7 +137,6 @@ function sortMediaSizes(capabilities: Cdd): Cdd {
 
 /**
  * Event types dispatched by the destination store.
- * @enum {string}
  */
 export enum DestinationStoreEventType {
   DESTINATION_SEARCH_DONE = 'DestinationStore.DESTINATION_SEARCH_DONE',
@@ -154,6 +147,8 @@ export enum DestinationStoreEventType {
       '.SELECTED_DESTINATION_CAPABILITIES_READY',
   // <if expr="is_chromeos">
   DESTINATION_EULA_READY = 'DestinationStore.DESTINATION_EULA_READY',
+  DESTINATION_PRINTER_STATUS_UPDATE =
+      'DestinationStore.DESTINATION_PRINTER_STATUS_UPDATE',
   // </if>
 }
 
@@ -234,11 +229,7 @@ export class DestinationStore extends EventTarget {
    */
   constructor(
       addListenerCallback:
-          (eventName: string,
-           listener:
-               (t: PrinterType,
-                p: LocalDestinationInfo[]|
-                ExtensionDestinationInfo[]) => void) => void) {
+          (eventName: string, listener: (p1: any, p2?: any) => void) => void) {
     super();
 
     this.destinationSearchStatus_ = new Map([
@@ -257,6 +248,13 @@ export class DestinationStore extends EventTarget {
         (type: PrinterType,
          printers: LocalDestinationInfo[]|ExtensionDestinationInfo[]) =>
             this.onPrintersAdded_(type, printers));
+
+    // <if expr="is_chromeos">
+    addListenerCallback(
+        'local-printers-updated',
+        (printers: LocalDestinationInfo[]) =>
+            this.onLocalPrintersUpdated_(printers));
+    // </if>
   }
 
   /**
@@ -302,8 +300,8 @@ export class DestinationStore extends EventTarget {
    * will be automatically selected.
    * @param pdfPrinterDisabled Whether the PDF print destination is
    *     disabled in print preview.
-   * @param isDriveMounted Whether Google Drive is mounted. Only used
-        on Chrome OS.
+   * @param saveToDriveDisabled Whether the 'Save to Google Drive' destination
+   *     is disabled in print preview. Only used on Chrome OS.
    * @param systemDefaultDestinationId ID of the system default
    *     destination.
    * @param serializedDefaultDestinationSelectionRulesStr Serialized
@@ -313,10 +311,10 @@ export class DestinationStore extends EventTarget {
   init(
       pdfPrinterDisabled: boolean,
       // <if expr="is_chromeos">
-      isDriveMounted: boolean,
+      saveToDriveDisabled: boolean,
       // </if>
       // <if expr="not is_chromeos">
-      _isDriveMounted: boolean,
+      _saveToDriveDisabled: boolean,
       // </if>
       systemDefaultDestinationId: string,
       serializedDefaultDestinationSelectionRulesStr: string|null,
@@ -355,7 +353,7 @@ export class DestinationStore extends EventTarget {
     this.pdfPrinterEnabled_ = !pdfPrinterDisabled;
     this.createLocalPdfPrintDestination_();
     // <if expr="is_chromeos">
-    if (isDriveMounted) {
+    if (!saveToDriveDisabled) {
       this.createLocalDrivePrintDestination_();
     }
     // </if>
@@ -364,6 +362,11 @@ export class DestinationStore extends EventTarget {
     // destinationsInserted_ may never be called.
     if (this.typesToSearch_.size === 0) {
       this.tryToSelectInitialDestination_();
+      // <if expr="is_chromeos">
+      // Start observing local printers if there is no attempt to load
+      // destinations.
+      this.observeLocalPrinters_();
+      // </if>
       return;
     }
 
@@ -580,12 +583,26 @@ export class DestinationStore extends EventTarget {
   }
 
   /**
-   * @param Destination to select.
+   * @param destination Destination to select.
+   * @param refreshDestination Set to true to allow the currently selected
+   *          destination to be re-selected.
    */
-  selectDestination(destination: Destination) {
+  selectDestination(
+      destination: Destination, refreshDestination: boolean = false) {
+    // <if expr="not is_chromeos">
+    assert(!refreshDestination, 'refreshDestination for CrOS only');
     if (destination === this.selectedDestination_) {
       return;
     }
+    // </if>
+    // <if expr="is_chromeos">
+    // Do not re-select the same destination unless explicitly requesting it to
+    // refetch the capabilities and reload the preview.
+    if (destination === this.selectedDestination_ && !refreshDestination) {
+      return;
+    }
+    // </if>
+
     if (destination === null) {
       this.selectedDestination_ = null;
       this.dispatchEvent(
@@ -866,6 +883,10 @@ export class DestinationStore extends EventTarget {
     } else if (this.typesToSearch_.size === 0) {
       this.tryToSelectInitialDestination_();
     }
+
+    // <if expr="is_chromeos">
+    this.observeLocalPrinters_();
+    // </if>
   }
 
   /**
@@ -946,6 +967,70 @@ export class DestinationStore extends EventTarget {
         (printer: LocalDestinationInfo|ExtensionDestinationInfo) =>
             parseDestination(type, printer)));
   }
+
+  // <if expr="is_chromeos">
+  private observeLocalPrinters_() {
+    this.nativeLayerCros_.observeLocalPrinters().then(
+        (printers: LocalDestinationInfo[]) =>
+            this.onLocalPrintersUpdated_(printers));
+  }
+
+  /**
+   * Inserts any new printers retrieved from the 'local-printers-updated' event.
+   * @param printerType The type of printer(s) added.
+   * @param printers Information about the printers that have been retrieved.
+   */
+  private onLocalPrintersUpdated_(printers: LocalDestinationInfo[]) {
+    if (!printers) {
+      return;
+    }
+
+    // The logic in insertDestinations_() ensures only new destinations are
+    // added to the store.
+    this.insertDestinations_(printers.map(
+        printer => parseDestination(PrinterType.LOCAL_PRINTER, printer)));
+
+    // Parse the printer status from the LocalDestinationInfo object.
+    for (const printer of printers) {
+      this.parsePrinterStatus(printer);
+    }
+  }
+
+  // Updates the printer status for an existing destination then fires an event
+  // for updating printer status icons and text.
+  private parsePrinterStatus(destinationInfo: LocalDestinationInfo): void {
+    const printerStatus = destinationInfo.printerStatus;
+    if (!printerStatus || !printerStatus.printerId) {
+      return;
+    }
+
+    const destinationKey = createDestinationKey(
+        destinationInfo.deviceName, DestinationOrigin.CROS);
+    const existingDestination = this.destinationMap_.get(destinationKey);
+    if (existingDestination === undefined) {
+      return;
+    }
+
+    // `nowOnline` captures the event where a previously offline printer
+    // becomes reachable. This will be used to trigger the destination to
+    // reload its preview.
+    const previousStatusReason = existingDestination.printerStatusReason;
+    const nextStatusReason = getStatusReasonFromPrinterStatus(printerStatus);
+    const nowOnline =
+        previousStatusReason === PrinterStatusReason.PRINTER_UNREACHABLE &&
+        (nextStatusReason !== PrinterStatusReason.PRINTER_UNREACHABLE &&
+         nextStatusReason !== PrinterStatusReason.UNKNOWN_REASON);
+
+    existingDestination.printerStatusReason = nextStatusReason;
+    this.dispatchEvent(new CustomEvent(
+        DestinationStoreEventType.DESTINATION_PRINTER_STATUS_UPDATE, {
+          detail: {
+            destinationKey: destinationKey,
+            nowOnline: nowOnline,
+          },
+        }));
+  }
+  // </if>
 }
 
 /**
