@@ -19,7 +19,6 @@
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
-#include "chrome/browser/ash/mobile/mobile_activator.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/url_constants.h"
@@ -50,61 +49,11 @@ namespace {
 // Host page JS API function names.
 const char kJsGetDeviceInfo[] = "getDeviceInfo";
 
-const char kJsDeviceStatusChangedCallback[] =
-    "mobile.MobileSetup.deviceStateChanged";
 const char kJsGetDeviceInfoCallback[] =
     "mobile.MobileSetupPortal.onGotDeviceInfo";
 const char kJsConnectivityChangedCallback[] =
     "mobile.MobileSetupPortal.onConnectivityChanged";
 
-// TODO(tbarzic): Localize these strings.
-const char16_t kDefaultActivationError[] =
-    u"$1 is unable to connect to $2 at this time. Please try again later.";
-const char16_t kCellularDisabledError[] =
-    u"Mobile network connections are not currently enabled on this device.";
-const char16_t kNoCellularDeviceError[] =
-    u"Mobile network modem is not present.";
-const char16_t kNoCellularServiceError[] =
-    u"$1 is unable to connect at this time due to insufficient coverage.";
-
-bool ActivationErrorRequiresCarrier(MobileActivator::ActivationError error) {
-  return error == MobileActivator::ActivationError::kActivationFailed;
-}
-
-std::u16string GetActivationErrorMessage(MobileActivator::ActivationError error,
-                                         const std::string& carrier) {
-  // If the activation error message requires the carrier name, and none was
-  // provider, fallback to kNoCellularServiceError.
-  if (carrier.empty() && ActivationErrorRequiresCarrier(error)) {
-    CHECK(!ActivationErrorRequiresCarrier(
-        MobileActivator::ActivationError::kNoCellularService));
-    return GetActivationErrorMessage(
-        MobileActivator::ActivationError::kNoCellularService, carrier);
-  }
-
-  switch (error) {
-    case MobileActivator::ActivationError::kNone:
-      return std::u16string();
-    case MobileActivator::ActivationError::kActivationFailed: {
-      return base::ReplaceStringPlaceholders(
-          kDefaultActivationError,
-          std::vector<std::u16string>(
-              {ui::GetChromeOSDeviceName(), base::UTF8ToUTF16(carrier)}),
-          nullptr);
-    }
-    case MobileActivator::ActivationError::kCellularDisabled:
-      return kCellularDisabledError;
-    case MobileActivator::ActivationError::kNoCellularDevice:
-      return kNoCellularDeviceError;
-    case MobileActivator::ActivationError::kNoCellularService:
-      return base::ReplaceStringPlaceholders(
-          kNoCellularServiceError, ui::GetChromeOSDeviceName(), nullptr);
-  }
-  NOTREACHED_IN_MIGRATION() << "Unexpected activation error";
-  return GetActivationErrorMessage(
-      MobileActivator::ActivationError::kActivationFailed, carrier);
-}
-
 void DataRequestFailed(const std::string& service_path,
                        content::URLDataSource::GotDataCallback callback) {
   NET_LOG(ERROR) << "Data Request Failed for Mobile Setup: "
@@ -117,10 +66,6 @@ void DataRequestFailed(const std::string& service_path,
 // cellular network information.
 namespace keys {
 
-// The current activation state:
-constexpr char kActivationState[] = "state";
-constexpr char kActivationErrorMessage[] = "error";
-
 // The cellular service properties:
 constexpr char kCellularActivationType[] = "activation_type";
 constexpr char kCarrier[] = "carrier";
@@ -142,8 +87,9 @@ constexpr char kMdn[] = "MDN";
 base::Value GetCellularNetworkInfoValue(const NetworkState* network,
                                         const DeviceState* device) {
   base::Value::Dict info;
-  if (!device || !network)
+  if (!device || !network) {
     return base::Value(std::move(info));
+  }
 
   DCHECK_EQ(network->device_path(), device->path());
 
@@ -169,7 +115,7 @@ class MobileSetupUIHTMLSource : public content::URLDataSource {
   MobileSetupUIHTMLSource(const MobileSetupUIHTMLSource&) = delete;
   MobileSetupUIHTMLSource& operator=(const MobileSetupUIHTMLSource&) = delete;
 
-  ~MobileSetupUIHTMLSource() override {}
+  ~MobileSetupUIHTMLSource() override = default;
 
   // content::URLDataSource implementation.
   std::string GetSource() override;
@@ -191,7 +137,6 @@ class MobileSetupUIHTMLSource : public content::URLDataSource {
 
 // The handler for Javascript messages related to the "register" view.
 class MobileSetupHandler : public content::WebUIMessageHandler,
-                           public MobileActivator::Observer,
                            public NetworkStateHandlerObserver {
  public:
   MobileSetupHandler();
@@ -205,17 +150,9 @@ class MobileSetupHandler : public content::WebUIMessageHandler,
   void RegisterMessages() override;
   void OnJavascriptDisallowed() override;
 
-  // MobileActivator::Observer.
-  void OnActivationStateChanged(
-      const NetworkState* network,
-      MobileActivator::PlanActivationState new_state,
-      MobileActivator::ActivationError error) override;
-
  private:
   enum Type {
     TYPE_UNDETERMINED,
-    // The network is not yet activated, and the webui is in activation flow.
-    TYPE_ACTIVATION,
     // The network is activated, the webui displays network portal.
     TYPE_PORTAL,
     // Same as TYPE_PORTAL, but the network technology is LTE. The webui is
@@ -260,7 +197,7 @@ class MobileSetupHandler : public content::WebUIMessageHandler,
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-MobileSetupUIHTMLSource::MobileSetupUIHTMLSource() {}
+MobileSetupUIHTMLSource::MobileSetupUIHTMLSource() = default;
 
 std::string MobileSetupUIHTMLSource::GetSource() {
   return chrome::kChromeUIMobileSetupHost;
@@ -346,46 +283,17 @@ MobileSetupHandler::~MobileSetupHandler() {
   Reset();
 }
 
-void MobileSetupHandler::OnActivationStateChanged(
-    const NetworkState* network,
-    MobileActivator::PlanActivationState state,
-    MobileActivator::ActivationError error) {
-  DCHECK_EQ(TYPE_ACTIVATION, type_);
-  if (!web_ui())
-    return;
-
-  NetworkStateHandler* network_state_handler =
-      NetworkHandler::Get()->network_state_handler();
-  const DeviceState* device =
-      network ? network_state_handler->GetDeviceState(network->device_path())
-              : nullptr;
-
-  // First generate cellular properties dictionary, if cellular service and
-  // device are available.
-  base::Value info = GetCellularNetworkInfoValue(network, device);
-
-  // Add the current activation flow state.
-  info.GetDict().Set(keys::kActivationState, static_cast<int>(state));
-  info.GetDict().Set(
-      keys::kActivationErrorMessage,
-      GetActivationErrorMessage(error, device ? device->operator_name() : ""));
-
-  CallJavascriptFunction(kJsDeviceStatusChangedCallback, info);
-}
-
 void MobileSetupHandler::OnJavascriptDisallowed() {
   Reset();
 }
 
 void MobileSetupHandler::Reset() {
-  if (!active_)
+  if (!active_) {
     return;
+  }
   active_ = false;
 
-  if (type_ == TYPE_ACTIVATION) {
-    MobileActivator::GetInstance()->RemoveObserver(this);
-    MobileActivator::GetInstance()->TerminateActivation();
-  } else if (type_ == TYPE_PORTAL_LTE) {
+  if (type_ == TYPE_PORTAL_LTE) {
     network_state_handler_observer_.Reset();
   }
 }
@@ -398,13 +306,14 @@ void MobileSetupHandler::RegisterMessages() {
 }
 
 void MobileSetupHandler::HandleGetDeviceInfo(const base::Value::List& args) {
-  DCHECK_NE(TYPE_ACTIVATION, type_);
-  if (!web_ui())
+  if (!web_ui()) {
     return;
+  }
 
   std::string path = web_ui()->GetWebContents()->GetURL().path();
-  if (path.empty())
+  if (path.empty()) {
     return;
+  }
 
   active_ = true;
   AllowJavascript();
@@ -447,12 +356,14 @@ void MobileSetupHandler::HandleGetDeviceInfo(const base::Value::List& args) {
 
 void MobileSetupHandler::DefaultNetworkChanged(
     const NetworkState* default_network) {
-  if (!web_ui())
+  if (!web_ui()) {
     return;
+  }
 
   std::string path = web_ui()->GetWebContents()->GetURL().path().substr(1);
-  if (path.empty())
+  if (path.empty()) {
     return;
+  }
 
   const NetworkState* network =
       NetworkHandler::Get()->network_state_handler()->GetNetworkState(path);
@@ -467,12 +378,14 @@ void MobileSetupHandler::DefaultNetworkChanged(
 
 void MobileSetupHandler::NetworkConnectionStateChanged(
     const NetworkState* network) {
-  if (!web_ui())
+  if (!web_ui()) {
     return;
+  }
 
   std::string path = web_ui()->GetWebContents()->GetURL().path().substr(1);
-  if (path.empty() || path != network->path())
+  if (path.empty() || path != network->path()) {
     return;
+  }
 
   UpdatePortalReachability(network, false /* do not force notification */);
 }
