@@ -7,21 +7,29 @@
 #include <string>
 #include <utility>
 
+#include "base/check_deref.h"
 #include "base/functional/bind.h"
 #include "base/functional/callback_helpers.h"
 #include "base/metrics/histogram_functions.h"
 #include "chrome/browser/ash/bruschetta/bruschetta_service.h"
+#include "chrome/browser/ash/bruschetta/bruschetta_service_factory.h"
 #include "chrome/browser/ash/bruschetta/bruschetta_util.h"
 #include "chrome/browser/ash/crostini/crostini_disk.h"
+#include "chrome/browser/ash/crostini/crostini_export_import.h"
+#include "chrome/browser/ash/crostini/crostini_export_import_factory.h"
 #include "chrome/browser/ash/crostini/crostini_features.h"
 #include "chrome/browser/ash/crostini/crostini_installer.h"
+#include "chrome/browser/ash/crostini/crostini_installer_factory.h"
 #include "chrome/browser/ash/crostini/crostini_port_forwarder.h"
+#include "chrome/browser/ash/crostini/crostini_port_forwarder_factory.h"
 #include "chrome/browser/ash/crostini/crostini_pref_names.h"
 #include "chrome/browser/ash/crostini/crostini_shared_devices.h"
+#include "chrome/browser/ash/crostini/crostini_shared_devices_factory.h"
 #include "chrome/browser/ash/crostini/crostini_types.mojom.h"
 #include "chrome/browser/ash/crostini/crostini_util.h"
 #include "chrome/browser/ash/guest_os/guest_os_pref_names.h"
 #include "chrome/browser/ash/guest_os/guest_os_session_tracker.h"
+#include "chrome/browser/ash/guest_os/guest_os_session_tracker_factory.h"
 #include "chrome/browser/ash/guest_os/guest_os_terminal.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/lifetime/application_lifetime.h"
@@ -210,16 +218,16 @@ void CrostiniHandler::RegisterMessages() {
         base::BindRepeating(&CrostiniHandler::HandleSetVmDeviceShared,
                             handler_weak_ptr_factory_.GetWeakPtr()));
   }
-    web_ui()->RegisterMessageCallback(
-        "requestBruschettaInstallerView",
-        base::BindRepeating(
-            &CrostiniHandler::HandleRequestBruschettaInstallerView,
-            handler_weak_ptr_factory_.GetWeakPtr()));
-    web_ui()->RegisterMessageCallback(
-        "requestBruschettaUninstallerView",
-        base::BindRepeating(
-            &CrostiniHandler::HandleRequestBruschettaUninstallerView,
-            handler_weak_ptr_factory_.GetWeakPtr()));
+  web_ui()->RegisterMessageCallback(
+      "requestBruschettaInstallerView",
+      base::BindRepeating(
+          &CrostiniHandler::HandleRequestBruschettaInstallerView,
+          handler_weak_ptr_factory_.GetWeakPtr()));
+  web_ui()->RegisterMessageCallback(
+      "requestBruschettaUninstallerView",
+      base::BindRepeating(
+          &CrostiniHandler::HandleRequestBruschettaUninstallerView,
+          handler_weak_ptr_factory_.GetWeakPtr()));
 }
 
 void CrostiniHandler::OnJavascriptAllowed() {
@@ -227,9 +235,11 @@ void CrostiniHandler::OnJavascriptAllowed() {
   crostini_manager->AddCrostiniDialogStatusObserver(this);
   crostini_manager->AddCrostiniContainerPropertiesObserver(this);
   crostini_manager->AddContainerShutdownObserver(this);
-  crostini::CrostiniExportImport::GetForProfile(profile_)->AddObserver(this);
-  crostini::CrostiniPortForwarder::GetForProfile(profile_)->AddObserver(this);
-  guest_os::GuestOsSessionTracker::GetForProfile(profile_)
+  crostini::CrostiniExportImportFactory::GetForProfile(profile_)->AddObserver(
+      this);
+  crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)->AddObserver(
+      this);
+  guest_os::GuestOsSessionTrackerFactory::GetForProfile(profile_)
       ->AddContainerStartedObserver(this);
 
   // Observe ADB sideloading device policy and react to its changes
@@ -259,10 +269,11 @@ void CrostiniHandler::OnJavascriptDisallowed() {
   crostini_manager->RemoveCrostiniDialogStatusObserver(this);
   crostini_manager->RemoveCrostiniContainerPropertiesObserver(this);
   crostini_manager->RemoveContainerShutdownObserver(this);
-  crostini::CrostiniExportImport::GetForProfile(profile_)->RemoveObserver(this);
-  crostini::CrostiniPortForwarder::GetForProfile(profile_)->RemoveObserver(
-      this);
-  guest_os::GuestOsSessionTracker::GetForProfile(profile_)
+  crostini::CrostiniExportImportFactory::GetForProfile(profile_)
+      ->RemoveObserver(this);
+  crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)
+      ->RemoveObserver(this);
+  guest_os::GuestOsSessionTrackerFactory::GetForProfile(profile_)
       ->RemoveContainerStartedObserver(this);
 
   adb_sideloading_device_policy_subscription_ = {};
@@ -273,7 +284,8 @@ void CrostiniHandler::OnJavascriptDisallowed() {
 void CrostiniHandler::HandleRequestCrostiniInstallerView(
     const base::Value::List& args) {
   AllowJavascript();
-  crostini::CrostiniInstaller::GetForProfile(Profile::FromWebUI(web_ui()))
+  crostini::CrostiniInstallerFactory::GetForProfile(
+      Profile::FromWebUI(web_ui()))
       ->ShowDialog(crostini::CrostiniUISurface::kSettings);
 }
 
@@ -316,8 +328,8 @@ void CrostiniHandler::HandleExportCrostiniContainer(
   guest_os::GuestId container_id(args[0]);
   VLOG(1) << "Exporting  = " << container_id;
 
-  crostini::CrostiniExportImport::GetForProfile(profile_)->ExportContainer(
-      container_id, web_ui()->GetWebContents());
+  crostini::CrostiniExportImportFactory::GetForProfile(profile_)
+      ->ExportContainer(container_id, web_ui()->GetWebContents());
 }
 
 void CrostiniHandler::HandleImportCrostiniContainer(
@@ -325,8 +337,8 @@ void CrostiniHandler::HandleImportCrostiniContainer(
   CHECK_EQ(1U, args.size());
   guest_os::GuestId container_id(args[0]);
   VLOG(1) << "Importing  = " << container_id;
-  crostini::CrostiniExportImport::GetForProfile(profile_)->ImportContainer(
-      container_id, web_ui()->GetWebContents());
+  crostini::CrostiniExportImportFactory::GetForProfile(profile_)
+      ->ImportContainer(container_id, web_ui()->GetWebContents());
 }
 
 void CrostiniHandler::HandleCrostiniInstallerStatusRequest(
@@ -342,8 +354,9 @@ void CrostiniHandler::HandleCrostiniExportImportOperationStatusRequest(
     const base::Value::List& args) {
   AllowJavascript();
   CHECK_EQ(0U, args.size());
-  bool in_progress = crostini::CrostiniExportImport::GetForProfile(profile_)
-                         ->GetExportImportOperationStatus();
+  bool in_progress =
+      crostini::CrostiniExportImportFactory::GetForProfile(profile_)
+          ->GetExportImportOperationStatus();
   OnCrostiniExportImportOperationStatusChanged(in_progress);
 }
 
@@ -368,8 +381,7 @@ void CrostiniHandler::OnCrostiniDialogStatusChanged(
                           base::Value(status));
         break;
       default:
-        NOTREACHED_IN_MIGRATION();
-        break;
+        NOTREACHED();
     }
   }
 }
@@ -548,7 +560,7 @@ void CrostiniHandler::HandleAddCrostiniPortForward(
     return;
   }
 
-  crostini::CrostiniPortForwarder::GetForProfile(profile_)->AddPort(
+  crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)->AddPort(
       container_id, port_number,
       static_cast<crostini::CrostiniPortForwarder::Protocol>(protocol_type),
       std::move(label),
@@ -572,7 +584,7 @@ void CrostiniHandler::HandleRemoveCrostiniPortForward(
     return;
   }
 
-  crostini::CrostiniPortForwarder::GetForProfile(profile_)->RemovePort(
+  crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)->RemovePort(
       container_id, port_number,
       static_cast<crostini::CrostiniPortForwarder::Protocol>(protocol_type),
       base::BindOnce(&CrostiniHandler::OnPortForwardComplete,
@@ -588,8 +600,8 @@ void CrostiniHandler::HandleRemoveAllCrostiniPortForwards(
     return;
   }
 
-  crostini::CrostiniPortForwarder::GetForProfile(profile_)->RemoveAllPorts(
-      guest_os::GuestId(args[0]));
+  crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)
+      ->RemoveAllPorts(guest_os::GuestId(args[0]));
 }
 
 void CrostiniHandler::HandleActivateCrostiniPortForward(
@@ -607,7 +619,7 @@ void CrostiniHandler::HandleActivateCrostiniPortForward(
     return;
   }
 
-  crostini::CrostiniPortForwarder::GetForProfile(profile_)->ActivatePort(
+  crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)->ActivatePort(
       container_id, port_number,
       static_cast<crostini::CrostiniPortForwarder::Protocol>(protocol_type),
       base::BindOnce(&CrostiniHandler::OnPortForwardComplete,
@@ -630,12 +642,13 @@ void CrostiniHandler::HandleDeactivateCrostiniPortForward(
     return;
   }
 
-  crostini::CrostiniPortForwarder::GetForProfile(profile_)->DeactivatePort(
-      container_id, port_number,
-      static_cast<crostini::CrostiniPortForwarder::Protocol>(protocol_type),
-      base::BindOnce(&CrostiniHandler::OnPortForwardComplete,
-                     callback_weak_ptr_factory_.GetWeakPtr(),
-                     std::move(callback_id)));
+  crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)
+      ->DeactivatePort(
+          container_id, port_number,
+          static_cast<crostini::CrostiniPortForwarder::Protocol>(protocol_type),
+          base::BindOnce(&CrostiniHandler::OnPortForwardComplete,
+                         callback_weak_ptr_factory_.GetWeakPtr(),
+                         std::move(callback_id)));
 }
 
 void CrostiniHandler::OnPortForwardComplete(std::string callback_id,
@@ -691,7 +704,7 @@ void CrostiniHandler::HandleGetCrostiniActivePorts(
 
   ResolveJavascriptCallback(
       base::Value(callback_id),
-      crostini::CrostiniPortForwarder::GetForProfile(profile_)
+      crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)
           ->GetActivePorts());
 }
 
@@ -703,7 +716,7 @@ void CrostiniHandler::HandleGetCrostiniActiveNetworkInfo(
   std::string callback_id = args[0].GetString();
   ResolveJavascriptCallback(
       base::Value(callback_id),
-      crostini::CrostiniPortForwarder::GetForProfile(profile_)
+      crostini::CrostiniPortForwarderFactory::GetForProfile(profile_)
           ->GetActiveNetworkInfo());
 }
 
@@ -764,7 +777,8 @@ void CrostiniHandler::HandleShutdownCrostini(const base::Value::List& args) {
 void CrostiniHandler::HandleShutdownBruschetta(const base::Value::List& args) {
   CHECK_EQ(0U, args.size());
 
-  bruschetta::BruschettaService::GetForProfile(profile_)->StopRunningVms();
+  bruschetta::BruschettaServiceFactory::GetForProfile(profile_)
+      ->StopRunningVms();
 }
 
 void CrostiniHandler::HandleCreateContainer(const base::Value::List& args) {
@@ -794,7 +808,7 @@ void CrostiniHandler::HandleCreateContainer(const base::Value::List& args) {
   if (isContainerBackupFile) {
     VLOG(1) << "backup_file = " << container_file
             << "will be used to create a new container.";
-    crostini::CrostiniExportImport::GetForProfile(profile_)
+    crostini::CrostiniExportImportFactory::GetForProfile(profile_)
         ->CreateContainerFromImport(
             container_id, container_file,
             base::BindOnce(&CrostiniHandler::OnContainerCreated,
@@ -886,9 +900,8 @@ void CrostiniHandler::HandleRequestContainerInfo(
        guest_os::GetContainers(profile_, guest_os::VmType::TERMINA)) {
     base::Value::Dict container_info_value;
     container_info_value.Set(kIdKey, container_id.ToDictValue());
-    auto info =
-        guest_os::GuestOsSessionTracker::GetForProfile(profile_)->GetInfo(
-            container_id);
+    auto info = guest_os::GuestOsSessionTrackerFactory::GetForProfile(profile_)
+                    ->GetInfo(container_id);
     if (info) {
       container_info_value.Set(kIpv4Key, info->ipv4_address);
     }
@@ -957,7 +970,7 @@ void CrostiniHandler::HandleRequestSharedVmDevices(
   constexpr char kMicrophone[] = "microphone";
 
   auto* crostini_shared_devices =
-      crostini::CrostiniSharedDevices::GetForProfile(profile_);
+      crostini::CrostiniSharedDevicesFactory::GetForProfile(profile_);
 
   base::Value::List shared_vmdevices;
   for (const auto& container_id :
@@ -986,7 +999,7 @@ void CrostiniHandler::HandleIsVmDeviceShared(const base::Value::List& args) {
 
   ResolveJavascriptCallback(
       base::Value(callback_id),
-      crostini::CrostiniSharedDevices::GetForProfile(profile_)
+      crostini::CrostiniSharedDevicesFactory::GetForProfile(profile_)
           ->IsVmDeviceShared(container_id, vm_device));
 }
 
@@ -997,23 +1010,25 @@ void CrostiniHandler::HandleSetVmDeviceShared(const base::Value::List& args) {
   const std::string& vm_device = args[2].GetString();
   bool shared = args[3].GetBool();
 
-  crostini::CrostiniSharedDevices::GetForProfile(profile_)->SetVmDeviceShared(
-      container_id, vm_device, shared,
-      base::BindOnce(
-          [](base::WeakPtr<CrostiniHandler> weak_this,
-             const std::string callback_id, bool was_applied) {
-            if (weak_this) {
-              weak_this->ResolveJavascriptCallback(base::Value(callback_id),
-                                                   was_applied);
-            }
-          },
-          callback_weak_ptr_factory_.GetWeakPtr(), callback_id));
+  crostini::CrostiniSharedDevicesFactory::GetForProfile(profile_)
+      ->SetVmDeviceShared(
+          container_id, vm_device, shared,
+          base::BindOnce(
+              [](base::WeakPtr<CrostiniHandler> weak_this,
+                 const std::string callback_id, bool was_applied) {
+                if (weak_this) {
+                  weak_this->ResolveJavascriptCallback(base::Value(callback_id),
+                                                       was_applied);
+                }
+              },
+              callback_weak_ptr_factory_.GetWeakPtr(), callback_id));
 }
 
 void CrostiniHandler::HandleRequestBruschettaInstallerView(
     const base::Value::List& args) {
   AllowJavascript();
   BruschettaInstallerView::Show(Profile::FromWebUI(web_ui()),
+                                CHECK_DEREF(g_browser_process->local_state()),
                                 bruschetta::GetBruschettaAlphaId());
 }
 
